! #####################################
! # Beta-pdf model for polynomial 
! # nonlinearity
! #####################################

FUNCTION PDF_BETA_POLY( mean, var, nord )

  IMPLICIT NONE

#include "types.h"

  TREAL mean, var, PDF_BETA_POLY
  TINTEGER nord

  TREAL a,b,GAMMLN
  EXTERNAL GAMMLN

  a = mean*( mean*(C_1_R-mean)/var - C_1_R )
  a = MAX(C_1EM6_R,a)
  b = a/mean - a

!  IF ( a .LE. C_0_R ) THEN
!     PDF_BETA_POLY = C_0_R
!  ELSE
  PDF_BETA_POLY = EXP( GAMMLN(a+b)+GAMMLN(a+nord)-&
       GAMMLN(a+b+nord) - GAMMLN(a) )
!  ENDIF

999 RETURN
END FUNCTION PDF_BETA_POLY

! #####################################
! # Beta-pdf model for Arrhenius
! # nonlinearity
! #####################################

FUNCTION PDF_BETA_ARRH( mean, var, inlf, sgs_tact, &
     sgs_zst, sgs_smooth, d0, d1, c0, c1, c2 )

  IMPLICIT NONE

#include "types.h"

  TREAL mean,var,sgs_tact,sgs_zst,sgs_smooth,d0,d1,c0,c1,c2,&
       PDF_BETA_ARRH
  TINTEGER inlf

  TREAL WORK,ABSERR,EPSABS,EPSREL
  TINTEGER IWORK,KEY,LAST,LENW,LIMIT,IER,NEVAL
  DIMENSION IWORK(100),WORK(400)

  TREAL PDF_BETA_ARRH_INT, GAMMLN, p(11), a, b
  EXTERNAL PDF_BETA_ARRH_INT, GAMMLN
  TREAL A1, A2, FUN, FUN2

  a = mean*( mean*(C_1_R-mean)/var - C_1_R )
  a = MAX(C_1EM6_R,a)
  b = a/mean - a

!  IF ( a .LE. C_0_R ) THEN
!     PDF_BETA_ARRH = C_0_R
!  ELSE

! Convergence constants (check the value of the function)
  EPSABS = C_1EM5_R
  EPSREL = C_1EM5_R
  KEY = 6
  LIMIT = 100
  LENW = LIMIT*4

  p(1) = a
  p(2) = b
  p(3) = sgs_tact
  p(4) = sgs_zst
  p(5) = sgs_smooth
  p(6) = d0
  p(7) = d1
  p(8) = c0
  p(9) = c1
  p(10) = c2
  p(11) = M_REAL(inlf)

  A1 = C_0_R
  A2 = C_1_R
  IF ( inlf .EQ. 2 .OR. inlf .EQ. 4 ) THEN
     CALL QUADAG(PDF_BETA_ARRH_INT,p,A1,sgs_zst,EPSABS,&
          EPSREL,KEY,FUN,ABSERR,NEVAL,IER,LIMIT,&
          LENW,LAST,IWORK,WORK)
     CALL QUADAG(PDF_BETA_ARRH_INT,p,sgs_zst,A2,EPSABS,&
          EPSREL,KEY,FUN2,ABSERR,NEVAL,IER,LIMIT,&
          LENW,LAST,IWORK,WORK)
     FUN = FUN + FUN2
  ELSE
     CALL QUADAG(PDF_BETA_ARRH_INT,p,A1,A2,EPSABS,&
          EPSREL,KEY,FUN,ABSERR,NEVAL,IER,LIMIT,&
          LENW,LAST,IWORK,WORK)
  ENDIF
!     PDF_BETA_ARRH = FUN
! normalized
  PDF_BETA_ARRH = FUN*EXP( GAMMLN(a+b) - GAMMLN(a) - GAMMLN(b) )
!  ENDIF

999 RETURN
END FUNCTION PDF_BETA_ARRH

! ##########################################
! # Integrand used in the beta-pdf aproach
! # for the case of the Arrhenius nonlinearity
! #
! # p(1) - a for the beta-pdf
! # p(2) - b for the beta-pdf 
! # p(3) - sgs_tact
! # p(4) - sgs_zst
! # p(5) - sgs_smooth
! # p(6) - a for the arrhenius term
! # p(7) - b for the arrhenius term
! # p(8) - c0
! # p(9) - c1
! # p(10) - c2
! # p(11) - inlf
! ##########################################

FUNCTION PDF_BETA_ARRH_INT(x,p)

  IMPLICIT NONE

#include "types.h"

  TREAL x, p(*), PDF_BETA_ARRH_INT

  TINTEGER inlf
  TREAL ARRHENIUS, zero, one, beta
! TREAL BETAPDF
  EXTERNAL ARRHENIUS

  zero = C_1EM5_R
  one = C_1_R - zero

  IF ( x.LE.zero .OR. x.GE.one ) THEN
     PDF_BETA_ARRH_INT = C_0_R
  ELSE
     inlf = INT(p(11))
! beta-pdf not normalized
!     beta = BETAPDF(p(1),p(2),x)
     beta = EXP( (p(1)-C_1_R)*LOG(X) + &
          (p(2)-C_1_R)*LOG(C_1_R-X) )
     PDF_BETA_ARRH_INT = beta*ARRHENIUS(inlf,&
          x,p(3),p(4),p(5),p(6),p(7),p(8),p(9),p(10))
  ENDIF

  RETURN
END FUNCTION PDF_BETA_ARRH_INT

! #####################################
! # Beta-pdf model for generic
! # nonlinearity
! #####################################

FUNCTION PDF_BETA_GNRIC( mean, var, funpar )

  IMPLICIT NONE

#include "types.h"

  TREAL PDF_BETA_GNRIC, mean, var, funpar(*)

  TREAL WORK,ABSERR,EPSABS,EPSREL
  TINTEGER IWORK,KEY,LAST,LENW,LIMIT,IER,NEVAL
  DIMENSION IWORK(100),WORK(400)

  TREAL a, b, p(4), PDF_BETA_GNRIC_INT
  EXTERNAL PDF_BETA_GNRIC_INT
  TREAL A1, A2, FUN

  a = mean*( mean*(C_1_R-mean)/var - C_1_R )
  b = a/mean - a

  IF ( a .LE. C_0_R ) THEN
     PDF_BETA_GNRIC = C_0_R
  ELSE

! Convergence constants
     EPSABS = C_1EM5_R
     EPSREL = C_1EM5_R
     KEY = 6
     LIMIT = 100
     LENW = LIMIT*4

     p(1) = a
     p(2) = b
     p(3) = funpar(1)
     p(4) = funpar(2)

     A1 = C_0_R
     A2 = C_1_R

     CALL QUADAG(PDF_BETA_GNRIC_INT,p,A1,A2,EPSABS,&
          EPSREL,KEY,FUN,ABSERR,NEVAL,IER,LIMIT,&
          LENW,LAST,IWORK,WORK)

     PDF_BETA_GNRIC = FUN

  ENDIF

999 RETURN
END FUNCTION PDF_BETA_GNRIC

! ###########################

FUNCTION PDF_BETA_GNRIC_INT(x,p)

  IMPLICIT NONE

#include "types.h"

  TREAL x, p(*), PDF_BETA_GNRIC_INT

  TREAL BETAPDF, FUN_GNRIC, zero, one
  EXTERNAL BETAPDF, FUN_GNRIC

  zero = C_1EM5_R
  one = C_1_R - zero

  IF ( x.LE.zero .OR. x.GE.one ) THEN
     PDF_BETA_GNRIC_INT = C_0_R
  ELSE
     PDF_BETA_GNRIC_INT = BETAPDF(p(1),p(2),x)*FUN_GNRIC(x,p(3))
  ENDIF

  RETURN
END FUNCTION PDF_BETA_GNRIC_INT

! ###########################

FUNCTION FUN_GNRIC(x,p)

  IMPLICIT NONE

#include "types.h"

  TREAL x, p(*), FUN_GNRIC

!  FUN_GNRIC = EXP(-C_05_R*((x-p(1))/p(2))**4)
!  FUN_GNRIC = x*(1-x)
  FUN_GNRIC = (1-x)**4

  RETURN
END FUNCTION FUN_GNRIC

! ###########################

FUNCTION BETAPDF(A,B,X)

  IMPLICIT NONE
  
#include "types.h"

  TREAL BETAPDF, X,A,B
  
  TREAL GAMMLN
  TINTEGER IER
  
  IER=0
  IF ( X .LT. C_0_R ) IER=1
  IF ( X .GT. C_1_R ) IER=1
  IF ( IER .NE. 0 ) GO TO 999
  
  BETAPDF = EXP( GAMMLN(A+B) - GAMMLN(A) - GAMMLN(B) +&
       (A-C_1_R)*LOG(X) + (B-C_1_R)*LOG(C_1_R-X) )

999 IF ( IER .NE. 0 ) THEN
     WRITE(*,*) 'mathlib : betapdf : ier = ',IER
     STOP
  ENDIF
  
  RETURN
END FUNCTION BETAPDF
