#include "types.h"
#include "les_const.h"

SUBROUTINE SGS_ARM_U(n0, x, y, z, dx, dy, dz, &
     rho, u, v, w, barvs, c0, mu_m, mv_m, mw_m, txc, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z)

! txc size is 4

  USE DNS_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER n0
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL rho(imax,jmax,kmax)
  TREAL u(imax,jmax,kmax)
  TREAL v(imax,jmax,kmax)
  TREAL w(imax,jmax,kmax)
  TREAL barvs(imax,jmax,kmax)
  TREAL c0(*)
  TREAL mu_m(imax,jmax,kmax)
  TREAL mv_m(imax,jmax,kmax)
  TREAL mw_m(imax,jmax,kmax)
  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(*)
  TREAL wrk3d(imax,jmax,*)
  TREAL txc(imax,jmax,kmax,*)
  TREAL cf0x(*), cf0y(*), cf0z(*)

  TINTEGER ij, j, k
!  TREAL SIMPSON_NU, a0_int, a1_int, a2_int, c0_int
!  TREAL AVG_IK, SIMPSON_NU
!  TREAL EK_dns, EK_les

#define RHOK_SG(j)  wrk1d(j,1)
#define A0(j)       wrk1d(j,2)
#define A1(j)       wrk1d(j,3)
#define A2(j)       wrk1d(j,4)
#define A0_LOC(j)   wrk1d(j,5)
#define A1_LOC(j)   wrk1d(j,6)
#define A2_LOC(j)   wrk1d(j,7)

! ARM inviscid
  IF ( iles_type_tran .EQ. LES_TRAN_ARMINVS ) THEN
     DO j = 1,jmax
        c0(j) = arm_c0_inviscid
     ENDDO

! ARM rans
  ELSE IF ( iles_type_tran .EQ. LES_TRAN_ARMRANS ) THEN
! ----------------------------------------------------------------
!                             calculating average 2*<rho*K_sg>
! ----------------------------------------------------------------
! Use from A0 as wrk1d array
     CALL SGS_ARM_RANS_USG(imax, jmax, kmax, kmax_total,&
          area, rtime, x, y, z, dx, dy, dz, &
          rho, u, v, w, RHOK_SG(1), A0(1), wrk3d)

! ----------------------------------------------------------------
!                                               calculating c0
! ----------------------------------------------------------------
! MU=rho*u. Stored in mv_m
     DO k = 1,kmax
        DO ij = 1,NN
           mv_m(ij,1,k) = rho(ij,1,k)*u(ij,1,k)
        ENDDO
     ENDDO
! txc size is 4
     CALL SGS_ARM_RANS_A(imax, jmax, kmax, kmax_total, n0,&
          i1, iunifx, iunify, iunifz, i1bc, j1bc, k1bc, area,&
          dx, dy, dz, rho, barvs, &
          mv_m, RHOK_SG(1), A0(1), A1(1), A2(1), &
          cf0x, cf0y, cf0z, txc, wrk3d)
     RHOK_SG(1:jmax) = C_0_R

! MV=rho*v. Stored in mw_m
     DO k = 1,kmax
        DO ij = 1,NN
           mw_m(ij,1,k) = rho(ij,1,k)*v(ij,1,k)
        ENDDO
     ENDDO
! txc size is 4
     CALL SGS_ARM_RANS_A(imax, jmax, kmax, kmax_total, n0,&
          i1, iunifx, iunify, iunifz, i1bc, j1bc, k1bc, area,&
          dx, dy, dz, rho, barvs, &
          mw_m, RHOK_SG(1), A0_LOC(1), A1_LOC(1), A2_LOC(1), &
          cf0x, cf0y, cf0z, txc, wrk3d)
     DO j = 1,jmax
        A0(j) = A0(j) + A0_LOC(j)
        A1(j) = A1(j) + A1_LOC(j)
        A2(j) = A2(j) + A2_LOC(j)
     ENDDO

! MW=rho*w. Stored in mu_m
     DO k = 1,kmax
        DO ij = 1,NN
           mu_m(ij,1,k) = rho(ij,1,k)*w(ij,1,k)
        ENDDO
     ENDDO
! txc size is 4
     CALL SGS_ARM_RANS_A(imax, jmax, kmax, kmax_total, n0,&
          i1, iunifx, iunify, iunifz, i1bc, j1bc, k1bc, area,&
          dx, dy, dz, rho, barvs, &
          mu_m, RHOK_SG(1), A0_LOC(1), A1_LOC(1), A2_LOC(1), &
          cf0x, cf0y, cf0z, txc, wrk3d)
     DO j = 1,jmax
        A0(j) = A0(j) + A0_LOC(j)
        A1(j) = A1(j) + A1_LOC(j)
        A2(j) = A2(j) + A2_LOC(j)
     ENDDO

#ifdef _DEBUG
     CALL IO_WRITE_ASCII_FIELD('a0_rans.dat', i1, jmax, i1, A0(1))
     CALL IO_WRITE_ASCII_FIELD('a1_rans.dat', i1, jmax, i1, A1(1))
     CALL IO_WRITE_ASCII_FIELD('a2_rans.dat', i1, jmax, i1, A2(1))
#endif

     CALL SGS_ARM_RANS_C0(jmax, A0(1), A1(1), &
          A2(1), c0)
!     a0_int = SIMPSON_NU(jmax,A0(1),y)
!     a1_int = SIMPSON_NU(jmax,A1(1),y)
!     a2_int = SIMPSON_NU(jmax,A2(1),y)
!     CALL SGS_ARM_RANS_C0(i1, a0_int, a1_int, a2_int, c0_int)
!     DO j = 1,jmax
!        c0(j) = c0_int
!     ENDDO

#ifdef _DEBUG
     CALL IO_WRITE_ASCII_FIELD('c0_rans.dat', i1, jmax, i1, c0)
#endif

  ENDIF

  arm_ucoef = MAXVAL(c0(1:jmax))

#undef RHOK_SG
#undef A0
#undef A1
#undef A2
#undef A0_LOC
#undef A1_LOC
#undef A2_LOC

! ----------------------------------------------------------------
!                                                  model field
! ----------------------------------------------------------------
! saving mu_m into txc
  DO k = 1,kmax
     DO ij = 1,NN
        txc(ij,1,k,1) = mu_m(ij,1,k)
     ENDDO
  ENDDO
! MU_M: Stored in mv_m
  CALL SGS_ARM_UM(n0, dx, dy, dz, &
       cf0x, cf0y, cf0z, c0, mv_m, mu_m, wrk1d, wrk3d)
! MV_M: Stored in mw_m
  CALL SGS_ARM_UM(n0, dx, dy, dz, &
       cf0x, cf0y, cf0z, c0, mw_m, mv_m, wrk1d, wrk3d)
! MW_M: Stored in txc
  CALL SGS_ARM_UM(n0, dx, dy, dz, &
       cf0x, cf0y, cf0z, c0, txc, mw_m, wrk1d, wrk3d)

  RETURN
END SUBROUTINE SGS_ARM_U
