      SUBROUTINE SGS_ARM_RANS_USG(imax, jmax, kmax, kmax_total,&
           area, rtime, x, y, z, dx, dy, dz, &
           rho, u, v, w, rhok_sg, wrk1d, wrk3d)

      IMPLICIT NONE

#include "types.h"
#include "integers.h"

      TINTEGER imax, jmax, kmax, kmax_total
      TREAL area, rtime

      TREAL x(imax)
      TREAL y(jmax)
      TREAL z(kmax_total)
      TREAL dx(imax)
      TREAL dy(jmax)
      TREAL dz(kmax_total)

      TREAL rho(imax,jmax,kmax)
      TREAL u(imax,jmax,kmax)
      TREAL v(imax,jmax,kmax)
      TREAL w(imax,jmax,kmax)

      TREAL wrk1d(jmax,*)
      TREAL wrk3d(imax,jmax,kmax) 

      TREAL rhok_sg(jmax)

      TINTEGER k, i, j
      TREAL EK_dns, EK_les
      TREAL SIMPSON_NU, AVG_IK

#ifdef SINGLE_PREC
      EK_dns = C_1EM2_R*( rtime*0.028525e0 + 0.24539e0 )
#else
      EK_dns = C_1EM2_R*( rtime*0.028525d0 + 0.24539d0 )
#endif

#define rR(j)           wrk1d(j,1)
#define fU(j)           wrk1d(j,2)
#define fV(j)           wrk1d(j,3)
#define fW(j)           wrk1d(j,4)

      DO j = 1,jmax

         DO k = 1,kmax
            DO i = 1,imax
! U velocity
               wrk3d(i,1,k) = rho(i,j,k)*u(i,j,k)
               wrk3d(i,2,k) = wrk3d(i,1,k)*u(i,j,k)
! V velocity
               wrk3d(i,3,k) = rho(i,j,k)*v(i,j,k)
               wrk3d(i,4,k) = wrk3d(i,3,k)*v(i,j,k)
! W velocity
               wrk3d(i,5,k) = rho(i,j,k)*w(i,j,k)
               wrk3d(i,6,k) = wrk3d(i,5,k)*w(i,j,k)
            ENDDO
         ENDDO

         rR(j) = AVG_IK(imax, jmax, kmax, j, &
              rho, dx, dz, area)
         fU(j) = AVG_IK(imax, jmax, kmax, i1, &
              wrk3d, dx, dz, area)/rR(j)
         fV(j) = AVG_IK(imax, jmax, kmax, i3, &
              wrk3d, dx, dz, area)/rR(j)
         fW(j) = AVG_IK(imax, jmax, kmax, i5, &
              wrk3d, dx, dz, area)/rR(j)
         rhok_sg(j) = &
              AVG_IK(imax, jmax, kmax, i2, wrk3d, dx, dz, area) +&
              AVG_IK(imax, jmax, kmax, i4, wrk3d, dx, dz, area) +&
              AVG_IK(imax, jmax, kmax, i6, wrk3d, dx, dz, area) -&
              rR(j)*( fU(j)*fU(j)+&
              fV(j)*fV(j)+fW(j)*fW(j) )

      ENDDO

!  DO j = 1,jmax
!     rhok_sg(j)= rhok_sg(j)/rR(j)
!  ENDDO
      EK_les =  SIMPSON_NU(jmax,rhok_sg(1),y)
      DO j = 1,jmax
! Careful, EK_dns is K and you are working with 2K.
!     rhok_sg(j) = rR(j)*rhok_sg(j)*
         rhok_sg(j) = rhok_sg(j)*(C_2_R*EK_dns/EK_les-C_1_R)
      ENDDO
      
#ifdef _DEBUG
      DO j = 1,jmax
         rR(j) = rhok_sg(j)/rR(j)
      ENDDO
      CALL IO_WRITE_ASCII_FIELD('KSG_rans.dat', i1, jmax, i1, rR(1))
#endif

#undef rR
#undef fU
#undef fV
#undef fW

      RETURN
      END
