      SUBROUTINE SGS_ARM_TABLE_K(ng, vg, gmin, c0)

      IMPLICIT NONE

#include "types.h"
      
      TINTEGER ng
      TREAL c0(ng)
      TREAL vg(ng)

      TREAL gamma, gmin

      TREAL params(9)
      TREAL eps, cl, cl0, rg
      TREAL A0, A1, A2
      TINTEGER niter, nmax, ig
      
      TREAL A,B,EPSABS,IER

      TREAL ESP_K_CL_ROOT
      EXTERNAL ESP_K_CL_ROOT
      TREAL ESP_K_A0, ESP_K_A1, ESP_K_A2 
      EXTERNAL ESP_K_A0, ESP_K_A1, ESP_K_A2

! Spectrum low wave number constant (never modified below)
      params(1) = C_2_R

! Spectrum Slope (should be 5/3, since this is for high Ret)
      params(7) = C_5_R/C_3_R

! Constant (Kolmogorov or Obukhov-Corrsin)
#ifdef SINGLE_PREC
      params(6) = 0.67e0
#else
      params(6) = 0.67d0
#endif

! Integration and Iteration constants
      nmax = 128
      eps = C_1EM3_R
      EPSABS = C_1EM8_R

! integration limit
      A = C_0_R
      B = C_100_R*(params(6)/C_1EM3_R/(params(7)-C_1_R))**&
           (C_1_R/(params(7)-C_1_R))
 
! Find cl function so k is exact.
      cl0 = C_1_R
      CALL NEWTON(ESP_K_CL_ROOT, params, cl0, nmax, eps, niter, cl)
#ifdef _DEBUG
      WRITE(*,*) niter, cl
#endif
      params(3) = cl

! Minimum gamma is eta/2 L = (Re**-3/4)/2
      gamma = gmin

! Gamma factor is set so last gamma = 1/2 (Delta_f == L)
      rg = EXP(LOG(C_05_R/gmin)/M_REAL(ng-1))
      DO ig = 1,ng

! Set integral scale array to log
         vg(ig) = LOG(gamma)
         
! Gamma parameter of model
         params(9) = gamma
         
         B = 1/gamma
         CALL INTEGSPC(ESP_K_A0,params,A,B,EPSABS,A0,IER)
            IF (IER.NE.0) THEN
               WRITE(*,*) 'Error in integration a0, ierr = ',IER
            ENDIF
         CALL INTEGSPC(ESP_K_A1,params,A,B,EPSABS,A1,IER)
            IF (IER.NE.0) THEN
               WRITE(*,*) 'Error in integration a1, ierr = ',IER
            ENDIF
         CALL INTEGSPC(ESP_K_A2,params,A,B,EPSABS,A2,IER)
            IF (IER.NE.0) THEN
               WRITE(*,*) 'Error in integration a2, ierr = ',IER
            ENDIF

! Calculate c0 from root of quadratic equation
         c0(ig) = (-A1+SQRT(A1**2-C_4_R*A0*A2))*C_05_R/A2

! Increase gamma
         gamma = gamma*rg
      ENDDO

      RETURN
      END
      
