SUBROUTINE CGBC2NU(imax, jmax, kmax, kmax_total, i1bc, j1bc, k1bc, &
     dx, dy, dz, d2xdi2, d2ydj2, d2zdk2, ci, cj, ck)
  
#include "types.h"

#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER imax
  TINTEGER jmax
  TINTEGER kmax
  TINTEGER kmax_total
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TREAL d2xdi2(imax)
  TREAL d2ydj2(jmax)
  TREAL d2zdk2(kmax_total)
  TINTEGER i1bc
  TINTEGER j1bc
  TINTEGER k1bc
  TINTEGER i
  TINTEGER j
  TINTEGER k, kp
  TREAL diag
  TREAL ci(imax,jmax,kmax,2)
  TREAL cj(imax,jmax,kmax,2)
  TREAL ck(imax,jmax,kmax,2)
  TREAL ap, bp, cp
  TREAL am, bm, cm
  TREAL ac, bc, cc
  TREAL didx, d2idx2, didx2
  TREAL djdy, d2jdy2, djdy2
  TREAL dkdz, d2kdz2, dkdz2

!#ifdef _DEBUG
!      PRINT *, 'I       DX          D2XDI2'
!      DO i=1, imax
!         PRINT *, i, dx(i), d2xdi2(i)
!      ENDDO
!
!      PRINT *, 'J       DY          D2YDJ2'
!      DO j=1, jmax
!         PRINT *, j, dy(j), d2ydj2(j)
!      ENDDO
!
!      PRINT *, 'K       DZ          D2ZDK2'
!      DO k=1, kmax_total
!         PRINT *, k, dz(k), d2zdk2(k)
!      ENDDO
!#endif

  DO k=1, kmax
     DO j=1, jmax
        DO i=1, imax

           diag = C_0_R

           didx = C_1_R/dx(i)
           d2idx2 = - d2xdi2(i)*didx**3

           didx2 = didx**2
           ap = didx2+d2idx2*C_05_R
           ac = C_2_R*didx2
           am = didx2-d2idx2*C_05_R

           diag = diag + ac

           djdy = C_1_R/dy(j)
           d2jdy2 = - d2ydj2(j)*djdy**3

           djdy2 = djdy**2
           bp = djdy2+d2jdy2*C_05_R
           bc = C_2_R*djdy2
           bm = djdy2-d2jdy2*C_05_R

           diag = diag + bc

           IF ( kmax_total .GT. 1 ) THEN

#ifdef PARALLEL
              kp = ims_offset_k + k
#else 
              kp = k
#endif

              dkdz = C_1_R/dz(kp)
              d2kdz2 = - d2zdk2(kp)*dkdz**3

              dkdz2 = dkdz**2
              cp = dkdz2+d2kdz2*C_05_R
              cc = C_2_R*dkdz2
              cm = dkdz2-d2kdz2*C_05_R

              diag = diag + cc 
           ELSE
              cp = C_0_R
              cc = C_0_R
              cm = C_0_R
           ENDIF

           ap = - ap/diag
           am = - am/diag

           IF ( i1bc .NE. 0 ) THEN
              IF ( i .EQ. 1 ) THEN
                 am = C_0_R
              ELSE IF ( i .EQ. imax ) THEN
                 ap = C_0_R
              ENDIF
           ENDIF

           bp = - bp/diag
           bm = - bm/diag

           IF ( j1bc .NE. 0 ) THEN
              IF ( j .EQ. 1 ) THEN
                 bm = C_0_R
              ELSE IF ( j .EQ. jmax ) THEN
                 bp = C_0_R
              ENDIF
           ENDIF

           IF ( kmax_total .GT. 1 ) THEN

              cp = - cp/diag
              cm = - cm/diag

              IF ( k1bc .NE. 0 ) THEN
                 IF ( kp .EQ. 1 ) THEN
                    cm = C_0_R
                 ELSE IF ( kp .EQ. kmax_total ) THEN
                    cp = C_0_R
                 ENDIF
              ENDIF

           ENDIF

           ci(i,j,k,1) = am
           ci(i,j,k,2) = ap
           cj(i,j,k,1) = bm
           cj(i,j,k,2) = bp
           ck(i,j,k,1) = cm
           ck(i,j,k,2) = cp
        ENDDO
     ENDDO
  ENDDO

  RETURN
END SUBROUTINE CGBC2NU
