SUBROUTINE CGBC4(iunif, imax, jmax, kmax, kmax_total, i1bc, j1bc, k1bc,&
     scalex, scaley, scalez, dx, dy, dz, ipos, jpos, kpos, ci, cj, ck, wrk)
  
#include "types.h"
#include "dns_error.h"

#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER iunif
  TINTEGER imax
  TINTEGER jmax
  TINTEGER kmax
  TINTEGER kmax_total
  TREAL scalex
  TREAL scaley
  TREAL scalez
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TINTEGER i1bc
  TINTEGER j1bc
  TINTEGER k1bc
  TINTEGER i
  TINTEGER j
  TINTEGER k, kp
  TINTEGER ipos(imax,4)
  TINTEGER jpos(jmax,4)
  TINTEGER kpos(kmax,4)
  TREAL ci(imax,4)
  TREAL cj(jmax,4)
  TREAL ck(kmax,4)
  TREAL diag, diag2
  TREAL chi, chi2
  TREAL mu, mu2
  TREAL eta, eta2
  TREAL wrk(*)

#ifdef PARALLEL
!  IF ( k1bc .NE. 0 ) THEN
!     CALL IO_WRITE_ASCII(efile, 'Poisson solver is periodic in k')
!     CALL DNS_STOP(DNS_ERROR_POISSON)
!  ENDIF
#endif

  chi = (dy(jmax/2)/dx(imax/2))**2

  IF ( kmax_total .GT. 1 ) THEN
     eta = (dy(jmax/2)/dz(kmax_total/2))**2
  ELSE
     eta = C_0_R
  ENDIF

  diag = C_5_R*(C_1_R+chi+eta)/C_2_R
  diag2 = C_2_R*(C_1_R+chi+eta)
  chi = chi / diag
  mu  = C_1_R / diag
  eta = eta / diag

  chi2 = chi / diag2
  mu2  = C_1_R / diag2
  eta2 = eta / diag2

  DO i=1, imax

     ci(i,1) = chi/C_12_R
     ci(i,2) = - C_4_R*chi/C_3_R
     ci(i,3) = - C_4_R*chi/C_3_R
     ci(i,4) = chi/C_12_R
     IF ( i1bc .NE. 0 ) THEN
        IF ( i .EQ. 1 ) THEN
           ci(i,1) = C_0_R
           ci(i,2) = C_0_R
           ci(i,3) = -chi2
           ci(i,4) = C_0_R
        ELSE IF ( i .EQ. 2 ) THEN
           ci(i,1) = C_0_R
        ELSE IF ( i .EQ. imax-1 ) THEN
           ci(i,4) = C_0_R
        ELSE IF ( i .EQ. imax ) THEN
           ci(i,1) = C_0_R
           ci(i,2) = -chi2
           ci(i,3) = C_0_R
           ci(i,4) = C_0_R
        ENDIF
     ENDIF
  ENDDO

  DO j=1, jmax

     cj(j,1) = mu/C_12_R
     cj(j,2) = - C_4_R*mu/C_3_R
     cj(j,3) = - C_4_R*mu/C_3_R
     cj(j,4) = mu/C_12_R
     IF ( j1bc .NE. 0 ) THEN
        IF ( j .EQ. 1 ) THEN
           cj(j,1) = C_0_R
           cj(j,2) = C_0_R
           cj(j,3) = -mu2
           cj(j,4) = C_0_R
        ELSE IF ( j .EQ. 2 ) THEN
           cj(j,1) = C_0_R
        ELSE IF ( j .EQ. jmax-1 ) THEN
           cj(j,4) = C_0_R
        ELSE IF ( j .EQ. jmax ) THEN
           cj(j,1) = C_0_R
           cj(j,2) = -mu2
           cj(j,3) = C_0_R
           cj(j,4) = C_0_R
        ENDIF
     ENDIF
  ENDDO

  IF ( kmax_total .EQ. 1 ) THEN

     ck(1,1) = C_0_R
     ck(1,2) = C_0_R
     ck(1,3) = C_0_R
     ck(1,4) = C_0_R
  ELSE
     DO k=1, kmax

        ck(k,1) = eta/C_12_R
        ck(k,2) = - C_4_R*eta/C_3_R
        ck(k,3) = - C_4_R*eta/C_3_R
        ck(k,4) = eta/C_12_R
        IF ( k1bc .EQ. 1 ) THEN
#ifdef PARALLEL
           kp = ims_offset_k + k
#else 
           kp = k
#endif
           IF ( kp .NE. 0 ) THEN
              ck(k,1) = C_0_R
              ck(k,2) = C_0_R
              ck(k,3) = -eta2
              ck(k,4) = C_0_R
           ELSE IF ( kp .EQ. 2 ) THEN
              ck(k,1) = C_0_R
           ELSE IF ( kp .EQ. kmax_total-1 ) THEN
              ck(k,4) = C_0_R
           ELSE IF ( kp .EQ. kmax_total ) THEN
              ck(k,1) = C_0_R
              ck(k,2) = -eta2
              ck(k,3) = C_0_R
              ck(k,4) = C_0_R
           ENDIF
        ENDIF
     ENDDO
  ENDIF

  CALL CGBC4POS(imax, jmax, kmax, kmax_total, &
       ipos, jpos, kpos)

!#ifdef _DEBUG
!      DO i=1, imax
!         WRITE(*,*) i, (ci(i,j), j=1,4)
!      ENDDO
!      DO i=1, jmax
!         WRITE(*,*) i, (cj(i,j), j=1,4)
!      ENDDO
!      DO i=1, kmax
!         WRITE(*,*) i, (ck(i,j), j=1,4)
!      ENDDO
!#endif
  RETURN
END SUBROUTINE CGBC4
