SUBROUTINE CGBC4RSC(imax, jmax, kmax, kmax_total, i1bc, j1bc, k1bc, dx, dy, dz, rhs)

#include "types.h"
#include "dns_error.h"

#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER imax
  TINTEGER jmax
  TINTEGER kmax
  TINTEGER kmax_total
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TREAL rhs(imax,jmax,kmax)
  TINTEGER i1bc
  TINTEGER j1bc
  TINTEGER k1bc
  TINTEGER i
  TINTEGER j
  TINTEGER k
  TINTEGER i1, imx, j1, jmx, k1, kmx, kp1, kpmx
  TREAL diag, diag2
  TREAL chi
  TREAL eta
  TREAL rsc, rsc2

  chi = (dy(jmax/2)/dx(imax/2))**2

  IF ( kmax_total .GT. 1 ) THEN
     eta = (dy(jmax/2)/dz(kmax_total/2))**2
  ELSE
     eta = C_0_R
  ENDIF

  diag = C_5_R*(C_1_R+chi+eta)/C_2_R
  rsc = (dy(jmax/2)**2) / diag

  diag2 = C_2_R*(C_1_R+chi+eta)
  rsc2 = (dy(jmax/2)**2) / diag2

  IF ( i1bc .NE. 0 ) THEN
     i1 = 2
     imx = imax-1
  ELSE
     i1 = 1
     imx = imax
  ENDIF

  IF ( j1bc .NE. 0 ) THEN
     j1 = 2
     jmx = jmax-1
  ELSE
     j1 = 1
     jmx = jmax
  ENDIF

#ifdef PARALLEL
  kp1 = ims_offset_k + 1
  kpmx = ims_offset_k + kmax
#else 
  kp1 = 1
  kpmx = kmax
#endif

  IF ( kp1 .EQ. 1 .AND. k1bc .NE. 0 ) THEN
     k1 = 2
  ELSE
     k1 = 1
  ENDIF

  IF ( kpmx .EQ. kmax_total .AND. k1bc .NE. 0 ) THEN
     kmx = kmax-1
  ELSE
     kmx = kmax
  ENDIF

! Core of the domain
  DO k=k1, kmx
     DO j=j1, jmx
        DO i=i1, imx
           rhs(i,j,k) = rhs(i,j,k)*rsc
        ENDDO
     ENDDO
  ENDDO

! I Boundaries
  IF ( i1 .NE. 1 ) THEN
     DO k=1, kmax
        DO j=1, jmax
           rhs(1,j,k) = rhs(1,j,k)*rsc2
        ENDDO
     ENDDO
  ENDIF

  IF ( imx .NE. imax ) THEN
     DO k=1, kmax
        DO j=1, jmax
           rhs(imax,j,k) = rhs(imax,j,k)*rsc2
        ENDDO
     ENDDO
  ENDIF

! J Boundaries
  IF ( j1 .NE. 1 ) THEN
     DO k=1, kmax
        DO i=1, imax
           rhs(i,1,k) = rhs(i,1,k)*rsc2
        ENDDO
     ENDDO
  ENDIF

  IF ( jmx .NE. jmax ) THEN
     DO k=1, kmax
        DO i=1, imax
           rhs(i,jmax,k) = rhs(i,jmax,k)*rsc2
        ENDDO
     ENDDO
  ENDIF

! K Boundaries
  IF ( k1 .NE. 1 ) THEN
     DO j=1, jmax
        DO i=1, imax
           rhs(i,j,1) = rhs(i,j,1)*rsc2
        ENDDO
     ENDDO
  ENDIF

  IF ( kmx .NE. kmax ) THEN
     DO j=1, jmax
        DO i=1, imax
           rhs(i,j,kmax) = rhs(i,j,kmax)*rsc2
        ENDDO
     ENDDO
  ENDIF

  RETURN
END SUBROUTINE CGBC4RSC
