      SUBROUTINE CGMATRIX2NU(imax, jmax, kmax, kmax_total, &
           p, q, ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

#include "types.h"

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER kmax_total
      TINTEGER i
      TINTEGER j
      TINTEGER k
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,2)
      TINTEGER jpos(jmax,2)
      TINTEGER kpos(kmax,2)
      TREAL ci(imax,jmax,kmax,2)
      TREAL cj(imax,jmax,kmax,2)
      TREAL ck(imax,jmax,kmax,2)
      TREAL pboundary(imax,jmax,2)
      

! ##################################
! #  Compute Matrix Product        #
! ##################################

! #########################
! #     Internal Volume   #
! #########################

      IF ( kmax_total .EQ. 1 ) THEN
         k = 1
         DO j=1, jmax
            DO i=1, imax
               q(i,j,1) = p(i,j,1) &
                    + ci(i,j,k,2)*p(ipos(i,2),j,1)&
                    + ci(i,j,k,1)*p(ipos(i,1),j,1)&
                    + cj(i,j,k,2)*p(i,jpos(j,2),1)&
                    + cj(i,j,k,1)*p(i,jpos(j,1),1)
            ENDDO
         ENDDO
         
      ELSE
         
#ifdef PARALLEL
         IF ( kmax .EQ. 1 ) THEN
            k=1
            DO j=1, jmax
               DO i=1, imax
                  q(i,j,k) = p(i,j,k) &
                       + ci(i,j,k,2)*p(ipos(i,2),j,k)&
                       + ci(i,j,k,1)*p(ipos(i,1),j,k)&
                       + cj(i,j,k,2)*p(i,jpos(j,2),k)&
                       + cj(i,j,k,1)*p(i,jpos(j,1),k)&
                       + ck(i,j,k,2)*pboundary(i,j,2)&
                       + ck(i,j,k,1)*pboundary(i,j,1)
               ENDDO
            ENDDO
         ELSE
            IF ( kmax .GT. 2 ) THEN
               DO k=2, kmax-1
                  DO j=1, jmax
                     DO i=1, imax
                        q(i,j,k) = p(i,j,k) &
                             + ci(i,j,k,2)*p(ipos(i,2),j,k)&
                             + ci(i,j,k,1)*p(ipos(i,1),j,k)&
                             + cj(i,j,k,2)*p(i,jpos(j,2),k)&
                             + cj(i,j,k,1)*p(i,jpos(j,1),k)&
                             + ck(i,j,k,2)*p(i,j,kpos(k,2))&
                             + ck(i,j,k,1)*p(i,j,kpos(k,1))
                     ENDDO
                  ENDDO
               ENDDO
            ENDIF
            
            k=1
            DO j=1, jmax
               DO i=1, imax
                  q(i,j,k) = p(i,j,k) &
                       + ci(i,j,k,2)*p(ipos(i,2),j,k)&
                       + ci(i,j,k,1)*p(ipos(i,1),j,k)&
                       + cj(i,j,k,2)*p(i,jpos(j,2),k)&
                       + cj(i,j,k,1)*p(i,jpos(j,1),k)&
                       + ck(i,j,k,2)*p(i,j,kpos(k,2))&
                       + ck(i,j,k,1)*pboundary(i,j,1)
               ENDDO
            ENDDO
            
            k = kmax
            DO j=1, jmax
               DO i=1, imax
                  q(i,j,k) = p(i,j,k) &
                       + ci(i,j,k,2)*p(ipos(i,2),j,k)&
                       + ci(i,j,k,1)*p(ipos(i,1),j,k)&
                       + cj(i,j,k,2)*p(i,jpos(j,2),k)&
                       + cj(i,j,k,1)*p(i,jpos(j,1),k)&
                       + ck(i,j,k,2)*pboundary(i,j,2)&
                       + ck(i,j,k,1)*p(i,j,kpos(k,1))
               ENDDO
            ENDDO
         ENDIF
#else
         DO k=1, kmax
            DO j=1, jmax
               DO i=1, imax
                  q(i,j,k) = p(i,j,k) &
                       + ci(i,j,k,2)*p(ipos(i,2),j,k)&
                       + ci(i,j,k,1)*p(ipos(i,1),j,k)&
                       + cj(i,j,k,2)*p(i,jpos(j,2),k)&
                       + cj(i,j,k,1)*p(i,jpos(j,1),k)&
                       + ck(i,j,k,2)*p(i,j,kpos(k,2))&
                       + ck(i,j,k,1)*p(i,j,kpos(k,1))
               ENDDO
            ENDDO
         ENDDO
#endif

      ENDIF

#ifdef _DEBUG
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-pb.dat', &
           imax, jmax, 2, pboundary)
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-p.dat', &
           imax, jmax, kmax, p)
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-q.dat', &
           imax, jmax, kmax, q)
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-ci1.dat', &
           imax, jmax, kmax, ci)
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-cj1.dat', &
           imax, jmax, kmax, cj)
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-ck1.dat', &
           imax, jmax, kmax, ck)
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-ci2.dat', &
           imax, jmax, kmax, ci(1,1,1,2))
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-cj2.dat', &
           imax, jmax, kmax, cj(1,1,1,2))
      CALL IO_WRITE_ASCII_FIELD('cgmatrix2nu-d0-ck2.dat', &
           imax, jmax, kmax, ck(1,1,1,2))
!  STOP
#endif

! #####################################
! # End Calculation of Matrix Product #
! #####################################

      RETURN
      END
