SUBROUTINE CHEM_BILGER1997(iop, rho, T, GM, z1, zh1, h4, tmp)
  
#include "types.h"
#include "dns_error.h"
  
  USE DNS_GLOBAL
  USE CHEM_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER iop
  TREAL, DIMENSION(imax,jmax,kmax)   :: rho, T, GM, h4
  TREAL, DIMENSION(imax,jmax,kmax,*) :: z1, zh1, tmp

  TINTEGER ij, k
  TINTEGER is, icp, im, ik
  TREAL cs(6), csw(6)
  TREAL dtmp, lnT
  TREAL c_ch4, c_o2, c_h2o, c_co2, c_i, c_n2, c_m, c_r
  TREAL lambda
  TREAL yomega(5), omegaT
  TREAL omegaI, omegaII
  TREAL kr(REAC_MAX), Keq
  TREAL TMAX, DHREF, CPN2
  TINTEGER is_fuel, is_inrt

#define k1f   kr(1)
#define k3f   kr(2)
!#define Ker   kr(3)
#ifdef SINGLE_PREC
  Keq = 4.0e+5
#else
  Keq = 4.0d+5
#endif
  Keq = Keq*(1.0e5/(TREF*RGAS)/1.0e3)

!  TMAX = C_2_R
  TMAX = C_05_R
  TGFM = C_0_R
  IF ( damkohler(inb_scal) .EQ. C_0_R ) RETURN

  is_fuel = 1
  is_inrt = NSP
  CPN2 = C_0_R
  DO icp=NCP_CHEMKIN, 1, -1
     CPN2 = CPN2 + THERMO_AI(icp,2,is_inrt)
  ENDDO
  DHREF = CHW_CE*CPN2*WGHT(is_fuel)*(NUP(is_fuel)-NUPP(is_fuel))/(WREF*YFUEL*XIST)

! Normalized molecular weight
  DO is=1, NSP
     csw(is) = WGHT(is)/WREF
  ENDDO

  cs(1) =-csw(1)
  cs(2) =-csw(2)
  cs(3) = csw(3)
  cs(4) = csw(4)
  cs(5) = csw(5)

! Exponential factor for radical concentration
  lambda = C_05_R*SQRT(C_15_R/C_4_R)

  DO k=1, kmax
     DO ij=1, imax*jmax

        IF ( T(ij,1,k) .GT. TMAX ) THEN

           lnT = LOG(T(ij,1,k))

           DO ik=1,NMECH
              kr(ik) = damkohler(inb_scal)*mech_pre(ik)*&
                   EXP(-mech_tac(ik)/T(ij,1,k)&
                   +mech_beta(ik)*lnT)
           ENDDO

!       Check all mass fractions are positive
           DO is=1, NSP-1
              z1(ij,1,k,is) = MAX(z1(ij,1,k,is),C_0_R)
           ENDDO

!       Concentrations of intervening species
           c_ch4 = rho(ij,1,k)*z1(ij,1,k,1)/csw(1)
           c_o2  = rho(ij,1,k)*z1(ij,1,k,2)/csw(2)
           c_h2o = rho(ij,1,k)*z1(ij,1,k,3)/csw(3)
           c_co2 = rho(ij,1,k)*z1(ij,1,k,4)/csw(4)
           c_i   = rho(ij,1,k)*z1(ij,1,k,5)/csw(5)

!       Radical concentration (H, WGHT=1kg/kmol)
           c_r = Keq*c_i*SQRT(c_i*c_o2)*&
                EXP(-lambda*c_ch4/c_o2)

           dtmp = C_1_R
           DO is=1,NSP-1
              dtmp = dtmp - z1(ij,1,k,is)
           ENDDO
           dtmp = dtmp - c_r/rho(ij,1,k)*(1/WREF)
           c_n2  = rho(ij,1,k)*dtmp/csw(NSP)

!       Third-body
!        c_m = c_ch4 + c_o2 + c_h2o + c_co2 +c_i + c_r + c_n2
           c_m = C_1_R

!       Compute Reaction Rates Here            
           omegaI  = k1f*c_ch4*c_r
           omegaII = k3f*c_o2*c_r*c_m
           IF ( iop .EQ. 1 ) THEN
              tmp(ij,1,k,1) = c_r/rho(ij,1,k)*(1/WREF)
              tmp(ij,1,k,2) = omegaI
              tmp(ij,1,k,3) = omegaII
           ENDIF

           yomega(1) = cs(1)*omegaI
           yomega(2) = cs(2)*(omegaI+omegaII)
           yomega(3) = cs(3)*(omegaI+omegaII)
           yomega(4) = cs(4)*omegaII
           yomega(5) = cs(5)*(omegaI-omegaII)

!       Reaction rate is zero if zero mass fraction of this species
!       and negative reaction rate. Not possible to consume where 
!       there is no reactant.
           DO is=1, NSP-1
              IF ( z1(ij,1,k,is) .LE. C_0_R .AND. &
                   yomega(is) .LT. C_0_R ) THEN
                 yomega(is) = C_0_R
              ENDIF
           ENDDO

!       temperature equation
           omegaT = (0.4*omegaI+0.6*omegaII)*DHREF

           DO is=1, NSP-1
              zh1(ij,1,k,is) = zh1(ij,1,k,is) + yomega(is)
           ENDDO
           h4(ij,1,k) = h4(ij,1,k) + omegaT
           TGFM = MAX(TGFM, ABS(omegaT))

        ENDIF

     ENDDO
  ENDDO

#ifdef PARALLEL      
  CALL MPI_ALLREDUCE(TGFM, dtmp, 1, MPI_REAL8, MPI_MAX, MPI_COMM_WORLD, ims_err)
  TGFM = dtmp
#endif

  RETURN
END SUBROUTINE CHEM_BILGER1997
