SUBROUTINE CHEM_BSZELDOVICH(rho, T, GM, z1, zh1, h4)
  
#include "types.h"
#include "dns_error.h"
  
  USE DNS_GLOBAL
  USE CHEM_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL rho(imax,jmax,kmax)
  TREAL T(imax,jmax,kmax)
  TREAL GM(imax,jmax,kmax)
  TREAL z1(imax,jmax,kmax,*)
  TREAL zh1(imax,jmax,kmax,*)
  TREAL h4(imax,jmax,kmax)

  TINTEGER ij, k
  TINTEGER is, ik
  TREAL cs
  TREAL lnT
  TREAL c_n2, c_o2
  TREAL yomega, csw(8)
  TREAL kr(REAC_MAX)
#ifdef PARALLEL
  TREAL dtmp
#endif

#define k1    kr(1)

  IF ( NMECH .NE. 1 ) THEN
     CALL IO_WRITE_ASCII(efile, 'Error: Wrong number of reactions for Zeldovich mechanism')
     CALL DNS_STOP(DNS_ERROR_NBREAC)
  ENDIF

! Normalized molecular weight
  DO is=1, NSP
     csw(is) = WGHT(is)/WREF
  ENDDO

! DEFINE MOLECULAR WEIGHT of NO
  cs = WGHT(NSP+1)*damkohler(inb_scal)/WREF

  TGFM = C_0_R
  IF ( damkohler(inb_scal) .EQ. C_0_R ) RETURN

  DO k=1, kmax
     DO ij=1, imax*jmax

        lnT = LOG(T(ij,1,k))
!       Compute Reaction rate constant
        DO ik=1,NMECH
           kr(ik) = mech_pre(ik)*&
                EXP(-mech_tac(ik)/T(ij,1,k)&
                +mech_beta(ik)*lnT)
        ENDDO

!       Check all mass fractions are positive
        z1(ij,1,k,1) = MAX(z1(ij,1,k,1),C_0_R)

!       Compute Mass fractions of BS species
#define MACRO_ZINPUT z1(ij,1,k,inb_scal)
#include "dns_chem_mass.h"

!       Concentrations of intervening species
        c_o2 = rho(ij,1,k)*YMASS(2)/csw(2)
        c_n2 = rho(ij,1,k)*YMASS(NSP)/csw(NSP)

!       Compute Reaction Rates Here
        yomega = cs*k1*SQRT(c_o2)*c_n2

        zh1(ij,1,k,1) = zh1(ij,1,k,1) + yomega
!        IF ( k1 .GT. C_0_R ) THEN
!           PRINT *, k1, T(ij,1,k)
!        ENDIF

        TGFM = MAX(TGFM, ABS(yomega))
     ENDDO
  ENDDO

! Probably need to control the time step for the reaction
! with TGFM. See courant.f

! Distribute TGFM across processors
#ifdef PARALLEL

  CALL MPI_ALLREDUCE(TGFM, dtmp, 1, MPI_REAL8, MPI_MAX, &
       MPI_COMM_WORLD, ims_err)
  TGFM = dtmp

#endif

  RETURN
END SUBROUTINE CHEM_BSZELDOVICH
