SUBROUTINE CHEM_INIT_PREMIXED(im_o2, im_n2_f, im_n2_o, im_ch4, &
     is_fuel, is_ox, is_inrt, is_prd, DHREF)
  
#include "types.h"
#include "dns_const.h"
#include "dns_error.h"

  USE DNS_GLOBAL
  USE CHEM_GLOBAL

  IMPLICIT NONE

  TINTEGER icp,is,im
  TREAL DHREF
  CHARACTER*46 str
  TINTEGER im_o2, im_n2_f, im_n2_o, im_ch4
  TINTEGER is_fuel, is_ox, is_inrt, is_prd(*)

! For Methane
#define is_h2o  is_prd(1)
#define is_co2  is_prd(2)
! For Single Step
#define is_prod is_prd(1)

  TREAL YTOT, Xch4_u, Xo2_u, Xco2, Xh2o, Xo2_b, Xch4_b
  TREAL XTOT, YCH4_b, YO2_b, YCO2, YH2O, YN2_b
  TREAL WUNB, WBUR, HUNB, TUNB
  TREAL a, b, z, z_error, rho1, rho2, u1, u2
  TINTEGER il, ilmax
  TREAL F1, F2, J11, J12, J22, J21, DET, DLT_T, DLT_Z

  IF ( imech_type .EQ. CHEM_TYPE_UNIDECOMP ) THEN
     CHW_CE = DHREF
     WUNB = C_1_R
     WBUR = C_1_R
  ELSE IF ( imech_type .EQ. CHEM_TYPE_ONESTEP ) THEN
     CALL IO_WRITE_ASCII(efile, 'Premixed mode is not implemented for '&
          //'other chemistry than Methane-Air')
     CALL DNS_STOP(DNS_ERROR_NOTIMPL)
  ELSE
     YTOT = YFUEL/WGHT(is_fuel) + YOX/WGHT(is_ox) &
          + (C_1_R-YFUEL-YOX)/WGHT(is_inrt)
     Xch4_u = YFUEL/(WGHT(is_fuel)*YTOT)
     Xo2_u = YOX/(WGHT(is_ox)*YTOT)

     Xco2 = Xch4_u
     Xo2_b = Xo2_u - C_2_R*Xco2
     Xch4_b = C_0_R
     IF ( Xo2_b .LT. C_0_R ) THEN
        Xco2 = Xo2_u*C_05_R
        Xch4_b = Xch4_u-Xco2
        Xo2_b = C_0_R
     ENDIF
     Xh2o = 2*Xco2

     XTOT = Xch4_b*WGHT(is_fuel) + Xo2_b*WGHT(is_ox) &
          + (C_1_R-Xch4_b-Xo2_b)*WGHT(is_inrt)

     YCH4_b = Xch4_b*WGHT(is_fuel)/XTOT
     YO2_b = Xo2_b*WGHT(is_ox)/XTOT
     YCO2 = Xco2*WGHT(is_co2)/XTOT
     YH2O = Xh2o*WGHT(is_h2o)/XTOT
     YN2_b = C_1_R-YH2O-YCO2-YCH4_b-YO2_b

     WUNB = C_1_R/(WREF*(YFUEL/WGHT(is_fuel)&
          +YOX/WGHT(is_ox)&
          +(C_1_R-YOX-YFUEL)/WGHT(is_inrt)))
     WBUR = C_1_R/(WREF*(YCO2/WGHT(is_co2)&
          +YH2O/WGHT(is_h2o)+YCH4_b/WGHT(is_fuel)&
          +YO2_b/WGHT(is_ox)&
          +YN2_b/WGHT(is_inrt)))

     CALL IO_WRITE_ASCII(lfile, 'Unburned composition')
     WRITE(str,'(A11,2(1X,E12.5E3))') 'Fuel     = ', &
          YFUEL, Xch4_u
     CALL IO_WRITE_ASCII(lfile, str)
     WRITE(str,'(A11,2(1X,E12.5E3))') 'Oxidizer = ', &
          YOX, Xo2_u
     CALL IO_WRITE_ASCII(lfile, str)
     CALL IO_WRITE_ASCII(lfile, 'Burned composition')
     WRITE(str,'(A11,2(1X,E12.5E3))') 'Fuel     = ', &
          YCH4_b, Xch4_b
     CALL IO_WRITE_ASCII(lfile, str)
     WRITE(str,'(A11,2(1X,E12.5E3))') 'Oxidizer = ', &
          YO2_b, Xo2_b
     CALL IO_WRITE_ASCII(lfile, str)
     WRITE(str,'(A11,2(1X,E12.5E3))') 'H2O      = ', &
          YH2O, Xh2o
     CALL IO_WRITE_ASCII(lfile, str)
     WRITE(str,'(A11,2(1X,E12.5E3))') 'CO2      = ', &
          YCO2, Xco2
     CALL IO_WRITE_ASCII(lfile, str)
  ENDIF

! Assumed two stream system with zeta=0 for oxidizer
! and zeta=1 for fuel.

  rho1 = mean_rho
  u1 = mean_u

  TUNB = MRATIO*p_init*WUNB/rho1
  a = MRATIO*u1**2
  b = (gama0-C_1_R)*a/(C_2_R*gama0)

  IF ( imech_type .EQ. CHEM_TYPE_UNIDECOMP ) THEN
     YMASS(1) = C_1_R
     YMASS(2) = C_0_R
  ELSE
     DO is=1, NSP
        YMASS(is) = C_0_R
     ENDDO
     YMASS(is_fuel) = YFUEL
     YMASS(is_ox) = YOX
     YMASS(is_inrt) = C_1_R - YFUEL - YOX
  ENDIF

#define MACRO_TINPUT TUNB
#include "dns_chem_enth.h"
#undef MACRO_TINPUT

  HUNB = CH_H

  z = C_1_R
  TFLAME = CHW_CE + C_1_R
! Composition of burned stream
  IF ( imech_type .EQ. CHEM_TYPE_UNIDECOMP ) THEN
     YMASS(1) = C_0_R
     YMASS(2) = C_1_R
  ELSE
     DO is=1, NSP
        YMASS(is) = C_0_R
     ENDDO
     YMASS(is_fuel) = YCH4_b
     YMASS(is_ox) = YO2_b
     YMASS(is_h2o) = YH2O
     YMASS(is_co2) = YCO2
     YMASS(is_inrt) = YN2_b
  ENDIF

! Loop here
  z_error = 1.0e-4
  ilmax = 5000
  il = 1
  DO WHILE (.TRUE.)

#define MACRO_TINPUT TFLAME
#include "dns_chem_enth.h"
#undef MACRO_TINPUT

     F1 = z*z*TFLAME/WBUR-z*(a+TUNB/WUNB)+a
     F2 = z*z*(CH_H - HUNB - b) + b

     J11 = C_2_R*z*TFLAME/WBUR-(a+TUNB/WUNB)
     J12 = z*z/WBUR
     J21 = C_2_R*z*(CH_H - HUNB - b)
     J22 = z*z*CHW_CPTOTAL

     DET = J11*J22-J12*J21
     DLT_Z = (-F1*J22+F2*J12)/DET
     DLT_T = (-F2*J11+F1*J21)/DET

     z = z+DLT_Z
     TFLAME = TFLAME + DLT_T

     IF ( ABS(DLT_Z/z)+ABS(DLT_T/TFLAME) .LT. z_error ) GOTO 123
     il = il + 1
     IF ( il .GT. ilmax ) GOTO 123
  ENDDO
! End loop

123 IF ( il .LT. ilmax ) THEN

     rho2 = z*rho1
     u2 = u1/z

! Set flow parameters
     mean_u = (u1+u2)/C_2_R
     delta_u = u1-u2
     mean_rho = (rho1+rho2)/C_2_R
     delta_rho = rho1-rho2
  ELSE
     TFLAME = C_1_R
  ENDIF

  WRITE(str,'(A8,1X,E12.5E3)') 'HUNB  = ', HUNB
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'TUNB  = ', TUNB
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'TBUR  = ', TFLAME
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'MEAN_R= ', mean_rho
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'MEAN_U= ', mean_u
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'DELT_R= ', delta_rho
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'DELT_U= ', delta_u
  CALL IO_WRITE_ASCII(lfile, str)
  IF ( il .GT. ilmax ) THEN
     CALL IO_WRITE_ASCII(lfile, 'Warning: Non-linear iteration did not converged')
  ENDIF

  RETURN
END SUBROUTINE CHEM_INIT_PREMIXED
