SUBROUTINE CHEM_ONESTEP(rho, T, GM, z1, zh1, h4)

#include "types.h"
#include "dns_error.h"
  
  USE DNS_GLOBAL
  USE CHEM_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL, DIMENSION(imax,jmax,kmax)   :: rho, T, GM, h4
  TREAL, DIMENSION(imax,jmax,kmax,*) :: z1, zh1

  TINTEGER ij, k
  TINTEGER is, icp, im, ik
  TREAL cda1, cda2, cs(7)
  TREAL lnT
  TREAL omegaT
  TREAL c_r, c_o, omega
  TREAL yomega(7), csw(8)
  TREAL kr(REAC_MAX)

#ifdef PARALLEL
  TREAL dtmp
#endif

#define k1    kr(1)

  IF ( NMECH .NE. 1 ) THEN
     CALL IO_WRITE_ASCII(efile, 'Error: Wrong number of reactions for One Step mechanism')
     CALL DNS_STOP(DNS_ERROR_NBREAC)
  ENDIF

  cda1 = C_1_R/(gama0*mach*mach)
  cda2 = C_1_R/((gama0-C_1_R)*mach*mach)

! Normalized molecular weight
  DO is=1, NSP
     csw(is) = WGHT(is)/WREF
  ENDDO

  cs(1) = (NUPP(1)-NUP(1))*csw(1)*damkohler(inb_scal)
  cs(2) = (NUPP(2)-NUP(2))*csw(2)*damkohler(inb_scal)
  cs(3) = (NUPP(3)-NUP(3))*csw(3)*damkohler(inb_scal)

  TGFM = C_0_R
  IF ( damkohler(inb_scal) .EQ. C_0_R ) RETURN

  DO k=1, kmax
     DO ij=1, imax*jmax

        lnT = LOG(T(ij,1,k))

        DO ik=1,NMECH
           kr(ik) = mech_pre(ik)*&
                EXP(-mech_tac(ik)/T(ij,1,k)&
                +mech_beta(ik)*lnT)
        ENDDO

!       Check all mass fractions are positive
        DO is=1, NSP-1
           z1(ij,1,k,is) = MAX(z1(ij,1,k,is),C_0_R)
        ENDDO

!       Concentrations of intervening species
        c_r = rho(ij,1,k)*z1(ij,1,k,1)/csw(1)
        c_o = rho(ij,1,k)*z1(ij,1,k,2)/csw(2)

!       Compute Reaction Rates Here
        omega = k1*c_r*c_o
        yomega(1) = cs(1)*omega
        yomega(2) = cs(2)*omega
        yomega(3) = cs(3)*omega

!       Reaction rate is zero if zero mass fraction of this species
!       and negative reaction rate. Not possible to consume where 
!       there is no reactant.
        DO is=1, NSP-1
           IF ( z1(ij,1,k,is) .LE. C_0_R .AND. &
                yomega(is) .LT. C_0_R ) THEN
              yomega(is) = C_0_R
           ENDIF
        ENDDO

        omegaT = C_0_R

        DO is=1, NSP-1
           IF ( T(ij,1,k) .LT. THERMO_TLIM(3,is) ) THEN
              im = 2
           ELSE
              im = 1
           ENDIF

!          Enthalpy
           CHW_HI = C_0_R
           DO icp=NCP_CHEMKIN, 1, -1
              CHW_HI = CHW_HI*T(ij,1,k)&
                   + THERMO_AI(icp,im,is)/M_REAL(icp)
           ENDDO
           CHW_HI = CHW_HI*T(ij,1,k) + THERMO_AI(6,im,is)

           omegaT = omegaT + (cda1*GM(ij,1,k)*T(ij,1,k)/csw(is)&
                -cda2*(GM(ij,1,k)-C_1_R)*CHW_HI)*yomega(is)

           zh1(ij,1,k,is) = zh1(ij,1,k,is) + yomega(is)
        ENDDO
        h4(ij,1,k) = h4(ij,1,k) + omegaT
        TGFM = MAX(TGFM, ABS(omegaT))
     ENDDO
  ENDDO

! Distribute TGFM across processors
#ifdef PARALLEL
  CALL MPI_ALLREDUCE(TGFM, dtmp, 1, MPI_REAL8, MPI_MAX, MPI_COMM_WORLD, ims_err)
  TGFM = dtmp
#endif

  RETURN
END SUBROUTINE CHEM_ONESTEP
