!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2010/01/28 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Extracted from DNS_READ_GLOBAL
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"
#include "dns_error.h"
#include "dns_const.h"

SUBROUTINE CHEM_READ_GLOBAL(inifile)

  USE DNS_GLOBAL
  USE CHEM_GLOBAL
  
  IMPLICIT NONE

  CHARACTER*(*) inifile

! -------------------------------------------------------------------
  CHARACTER*512 sRes
  CHARACTER*32 bakfile

! ###################################################################
  bakfile = TRIM(ADJUSTL(inifile))//'.bak'

! ###################################################################
  CALL IO_WRITE_ASCII(bakfile, '#')
  CALL IO_WRITE_ASCII(bakfile, '#[Chem]')
  CALL IO_WRITE_ASCII(bakfile, '#Active=<yes/no>')
  CALL IO_WRITE_ASCII(bakfile, '#Configuration=<premixed/non-premixed>')
  CALL IO_WRITE_ASCII(bakfile, '#UseChemkin=<yes/no>')
  CALL IO_WRITE_ASCII(bakfile, '#ChemkinFile=<name>')
  CALL IO_WRITE_ASCII(bakfile, '#MechanismFile=<name>')
  CALL IO_WRITE_ASCII(bakfile, '#Normalize=<yes/no>')
  CALL IO_WRITE_ASCII(bakfile, '#TransportFile=<name>')
  CALL IO_WRITE_ASCII(bakfile, '#FlameInitFile=<name>')
  CALL IO_WRITE_ASCII(bakfile, '#JetPilot=<yes/no>')
  CALL IO_WRITE_ASCII(bakfile, '#PilotThickness=<value>')
  CALL IO_WRITE_ASCII(bakfile, '#PilotMaxTRatio=<value>')
  CALL IO_WRITE_ASCII(bakfile, '#PilotVelocity=<value>')
  CALL IO_WRITE_ASCII(bakfile, '#YOX=<value>')
  CALL IO_WRITE_ASCII(bakfile, '#YFUEL=<value>')

  imech_type   = 0
  ichem_config = 0
  ijet_pilot   = 0

  CALL SCANINICHAR(bakfile, inifile, 'Chem', 'Active', 'no', sRes)

  IF ( TRIM(ADJUSTL(sRes)) .EQ. 'yes' ) THEN
! check that the mixture has not been previously defined
     IF ( imixture .GT. 0 ) THEN
        CALL IO_WRITE_ASCII(efile, 'DNS_READ_GLOBAL: Mixture multiply defined')
        CALL DNS_STOP(DNS_ERROR_THERMOCONT)
     ENDIF

! Premixed or non-premixed
     CALL SCANINICHAR(bakfile, inifile, 'Chem', 'Configuration', 'non-premixed', sRes)
     IF ( TRIM(ADJUSTL(sRes)) .EQ. 'premixed' ) THEN
        ichem_config = 1
     ENDIF

! Main composition
     CALL SCANINIREAL(bakfile, inifile, 'Chem', 'YOX', '0.23', YOX)
     CALL SCANINIREAL(bakfile, inifile, 'Chem', 'YFUEL', '1.0', YFUEL)
     CALL SCANINIREAL(bakfile, inifile, 'Chem', 'SMOOTH', '0.02', dsmooth)

! Reaction type
     CALL SCANINICHAR(bakfile, inifile, 'Chem', 'Type', 'Burke-Schumann', sRes)
     IF ( TRIM(ADJUSTL(sRes)) .EQ. 'burke-schumann' ) THEN
        imech_type = CHEM_TYPE_BS
        imixture = MIXT_TYPE_BS 
        ireactive    = CHEM_INFINITE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'peters1991' ) THEN
        imech_type = CHEM_TYPE_PETERS1991
        imixture = MIXT_TYPE_PETERS1991
        ireactive    = CHEM_FINITE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'peters1988' ) THEN
        imech_type = CHEM_TYPE_PETERS1988
        imixture = MIXT_TYPE_PETERS1988
        ireactive    = CHEM_FINITE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'unidecomp' ) THEN
        imech_type = CHEM_TYPE_UNIDECOMP
        imixture = MIXT_TYPE_UNIDECOMP
        ireactive    = CHEM_FINITE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'bszeldovich' ) THEN
        imech_type = CHEM_TYPE_BSZELDOVICH
        imixture = MIXT_TYPE_BSZELDOVICH
        ireactive    = CHEM_INFINITE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'onestep' ) THEN
        imech_type = CHEM_TYPE_ONESTEP
        imixture = MIXT_TYPE_ONESTEP
        ireactive    = CHEM_FINITE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'bilger1997' ) THEN
        imech_type = CHEM_TYPE_BILGER1997
        imixture = MIXT_TYPE_BILGER1997
        ireactive    = CHEM_FINITE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'quasi-burke-schumann' ) THEN
        imech_type = CHEM_TYPE_QUASIBS
        imixture = MIXT_TYPE_QUASIBS
        ireactive    = CHEM_FINITE
     ENDIF

! Files names
     IF ( ( ireactive .EQ. CHEM_FINITE .AND. imech_type .NE. CHEM_TYPE_QUASIBS ) .OR. &
          imech_type .EQ. CHEM_TYPE_BSZELDOVICH ) THEN 

        CALL SCANINICHAR(bakfile, inifile, 'Chem', 'MechanismFile', 'no-mechanism-file', mech_file)

        CALL SCANINICHAR(bakfile, inifile, 'Chem', 'Normalize', 'undef', sRes)
        IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'yes' ) THEN; ichem_normalized = 1
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'no'  ) THEN; ichem_normalized = 0
        ELSE
           CALL IO_WRITE_ASCII(efile, 'CHEM_READ_GLOBAL: Must specify chemistry normalization')
           CALL DNS_STOP(DNS_ERROR_NORMALIZE)
        ENDIF

        CALL SCANINICHAR(bakfile, inifile, 'Chem', 'FlameInitFile', 'no-flame-file', flame_ini_file)
     ENDIF

! Pilot flame decription
     CALL SCANINICHAR(bakfile, inifile, 'Chem', 'JetPilot', 'no', sRes)
     IF ( TRIM(ADJUSTL(sRes)) .EQ. 'yes' ) THEN
        ijet_pilot = 1

        CALL SCANINIREAL(bakfile, inifile, 'Chem', 'PilotThickness', '0.1',  rjet_pilot_thickness)
        CALL SCANINIREAL(bakfile, inifile, 'Chem', 'PilotMaxTRatio', '0.99', rjet_pilot_maxtratio)
        CALL SCANINIREAL(bakfile, inifile, 'Chem', 'PilotVelocity',  '0.25', rjet_pilot_velocity)
     ELSE
        ijet_pilot = 0
     ENDIF

! use chemkin for the thermodynamic data
     CALL SCANINICHAR(bakfile, inifile, 'Chem', 'ChemkinFile', 'none', chemkin_file)
     IF ( TRIM(ADJUSTL(chemkin_file)) .EQ. 'none' ) THEN; iuse_chemkin = 0
     ELSE;                                                iuse_chemkin = 1; ENDIF

  ELSE
     ireactive = CHEM_NONE

  ENDIF

! ###################################################################
! Initialization
! ###################################################################
  IF ( ireactive .NE. CHEM_NONE ) THEN
     CALL CHEM_INIT
! Initializing reaction mechanism constants and transport properties
     IF ( imech_type .GT. 1 .AND. imech_type .NE. CHEM_TYPE_QUASIBS) THEN
        CALL CHEM_READ_MECH(mech_file)
     ENDIF
  ENDIF

! ###################################################################
! Check
! ###################################################################
  IF ( ireactive .NE. CHEM_NONE .AND. icalc_scal .EQ. 0 ) THEN
     CALL IO_WRITE_ASCII(efile,'Scalar must be calculated for a reactive case.')
     CALL DNS_STOP(DNS_ERROR_CHECKSCALAR)
  ENDIF

  RETURN
END SUBROUTINE CHEM_READ_GLOBAL
