!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2009/10/03 - J.P. Mellado
!#              Modified
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate autocorrelation at a plane j of array u.
!# First element of rux returns the variance
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE CORR1V2PTX(nx,ny,nz, j, dx, u, rux, lux, tmp)

  IMPLICIT NONE

#include "types.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

#include "integers.h"

  TINTEGER nx, ny, nz, j
  TREAL lux

  TREAL, DIMENSION(nx,ny,nz) :: u
  TREAL, DIMENSION(nx,nz)    :: tmp
  TREAL, DIMENSION(nx)       :: rux, dx

! -----------------------------------------------------------------------
  TINTEGER i, k, id, index, idmax
  TREAL AVG1V3D, SIMPSON

! #######################################################################
! maximum translational off-set in X
  idmax = nx/2

! saving along second index in tmp the 
! product of the signal with its translation defined by idlocal
  DO id = 0,idmax

      DO k = 1,nz
        DO i = 1,nx
           index = i + id; index = index + nx - 1; index = MOD(index,nx)+1
           tmp(i,k) = u(index,j,k)*u(i,j,k)
        ENDDO
     ENDDO

     rux(id+1) = AVG1V3D(nx,i1,nz, i1, tmp)

  ENDDO

  IF ( rux(1) .NE. C_0_R ) THEN
     lux = SIMPSON(idmax+1, rux, dx)/rux(1)
     DO id=2,idmax+1
        rux(id) = rux(id)/rux(1)
     ENDDO
  ELSE
     lux = C_0_R
  ENDIF

  RETURN
END SUBROUTINE CORR1V2PTX
