#!/bin/csh 
################################################################################
# 13 December 2011 cedrick.ansorge@zmaw.de                                            
# This script sets up dns.ini files and runscripts for scaling measurements 
# in the DNS code (version 5.6.6)
#
# - dns.ini files are named as dns.ini_<ims_npro_k>x<ims_npro_i>
#    
# - job-scripts have names test_<ims_npro>.sh and in these script the files 
#   dns.ini_<ims_npro_k>x<ims_npro_i> are linked to dns.ini 
#
################################################################################
# ARGUMENTS
# working directory (folder where simulations are done; as in qsend) 
#
################################################################################
# DEPENDENCIES 
# - needs dns.ini.template in the directory where it is run. 
#   From this template information about the geometry is extracted which is 
#   then used to figure out possible domain decompositions given a number
#   of cores 
# - dns.ini.template needs to have marked the line with Kmax(*) and Imax(*) 
#   by N_III respectively N_KKK - these strings are then replaced by the actual
#   of points    
################################################################################

if ( $1 == "" ) then 
    echo "Usage: ./make_dns-ini.sh <WORKING DIRECTORY>" 
    exit 1
endif

set WDIR="$1"

set imax_total=`grep Imax dns.ini.template | head -1 | cut -d= -f2`
set kmax_total=`grep Kmax dns.ini.template | head -1 | cut -d= -f2`

echo "${imax_total}x${kmax_total}"

foreach ims_npro ( 0256 0512 1024 2048 4096 8192 ) 
    set first=1
    set script_name="test_${ims_npro}.sh"
    if ( ${ims_npro} == "0256" ) then 
	set SHAPE="1x1x1" 
	set WALLCLOCK="04:00:00" 
    else if ( ${ims_npro} == "0512" ) then 
	set SHAPE="1x1x1" 
	set WALLCLOCK="04:00:00"
    else if ( ${ims_npro} == "1024" ) then 
	set SHAPE="2x1x1"
	set WALLCLOCK="04:00:00"
    else if ( ${ims_npro} == "2048" ) then 
        set SHAPE="2x2x1"
	set WALLCLOCK="03:00:00"
    else if ( ${ims_npro} == "4096" ) then 
        set SHAPE="2x2x2"
	set WALLCLOCK="02:00:00"
    else if ( ${ims_npro} == "8192" ) then 
	set SHAPE="4x2x2" 
	set WALLCLOCK="01:00:00"
    endif 


    
    cat <<EOF>${script_name} 
#!/bin/csh
#
#@ job_type = bluegene
#@ environment = COPY_ALL;
#@ notify_user = cedrick.ansorge@zmaw.de
#@ notification = always
#@ bg_connection = TORUS
########################### CHANGE ONLY THIS ###############################
#@ job_name = ${WDIR}_${ims_npro} 
#@ output = ll_out_${ims_npro}
#@ error  = ll_err_${ims_npro}
#@ bg_shape= ${SHAPE}
#@ bg_rotate= FALSE
#@ wall_clock_limit = ${WALLCLOCK}
########################### END LSF DIRECTIVES #############################
#@ queue

############################################################################
cd /work/hhh07/hhh071/${WDIR} 

setenv MPIRUN_MODE SMP
setenv MPIRUN_NP ${ims_npro} 
setenv MPIRUN_VERBOSE 2

setenv MPIRUN_ENV "OMP_NUM_THREADS=4,DCMF_RECFIFO=16777216"

EOF
   foreach ims_npro_k ( 4096 2048 1024 0512 0256 0128 0064 0032 0016 0008 0004 0002 ) 
	if ( ${ims_npro} >= ${ims_npro_k} ) then  
	    set ims_npro_i=`echo "${ims_npro}/${ims_npro_k}" | bc` 
	    set imax=`echo "${imax_total}/${ims_npro_i}" | bc`
	    set kmax=`echo "${kmax_total}/${ims_npro_k}" | bc`
	    set imax_check=`echo "${imax}*${ims_npro_i} -${imax_total}" | bc `
	    set kmax_check=`echo "${kmax}*${ims_npro_k} -${kmax_total}" | bc `
	    if ( ${ims_npro_k} >= ${ims_npro_i} && ${imax_check} == 0 && ${kmax_check} == 0 ) then
		echo "${ims_npro} Procs: ${ims_npro_k}x${ims_npro_i} Grid: ${imax}x${kmax} Problems: ${imax_check}x${kmax_check}"
		sed -e"s/N_III/${imax}/g" dns.ini.template > tmp
		sed -e"s/N_KKK/${kmax}/g" tmp > dns.ini_${ims_npro_k}x${ims_npro_i}
		if ( ${first} == 1 ) then 
		    cat <<EOF>>${script_name} 
EOF
		endif 
		cat <<EOF>>${script_name} 
ln -sf dns.ini_${ims_npro_k}x${ims_npro_i} dns.ini 
mpirun ./dns_omp.x 
EOF
	    endif 	
	endif
    end
end
