!########################################################################
!# Tool/Library FIELDS
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/17 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the gradient diffusion term as given by G_i lap G_i
!# The diffusivity D is not multiplied here.
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"

SUBROUTINE FI_GRADIENT_DIFFUSION&
     (iunifx, iunify, iunifz, imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
     dx,dy,dz, z1, result, grad, tmp1, tmp2, tmp3, tmp4, wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "integers.h"

  TINTEGER iunifx, iunify, iunifz
  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL, DIMENSION(*) :: dx, dy, dz

  TREAL z1(*), result(*)
  TREAL grad(*)
  TREAL tmp1(*), tmp2(*), tmp3(*), tmp4(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

! -------------------------------------------------------------------
  TINTEGER ij

! ###################################################################
! -------------------------------------------------------------------
! G_x
! -------------------------------------------------------------------
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, z1, grad, i0, i0, wrk1d, wrk2d, wrk3d)

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, grad, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, grad, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, grad, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = grad(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! G_y
! -------------------------------------------------------------------
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, z1, grad, i0, i0, wrk1d, wrk2d, wrk3d)

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, grad, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, grad, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, grad, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + grad(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! G_z
! -------------------------------------------------------------------
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, z1, grad, i0, i0, wrk1d, wrk2d, wrk3d)

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, grad, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, grad, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, grad, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + grad(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

  RETURN
END SUBROUTINE FI_GRADIENT_DIFFUSION
