!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/11 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the strain as given by s_ij s_ij:
!# u,x^2 + v,y^2 + w,z^2 +1/2( (u,y+v,x)^2 + (u,z+w,x)^2 + (v,z+w,y)^2 )
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE FI_STRAIN(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx, dy, dz, u, v, w, result, tmp1, tmp2, wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "types.h"
#include "integers.h"

  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax)

  TREAL u(*), v(*), w(*), result(*)
  TREAL tmp1(*), tmp2(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

  ! -------------------------------------------------------------------
  TINTEGER ij
  TREAL dummy

  ! ###################################################################
  ! Ux, Vy, Wz
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = tmp1(ij)*tmp1(ij) + tmp2(ij)*tmp2(ij)
  ENDDO
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp2(ij)*tmp2(ij)
  ENDDO

  ! Uy, Vx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     dummy = tmp1(ij)+tmp2(ij)
     result(ij) = result(ij) + C_05_R*dummy*dummy
  ENDDO

  ! Uz, Wx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     dummy = tmp1(ij)+tmp2(ij)
     result(ij) = result(ij) + C_05_R*dummy*dummy
  ENDDO

  ! Vz, Wy
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     dummy = tmp1(ij)+tmp2(ij)
     result(ij) = result(ij) + C_05_R*dummy*dummy
  ENDDO

  RETURN
END SUBROUTINE FI_STRAIN
