!########################################################################
!# Tool/Library FIELDS
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/12 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the vorticity production term as given by w_i w_j s_ij
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE FI_VORTICITY_PRODUCTION&
     (imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx, dy, dz, u, v, w, result, vort_x, vort_y, vort_z, tmp1, tmp2, &
     wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "types.h"
#include "integers.h"

  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax)

  TREAL u(*), v(*), w(*), result(*)
  TREAL vort_x(*), vort_y(*), vort_z(*)
  TREAL tmp1(*), tmp2(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

! -------------------------------------------------------------------
  TINTEGER ij

! ###################################################################
! Vorticity vector
! ###################################################################
! Uy, Vx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     vort_z(ij) = tmp1(ij)-tmp2(ij)
  ENDDO

! Uz, Wx
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     vort_y(ij) = tmp1(ij)-tmp2(ij)
  ENDDO

! Vz, Wy
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     vort_x(ij) = tmp1(ij)-tmp2(ij)
  ENDDO

! ###################################################################
! Production term
! ###################################################################
! Ux, Vy, Wz
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = tmp1(ij)*vort_x(ij)*vort_x(ij) + tmp2(ij)*vort_y(ij)*vort_y(ij)
  ENDDO
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp2(ij)*vort_z(ij)*vort_z(ij)
  ENDDO

! Uy, Vx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + (tmp1(ij)+tmp2(ij))*vort_x(ij)*vort_y(ij)
  ENDDO

! Uz, Wx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + (tmp1(ij)+tmp2(ij))*vort_x(ij)*vort_z(ij)
  ENDDO

! Vz, Wy
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + (tmp1(ij)+tmp2(ij))*vort_y(ij)*vort_z(ij)
  ENDDO

  RETURN
END SUBROUTINE FI_VORTICITY_PRODUCTION
