SUBROUTINE FILTERED_CHI(n0, x,y,z,dx,dy,dz, z1, chi,&
     tmp1,tmp2,tmp3, wrk1d,wrk2d,wrk3d, cf0x,cf0y,cf0z)

! # This subroutine provides a model for filtered scalar 
! # dissipation \b{\rho\chi}:
! #
! # 1. SGS scalar dissipation: input
! # 2. Resolved part: thermal variation of \rho*D is neglected
! #
! # The factor of 2 is NOT included here. Do not forget later !!!

#include "types.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER n0

  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL z1(imax,jmax,kmax)
  TREAL chi(imax,jmax,kmax)

  TREAL tmp1(imax,jmax,kmax)
  TREAL tmp2(imax,jmax,kmax)
  TREAL tmp3(imax,jmax,kmax)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(*)
  TREAL wrk3d(imax,jmax,kmax)

  TREAL cf0x(*), cf0y(*), cf0z(*)

  TINTEGER ij, k, j, i
  TREAL diff, SIMPSON_NU, AVG_IK, chi_coef, r0

  r0 = C_0_R

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING FILTERED_CHI')
#endif

! ----------------------------------------------------------------------
!                       Adding resolved part to the subgrid transfer 
! ----------------------------------------------------------------------
  IF ( iles_inviscid .NE. 1 ) THEN
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, z1, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, z1, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, z1, tmp3, i0, i0, wrk1d, wrk2d, wrk3d)
     IF ( idiffusion .EQ. EQNS_NONE ) THEN; diff = C_0_R
     ELSE;                                  diff = visc/schmidt(inb_scal); ENDIF
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              tmp1(ij,1,k) =  &
                   tmp1(ij,1,k)*tmp1(ij,1,k) +&
                   tmp2(ij,1,k)*tmp2(ij,1,k) +&
                   tmp3(ij,1,k)*tmp3(ij,1,k) 
              chi(ij,1,k) = chi(ij,1,k) + diff*tmp1(ij,1,k)
           ENDDO
        ENDDO
     ENDIF

     IF ( iles_type_dischem .EQ. LES_DISCHEM_DISSSG ) THEN
! ----------------------------------------------------------------------
!                                                          SGS model 
! ----------------------------------------------------------------------
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              tmp1(ij,1,k) = MAX(r0,chi(ij,1,k))
           ENDDO
        ENDDO
! conserving the mean profile
        DO j = 1,jmax
           wrk1d(j,1) = AVG_IK(imax, jmax, kmax, j, chi, &
                dx, dz, area)
           wrk1d(j,2) = AVG_IK(imax, jmax, kmax, j, tmp1, &
                dx, dz, area)
        ENDDO
        chi_coef = C_2_R*SIMPSON_NU(jmax, wrk1d(1,1), y)/&
             SIMPSON_NU(jmax, wrk1d(1,2), y)
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              tmp1(ij,1,k) = chi_coef*tmp1(ij,1,k)
           ENDDO
        ENDDO
     ELSE IF ( iles_type_dischem .EQ. LES_DISCHEM_GRAD ) THEN
! ----------------------------------------------------------------------
!                                                     gradient model
! ----------------------------------------------------------------------
        IF ( iles_inviscid .NE. 1 ) THEN
           CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
                dx, z1, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
           CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
                dy, z1, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
           CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
                dz, z1, tmp3, i0, i0, wrk1d, wrk2d, wrk3d)
           DO k = 1,kmax
              DO ij = 1,imax*jmax
                 tmp1(ij,1,k) =  &
                      tmp1(ij,1,k)*tmp1(ij,1,k) +&
                      tmp2(ij,1,k)*tmp2(ij,1,k) +&
                      tmp3(ij,1,k)*tmp3(ij,1,k) 
              ENDDO
           ENDDO
        ENDIF

        DO j = 1,jmax
           wrk1d(j,1) = AVG_IK(imax, jmax, kmax, j, chi, &
                dx, dz, area)
           wrk1d(j,2) = AVG_IK(imax, jmax, kmax, j, tmp1, &
                dx, dz, area)
           IF ( wrk1d(j,2) .GT. C_0_R ) THEN
              DO k = 1,kmax
                 DO i = 1,imax
                    tmp1(i,j,k) = C_2_R*tmp1(i,j,k)*&
                         wrk1d(j,1)/wrk1d(j,2)
                 ENDDO
              ENDDO
           ELSE
              DO k = 1,kmax
                 DO i = 1,imax
                    tmp1(i,j,k) = C_0_R
                 ENDDO
              ENDDO
           ENDIF
        ENDDO

     ENDIF

! ----------------------------------------------------------------------
!                                                          filtering
! ----------------------------------------------------------------------
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, tmp1, chi, wrk3d)

#ifdef TRACE_ON
     CALL IO_WRITE_ASCII(tfile, 'LEAVING FILTERED_CHI' )
#endif


  END IF
  RETURN
END SUBROUTINE FILTERED_CHI
