!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# txc size is 10 if dynamic and ssmflg is 0
!# txc size is 12 if dynamic and ssmflg is 1
!# txc size is 11 if ssmflg is 1
!#
!########################################################################
SUBROUTINE LES_AVG_SCAL_TEMPORAL_LAYER&
     (is, x,y,z,dx,dy,dz, rho,u,v,w,z1, u_sg,v_sg,w_sg,aux_sg, &
     txc, mean2d, chi, wrk1d,wrk2d,wrk3d,&
     cf0x,cf0y,cf0z,cf1x,cf1y,cf1z,cf2x,cf2y,cf2z, carm_uf,carm_zf,carm_uh,carm_zh)

#include "types.h"
#include "dns_error.h"
#include "dns_const.h"
#include "les_const.h"

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
  USE LES_GLOBAL
  USE NFILE,     ONLY : efile

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER is

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: u, v, w, rho, z1, u_sg, v_sg, w_sg, aux_sg, chi
  TREAL, DIMENSION(imax,jmax,kmax,*) :: txc

  TREAL mean2d(jmax,*)
  TREAL wrk1d(jmax,*)
  TREAL wrk2d(imax,kmax,*)
  TREAL wrk3d(imax,jmax,kmax)

  TREAL, DIMENSION(*) :: cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z
  TREAL, DIMENSION(*) :: carm_uf, carm_uh, carm_zf, carm_zh

! -----------------------------------------------------------------------
#define smg_cs_Z(j)    mean2d(j,1)
#define smg_mu_Z(j)    mean2d(j,2)

#define smg_qx(j)      mean2d(j,3)
#define smg_qy(j)      mean2d(j,4) 
#define smg_qz(j)      mean2d(j,5)
#define smg_dis(j)     mean2d(j,6)
#define smg_trans(j)   mean2d(j,7)
#define smg_force(j)   mean2d(j,8)

#define ssm_cs_Z(j)    mean2d(j,9)

#define ssm_qx(j)      mean2d(j,10)
#define ssm_qy(j)      mean2d(j,11) 
#define ssm_qz(j)      mean2d(j,12)
#define ssm_dis(j)     mean2d(j,13)
#define ssm_trans(j)   mean2d(j,14)
#define ssm_force(j)   mean2d(j,15)

#define tot_dis(j)     mean2d(j,16)
#define tot_trans(j)   mean2d(j,17)
#define tot_mean(j)    mean2d(j,18)
#define tot_force(j)   mean2d(j,19)

#define tot_RZx(j)     mean2d(j,20)
#define tot_RZy(j)     mean2d(j,21)
#define tot_RZz(j)     mean2d(j,22)

#define L_AVGMAX 22

  TINTEGER i, j, k, ij
  TREAL AVG_IK, UPPER_THRESHOLD, LOWER_THRESHOLD
  TREAL delta_w, SimVar1, SimVar2, dummy
  TREAL delta_ht01, delta_hb01, delta_h01
  TREAL r23, r13, coefdelta
  TREAL reyZ, favZ, den, dreyZdy
  TINTEGER n0, n1, l1, l2, ll1, ll2, ssmflg
  TINTEGER ip_tmp, ip_den
  CHARACTER*32 num
  CHARACTER*250 line1
  CHARACTER*500 line2

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_AVG_SCAL_TEMPORAL_LAYER' )
#endif

  r13 = C_1_R/C_3_R
  r23 = C_2_R/C_3_R

  n0 = isgs_f0size
  n1 = isgs_f1size
  coefdelta = (M_REAL(n1)/M_REAL(n0))**C_2_R + C_1_R

  IF ( iles_type_tran .EQ. LES_TRAN_NONE ) THEN 
     ssmflg = 0
  ELSE
     ssmflg = 1
  ENDIF

  IF ( MAX_AVG_TEMPORAL .LT. L_AVGMAX ) THEN
     CALL IO_WRITE_ASCII(efile,'LES_AVG_SCAL_TEMPORAL_LAYER. Not enough space.')
     CALL DNS_STOP(LES_ERROR_AVGTMP)
  ENDIF
  mean2d(:,1:L_AVGMAX) = C_0_R

! ###################################################################
! Header
! ###################################################################
  num='lesavg '
  CALL CONCATI(num, is)
  CALL CONCAT(num, 's')
  CALL CONCATI(num, itime)

#ifdef PARALLEL
  CALL MPI_COMM_RANK(MPI_COMM_WORLD,ims_pro,ims_err)

  IF ( ims_pro .EQ. 0 ) THEN
#endif

#ifdef USE_RECLEN
     OPEN(UNIT=23, RECL=352, FILE=num, STATUS='unknown')
#else
     OPEN(UNIT=23, FILE=num, STATUS='unknown')
#endif

     WRITE(i23,1010) '#################'
     WRITE(i23,1010) '#  Version 1.0  #'
     WRITE(i23,1010) '#################'
     WRITE(i23,'(A8,E14.7E3)') 'RTIME = ', rtime

     line2='I J Y S1'
     CALL CLIP(line2,ll1,ll2)

     line1 = 'Cs_Z rho*D_sg smg_qx smg_qy smg_qz smg_Chi '&
          //'smg_trans smg_force'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = SMG '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='Cssm ssm_qx ssm_qy ssm_qz ssm_Chi ssm_trans ssm_force'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = SSM '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='tot_Chi tot_trans tot_mean tot_force'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = Balances '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='RZx RZy RZz'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = Reynolds_streeses '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     WRITE(i23,1010) line2(ll1:ll2)

1010 FORMAT(A)

#ifdef PARALLEL
  ENDIF
#endif

! #######################################################################
! Scale similarity  part
! #######################################################################
#define BARVS(i,j,k)    txc(i,j,k,1)
#define MU_M(i,j,k)     txc(i,j,k,2)
#define MV_M(i,j,k)     txc(i,j,k,3)
#define MW_M(i,j,k)     txc(i,j,k,4)
#define RZ_M(i,j,k)     txc(i,j,k,5)
#define BARMU_M(i,j,k)  txc(i,j,k,6)
#define BARMV_M(i,j,k)  txc(i,j,k,7)
#define BARMW_M(i,j,k)  txc(i,j,k,8)
#define BARRZ_M(i,j,k)  txc(i,j,k,9)
#define Q_M(i,j,k)      txc(i,j,k,10)
#define TMP1_LOC(i,j,k) txc(i,j,k,11)
  IF ( ssmflg .EQ. 1 ) THEN
! ------------------------------------------------------------
!                                Defining the model fields
! ------------------------------------------------------------
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, rho, BARVS(1,1,1), wrk3d)
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           BARVS(ij,1,k) = C_1_R/BARVS(ij,1,k)
        ENDDO
     ENDDO
! ------------------------------------------------------------
!                              small resolved scale fields
! ------------------------------------------------------------
! txc_size=1
!     CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!          cf0x, cf0y, cf0z, rho, u, u_sg, TMP1_LOC(1,1,1), wrk2d, wrk3d)
!     CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!          cf0x, cf0y, cf0z, rho, v, v_sg, TMP1_LOC(1,1,1), wrk2d, wrk3d)
!     CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!          cf0x, cf0y, cf0z, rho, w, w_sg, TMP1_LOC(1,1,1), wrk2d, wrk3d)
!     CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!          cf0x, cf0y, cf0z, rho, z1, aux_sg, TMP1_LOC(1,1,1), wrk2d, wrk3d)

     IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              MU_M(ij,1,k) = rho(ij,1,k)*u(ij,1,k)
              MV_M(ij,1,k) = rho(ij,1,k)*v(ij,1,k)
              MW_M(ij,1,k) = rho(ij,1,k)*w(ij,1,k)
              RZ_M(ij,1,k) = rho(ij,1,k)*z1(ij,1,k)
!              MU_M(ij,1,k) = rho(ij,1,k)*u_sg(ij,1,k)
!              MV_M(ij,1,k) = rho(ij,1,k)*v_sg(ij,1,k)
!              MW_M(ij,1,k) = rho(ij,1,k)*w_sg(ij,1,k)
!              RZ_M(ij,1,k) = rho(ij,1,k)*aux_sg(ij,1,k)
           ENDDO
        ENDDO
     ELSE 
! txc size is 4 and goes into BAR_MU
!            CALL SGS_ARM_U(n0, x, y, z, dx, dy, dz, &
!                 rho, u_sg, v_sg, w_sg, BARVS(1,1,1), carm_uf, &
!                 MU_M(1,1,1), MV_M(1,1,1), MW_M(1,1,1),&
!                 BARMU_M(1,1,1), &
!                 wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z) 
!            CALL SGS_ARM_Z(iles_type_tran,&
!                 n0, r0, x, y, z, dx, dy, dz, &
!                 rho, aux_sg, BARVS(1,1,1), carm_zf, RZ_M(1,1,1), &
!                 BARMU_M(1,1,1), wrk1d, wrk2d, wrk3d,&
!                 cf0x, cf0y, cf0z)
     ENDIF

     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, MU_M(1,1,1), BARMU_M(1,1,1), wrk3d)
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, MV_M(1,1,1), BARMV_M(1,1,1), wrk3d)
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, MW_M(1,1,1), BARMW_M(1,1,1), wrk3d)
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, RZ_M(1,1,1), BARRZ_M(1,1,1), wrk3d)

! -----------------------------------------------------------------    
!                                             qx = (Z_m*U_m)_sg
! -----------------------------------------------------------------    
     CALL SGS_RECTN_TAUIJ(n0, rho, RZ_M(1,1,1), MU_M(1,1,1), &
          BARVS(1,1,1), BARRZ_M(1,1,1), BARMU_M(1,1,1), Q_M(1,1,1),&
          TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, z1, TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = Q_M(i,j,k)*TMP1_LOC(i,j,k)
           ENDDO
        ENDDO
        ssm_qx(j) = AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
        ssm_dis(j) = ssm_dis(j)+&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              chi(ij,1,k) = chi(ij,1,k)-&
                   Q_M(ij,1,k)*TMP1_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, Q_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = z1(i,j,k)*TMP1_LOC(i,j,k)
           ENDDO
        ENDDO
        ssm_force(j) = ssm_force(j)-&
             AVG_IK(imax, jmax, kmax, j, TMP1_LOC(1,1,1), dx, dz, area)
        ssm_trans(j) = ssm_trans(j)+&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

! -----------------------------------------------------------------    
!                                             qy = (Z_m*V_m)_sg
! -----------------------------------------------------------------    
     CALL SGS_RECTN_TAUIJ(n0, rho, RZ_M(1,1,1), MV_M(1,1,1), &
          BARVS(1,1,1), BARRZ_M(1,1,1), BARMV_M(1,1,1), Q_M(1,1,1),&
          TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, z1, TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = Q_M(i,j,k)*TMP1_LOC(i,j,k)
           ENDDO
        ENDDO
        ssm_qy(j) = AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
        ssm_dis(j) = ssm_dis(j)+&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              chi(ij,1,k) = chi(ij,1,k)-&
                   Q_M(ij,1,k)*TMP1_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, Q_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = z1(i,j,k)*TMP1_LOC(i,j,k)
           ENDDO
        ENDDO
        ssm_force(j) = ssm_force(j)-&
             AVG_IK(imax, jmax, kmax, j, TMP1_LOC(1,1,1), dx, dz, area)
        ssm_trans(j) = ssm_trans(j)+&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

     IF ( kmax_total .GT. 1 ) THEN
! -----------------------------------------------------------------    
!                                             qz = (Z_m*W_m)_sg
! -----------------------------------------------------------------    
        CALL SGS_RECTN_TAUIJ(n0, rho, RZ_M(1,1,1), MW_M(1,1,1), &
             BARVS(1,1,1), BARRZ_M(1,1,1), BARMW_M(1,1,1), Q_M(1,1,1),&
             TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
             dz, z1, TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = Q_M(i,j,k)*TMP1_LOC(i,j,k)
              ENDDO
           ENDDO
           ssm_qz(j) = AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
           ssm_dis(j) = ssm_dis(j)+&
                AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
        ENDDO
        IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
           DO k = 1,kmax
              DO ij = 1,imax*jmax
                 chi(ij,1,k) = chi(ij,1,k)-&
                      Q_M(ij,1,k)*TMP1_LOC(ij,1,k)
              ENDDO
           ENDDO
        ENDIF
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
             dz, Q_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = z1(i,j,k)*TMP1_LOC(i,j,k)
              ENDDO
           ENDDO
           ssm_force(j) = ssm_force(j)-&
                AVG_IK(imax, jmax, kmax, j, TMP1_LOC(1,1,1), dx, dz, area)
           ssm_trans(j) = ssm_trans(j)+&
                AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
        ENDDO
     ENDIF

  ENDIF
#undef BARVS
#undef BARMU_M
#undef BARMV_M
#undef BARMW_M
#undef BARRZ_M
#undef MU_M
#undef MV_M
#undef MW_M
#undef RZ_M
#undef Q_M
#undef TMP1_LOC

! Output data
!  DO j = 1,jmax
!     ssm_cs_Z(j) = carm_zf(j)
!  ENDDO

! #######################################################################
! Smagorinsky part 
! #######################################################################
! defining the array distribution
  ip_den = 3
  IF ( ssmflg .EQ. 1 ) THEN
     ip_tmp = ip_den + 3
  ELSE
     ip_tmp = ip_den + 1
  ENDIF

#define STRAIN0(i,j,k)        txc(i,j,k,1)
#define STRAIN1(i,j,k)        txc(i,j,k,2)
#define RHOINV1(i,j,k)        txc(i,j,k,ip_den)
#define DIFFSG(i,j,k)         txc(i,j,k,ip_tmp)

  IF ( iles_type_regu .NE. LES_REGU_NONE ) THEN
!-------------------------------------------------------------
!                                                 Preamble
!-------------------------------------------------------------
     IF ( iles_type_regu .EQ. LES_REGU_SMGDYN .OR.&
          iles_type_regu .EQ. LES_REGU_SMGDYNRMS ) THEN
! Specific volumes (txc_size is 1)
        CALL SGS_DYN_BARVS(n0, n1, rho, RHOINV1(1,1,1), txc(1,1,1,ip_tmp), wrk3d, &
             cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z)
! remove mean if needed (txc_size is 1)
        CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
             cf0x, cf0y, cf0z, rho, u, u_sg, txc(1,1,1,ip_tmp), wrk2d, wrk3d)
        CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
             cf0x, cf0y, cf0z, rho, v, v_sg, txc(1,1,1,ip_tmp), wrk2d, wrk3d)
        CALL SGS_FIELDS(i0, n0, dx, dy, dz, &
             cf0x, cf0y, cf0z, rho, w, w_sg, txc(1,1,1,ip_tmp), wrk2d, wrk3d)
! ARM coefficients (txc_size is 6) 
!        IF ( iles_type_tran .EQ. LES_TRAN_ARMRANS .OR.&
!             iles_type_tran .EQ. LES_TRAN_ARMSPTL .OR.&
!             iles_type_tran .EQ. LES_TRAN_ARMINVS ) THEN
!            CALL SGS_DYN_ARM_U(n0, n1, x, y, z, dx, dy, dz, &
!                 rho, u_sg, v_sg, w_sg, RHOINV1(1,1,1), &
!                 txc(1,1,1,ip_tmp), wrk1d, wrk2d, wrk3d, &
!                 cf0x, cf0y, cf0z,&
!                 cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, &
!                 carm_uf, carm_uh)
!        ENDIF
! Norms of strain rate tensor (txc_size is 6)
        CALL SGS_DYN_SIJ(n0, n1, x, y, z, dx, dy, dz,&
             rho, u_sg, v_sg, w_sg, STRAIN0(1,1,1), STRAIN1(1,1,1), &
             RHOINV1(1,1,1), txc(1,1,1,ip_tmp), wrk1d, wrk2d, wrk3d, cf1x, cf1y, cf1z)
! Dynamic coefficients
        CALL SGS_DYN_Z(ssmflg, coefdelta, n0,n1, x,y,z,dx,dy,dz, &
             rho,u,v,w,z1, u_sg,v_sg,w_sg,aux_sg, &
             STRAIN0(1,1,1), STRAIN1(1,1,1), RHOINV1(1,1,1), txc(1,1,1,ip_tmp), &
             smg_cs_Z(1), wrk1d, wrk2d, wrk3d, &
             cf0x,cf0y,cf0z,cf1x,cf1y,cf1z,cf2x,cf2y,cf2z, carm_uf,carm_uh,carm_zf,carm_zh)
! rho*D_sg = rho*CsZ*delta_f^2*S in txc(it_tmp)
        DO k = 1,kmax
           DO j = 1,jmax
              DO i = 1,imax
                 txc(i,j,k,ip_tmp) = smg_cs_Z(j)*STRAIN0(i,j,k)
              ENDDO
           ENDDO
        ENDDO
        DO j = 1,jmax
           smg_cs_Z(j) = wrk1d(j,1)
           smg_mu_Z(j) = AVG_IK(imax, jmax, kmax, j, txc(1,1,1,ip_tmp), dx, dz, area)
        ENDDO
     ELSE IF ( iles_type_regu .EQ. LES_REGU_SMGSTA ) THEN
! static part to be developed
     ENDIF

#define ZPSG(i,j,k)     txc(i,j,k,ip_tmp+1)
#define Q_M(i,j,k)      txc(i,j,k,ip_tmp+2)
#define TMP_LOC(i,j,k)  txc(i,j,k,ip_tmp+3)

! --------------------------------------------------------
!                                               X gradient
! --------------------------------------------------------
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, aux_sg, ZPSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, z1, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              Q_M(i,j,k) = &
                   DIFFSG(i,j,k)*ZPSG(i,j,k)
              wrk3d(i,1,k) = Q_M(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_qx(j) =-AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
        smg_dis(j) = smg_dis(j)-&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              chi(ij,1,k) = chi(ij,1,k)+&
                   Q_M(ij,1,k)*TMP_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, Q_M(1,1,1), TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = z1(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_force(j) = smg_force(j)+&
             AVG_IK(imax, jmax, kmax, j, TMP_LOC(1,1,1), dx, dz, area)
        smg_trans(j) = smg_trans(j)-&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

! --------------------------------------------------------
!                                               Y gradient
! --------------------------------------------------------
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, aux_sg, ZPSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, z1, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              Q_M(i,j,k) = &
                   DIFFSG(i,j,k)*ZPSG(i,j,k)
              wrk3d(i,1,k) = Q_M(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_qy(j) =-AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
        smg_dis(j) = smg_dis(j)-&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              chi(ij,1,k) = chi(ij,1,k)+&
                   Q_M(ij,1,k)*TMP_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, Q_M(1,1,1), TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = z1(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_force(j) = smg_force(j)+&
             AVG_IK(imax, jmax, kmax, j, TMP_LOC(1,1,1), dx, dz, area)
        smg_trans(j) = smg_trans(j)-&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

! --------------------------------------------------------
!                                               Z gradient
! --------------------------------------------------------
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, aux_sg, ZPSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, z1, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              Q_M(i,j,k) = &
                   DIFFSG(i,j,k)*ZPSG(i,j,k)
              wrk3d(i,1,k) = Q_M(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_qz(j) =-AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
        smg_dis(j) = smg_dis(j)-&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              chi(ij,1,k) = chi(ij,1,k)+&
                   Q_M(ij,1,k)*TMP_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, Q_M(1,1,1), TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = z1(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_force(j) = smg_force(j)+&
             AVG_IK(imax, jmax, kmax, j, TMP_LOC(1,1,1), dx, dz, area)
        smg_trans(j) = smg_trans(j)-&
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

#undef ZPSG
#undef Q_M
#undef TMP_LOC

  ENDIF

#undef STRAIN0
#undef STRAIN1
#undef RHOINV1
#undef DIFFSG

! #######################################################################
! Total quantities
! #######################################################################
! Signs: (smg,ssm)_qi .....: +<Q_Z>_i
!    (smg,ssm)_force ..: -<Q_Z>_i,i
!    (smg,ssm)_dis ....: +<(Q_Z)_i Z_,i>
!    (smg,ssm)_trans ..: +<(Q_Z)_i,i Z>

  DO j = 1,jmax
     wrk1d(j,1) = AVG_IK(imax, jmax, kmax, j, z1, dx, dz, area)
  ENDDO
  CALL PARTIAL_Y(imode_fdm, i1, jmax, i1, j1bc, dy, wrk1d(1,1), &
       wrk1d(1,2), i0, i0, wrk1d(1,3), wrk2d, wrk3d)
  DO j = 1,jmax
     den = AVG_IK(imax, jmax, kmax, j, rho, dx, dz, area)
     DO k = 1,kmax
        DO i = 1,imax
           wrk3d(i,1,k) = rho(i,j,k)*z1(i,j,k)
        ENDDO
     ENDDO
     reyZ = wrk1d(j,1)
     dreyZdy = wrk1d(j,2)
     favZ = AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)/den

! Reynolds stresses, <Q_Z>_i/<\rho>
     tot_RZx(j) = (smg_qx(j)+ssm_qx(j))/den
     tot_RZy(j) = (smg_qy(j)+ssm_qy(j))/den
     tot_RZz(j) = (smg_qz(j)+ssm_qz(j))/den

! Force, -<Q_Z>_i,i
     tot_force(j) = smg_force(j)+&
          ssm_force(j)

! Dissipation (-), 2<(Q_Z)'_i Z''_,i>/<\rho> 
     tot_dis(j) = C_2_R*(smg_dis(j)+&
          ssm_dis(j)-&
          (smg_qy(j)+ssm_qy(j))*dreyZdy)/&
          den

! Transport (-), -2<(Q_Z)'_i Z''>,i/<\rho> (Careful w sign tot_force)
     tot_trans(j) =-tot_dis(j)-C_2_R*&
          (smg_trans(j)+&
          ssm_trans(j)+&
          tot_force(j)*reyZ)/den

! Mean, -2<Z''><Q_Z>_i,i/<\rho>
     tot_mean(j) = C_2_R*(reyZ-favZ)*&
          tot_force(j)/den
  ENDDO

! #####################
! # Global quantities #
! #####################
#define rR(j)    wrk1d(j,1)
  IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN

! -------------------------------------------------------------------
! Based on delta_rho 
! -------------------------------------------------------------------
     IF ( ABS(delta_rho) .GT. C_SMALL_R ) THEN
        DO j = 1,jmax
           rR(j) = AVG_IK(imax, jmax, kmax, j, rho, dx, dz, area)
        ENDDO
        dummy = mean_rho + (C_05_R-C_1EM2_R)*delta_rho
        delta_hb01 = LOWER_THRESHOLD(jmax, dummy, rR(1), y)
        dummy = mean_rho - (C_05_R-C_1EM2_R)*delta_rho
        delta_ht01 = UPPER_THRESHOLD(jmax, dummy, rR(1), y)

        delta_hb01 = (y(1) + scaley*ycoor_rho) - delta_hb01  
        delta_ht01 = delta_ht01 - (y(1) + scaley*ycoor_rho)
        delta_h01 = delta_ht01 - delta_hb01

     ELSE
        delta_h01 = C_1_R

     ENDIF

! -------------------------------------------------------------------
! Based on delta_u 
! -------------------------------------------------------------------
! Vorticity thickness 
     IF ( ABS(delta_u) .GT. C_SMALL_R ) THEN
#define fU(j)    wrk1d(j,2)
#define fU_y(j)  wrk1d(j,3)
        DO j = 1,jmax
           rR(j) = AVG_IK(imax, jmax, kmax, j, rho, dx, dz, area)
        ENDDO
        
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = rho(i,j,k)*u(i,j,k)
              ENDDO
           ENDDO
           fU(j) = AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)/rR(j)
        ENDDO
        
        CALL PARTIAL_Y(imode_fdm, i1, jmax, i1, j1bc, dy, fU(1), &
             fU_y(1), i0, i0, wrk1d(1,4), wrk2d, wrk3d)
        delta_w = delta_u/MAXVAL(ABS(fU_y(1:jmax)))

        ELSE
           delta_w = C_1_R

        ENDIF

#undef fU
#undef fU_y

! -----------------------------------------------------------
!                                                     Jet
! -----------------------------------------------------------
  ELSE IF (imode_flow .EQ. DNS_FLOW_JET) THEN
! not developed yet

  ENDIF

#undef rR

! ################################
! # Final writing to TkStat file #
! ################################
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     DO j = 1,jmax
        IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN
           SimVar1 = (y(j)-scaley*C_05_R-y(1))/delta_w
           SimVar2 = (y(j)-scaley*ycoor_rho-y(1))/delta_h01
           WRITE(i23,1300) i1, j, y(j), SimVar1, SimVar2, (mean2d(j,l1),l1=1,L_AVGMAX)

        ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN ! for compatibility of format
           WRITE(i23,1300) i1, j, y(j), y(j), y(j), (mean2d(j,l1),l1=1,L_AVGMAX)

        ENDIF
     ENDDO
#ifdef PARALLEL
  ENDIF
#endif

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CLOSE(23)
#ifdef PARALLEL
  ENDIF
#endif

1300 FORMAT(I5,1X,I5,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))

  RETURN
END SUBROUTINE LES_AVG_SCAL_TEMPORAL_LAYER

