#include "types.h"
#include "dns_const.h"

SUBROUTINE LES_DELTAF3D(imode_sim, i1bc,j1bc,k1bc, imax,jmax, kmax_total, &
     ijkmax, nx,ny,nz, scalex,scaley,scalez, x,y,z, delf, wrk1d)

#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

  TINTEGER imode_sim, imax, jmax, kmax_total, ijkmax
  TINTEGER i1bc, j1bc, k1bc
  TINTEGER nx, ny, nz
  TREAL scalex, scaley, scalez
  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL delf(*)
  TREAL wrk1d(ijkmax,3)

  TINTEGER i, j, ij 
  TINTEGER i1
  TREAL a

  i1 = 1

  IF ( kmax_total .GT. 1 ) THEN
     a = C_1_R/C_3_R
     wrk1d(1,3) = z(1+nz)-z(1)
  ELSE
     a = C_05_R
     wrk1d(1,3) = C_1_R
  ENDIF

  IF ( imode_sim .EQ. DNS_MODE_TEMPORAL ) THEN
     CALL LES_DELTAF1D(j1bc, jmax, ny, scaley, y, wrk1d(1,2))
     wrk1d(1,1) = x(1+nx)-x(1)
     DO j = 1,jmax
        delf(j) = (wrk1d(1,1)*wrk1d(j,2)*wrk1d(1,3))**a
     ENDDO

  ELSE IF ( imode_sim .EQ. DNS_MODE_SPATIAL ) THEN
     CALL LES_DELTAF1D(i1bc, imax, nx, scalex, x, wrk1d(1,1))
     CALL LES_DELTAF1D(j1bc, jmax, ny, scaley, y, wrk1d(1,2))
     DO j = 1,jmax
        DO i = 1,imax
           ij = (j-1)*imax + i
           delf(ij) = (wrk1d(i,1)*wrk1d(j,2)*wrk1d(1,3))**a
        ENDDO
     ENDDO

  ELSE
     CALL LES_DELTAF1D(i1bc, imax, nx, scalex, x, wrk1d(1,1))
     CALL LES_DELTAF1D(j1bc, jmax, ny, scaley, y, wrk1d(1,2))
     CALL LES_DELTAF1D(k1bc, kmax_total, nz, scalez, z, wrk1d(1,3))

  ENDIF

#ifdef _DEBUG
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     IF      ( imode_sim .EQ. DNS_MODE_TEMPORAL ) THEN
        CALL IO_WRITE_ASCII_FIELD('deltaf.dat', jmax, i1, i1, delf)
     ELSE IF ( imode_sim .EQ. DNS_MODE_SPATIAL ) THEN
        CALL IO_WRITE_ASCII_FIELD('deltaf.dat', imax, jmax, i1, delf)
     ENDIF
#ifdef PARALLEL
  ENDIF
#endif
#endif

  RETURN
END SUBROUTINE LES_DELTAF3D
