!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE LES_RHS(istep, x,y,z,dx,dy,dz, q,h, z1,zh1, txc, vaux, wrk1d,wrk2d,wrk3d)

#include "types.h"
#include "dns_const.h"
#include "avgij_map.h"
#include "les_avgij_map.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER statflg, istep

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax*jmax*kmax,*) :: q, h, z1, zh1, txc
  TREAL, DIMENSION(*)                :: wrk1d, wrk2d, wrk3d, vaux

! -----------------------------------------------------------------------
  TINTEGER ssmflg, ip_mean_les, ip_chi

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_RHS' )
#endif

  ip_chi      = inb_txc - 1 ! only used in chem active
  ip_mean_les = vindex(VA_MEAN_WRK)+(MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*inb_scal)*nstatavg*jmax 

  IF ( imode_sim .EQ. DNS_MODE_SPATIAL .AND. frunstat .EQ. 1 .AND.&
       itime .EQ. iupdate_stat .AND. istep .EQ. 1 ) THEN
     statflg = 1
  ELSE
     statflg = 0
  ENDIF

  IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) txc(:,ip_chi) = C_0_R

! #######################################################################
! Nonreactive terms
! #######################################################################
  IF ( iles_type_tran .EQ. LES_TRAN_NONE ) THEN 
     ssmflg = 0
  ELSE
     ssmflg = 1
  ENDIF

! -------------------------------------------------------------------
!                                                       Transport
! -------------------------------------------------------------------
  IF ( iles_type_tran .EQ. LES_TRAN_NONE) THEN
  ELSE 
! This subroutine demands txc_size=16 (plus wrk3d)
     CALL SGS_RECTN(statflg, x,y,z,dx,dy,dz, q,z1, &
          txc(1,1),txc(1,2),txc(1,3),txc(1,4), h(1,1),h(1,2),h(1,3),h(1,4),zh1, vaux(ip_mean_les), &
          txc(1,5),txc(1,6),txc(1,7),txc(1,8),txc(1,9),txc(1,10),txc(1,11), &
          txc(1,12), txc(1,ip_chi), wrk1d,wrk2d,wrk3d, &
          vaux(vindex(VA_LES_FLT0X)), vaux(vindex(VA_LES_FLT0Y)), vaux(vindex(VA_LES_FLT0Z)),  &
          vaux(vindex(VA_LES_ARM_UF)),vaux(vindex(VA_LES_ARM_PF)),vaux(vindex(VA_LES_ARM_ZF)))
  ENDIF

! -------------------------------------------------------------------
!                                                  Regularization
! -------------------------------------------------------------------
  IF ( iles_type_regu .EQ. LES_REGU_NONE) THEN
  ELSE IF ( iles_type_regu .EQ. LES_REGU_SMGSTA       .OR. &
            iles_type_regu .EQ. LES_REGU_SMGSTARMS  ) THEN
     CALL SGS_STA(x,dx,dy,dz, q,z1, h(1,1),h(1,2),h(1,3),h(1,4),zh1, vaux(ip_mean_les), &
          txc, txc(1,ip_chi), wrk1d,wrk2d)

  ELSE IF ( iles_type_regu .EQ. LES_REGU_SMGDYN       .OR. &
            iles_type_regu .EQ. LES_REGU_SMGDYNRMS  ) THEN
! This subroutine demands txc_size=10 (plus wrk3d) for ssmflg = 0
! This subroutine demands txc_size=13 (plus wrk3d) for ssmflg = 1
     CALL SGS_DYN(istep, statflg, ssmflg, x,y,z,dx,dy,dz, q,z1, &
          txc(1,1),txc(1,2),txc(1,3),txc(1,4), h(1,1),h(1,2),h(1,3),h(1,4),zh1, txc(1,5), &
          vaux(ip_mean_les), txc(1,ip_chi), wrk1d,wrk2d,wrk3d,&
          vaux(vindex(VA_LES_FLT0X)),  vaux(vindex(VA_LES_FLT0Y)), vaux(vindex(VA_LES_FLT0Z)), &
          vaux(vindex(VA_LES_FLT1X)),  vaux(vindex(VA_LES_FLT1Y)), vaux(vindex(VA_LES_FLT1Z)), &
          vaux(vindex(VA_LES_FLT2X)),  vaux(vindex(VA_LES_FLT2Y)), vaux(vindex(VA_LES_FLT2Z)), &
          vaux(vindex(VA_LES_ARM_UF)), vaux(vindex(VA_LES_ARM_UH)), &
          vaux(vindex(VA_LES_ARM_PF)), vaux(vindex(VA_LES_ARM_PH)), &
          vaux(vindex(VA_LES_ARM_ZF)), vaux(vindex(VA_LES_ARM_ZH)), &
          vaux(vindex(VA_LES_SGSCOEFU)), vaux(vindex(VA_LES_SGSCOEFE)), vaux(vindex(VA_LES_SGSCOEFZ)))
  ENDIF

! -------------------------------------------------------------------
!                                             Pressure-dilatation
! -------------------------------------------------------------------
!     CALL SGS_DYN_PDIL(istep, statflg, x, y, z, dx, dy, dz, rho, u, v, w, p, h4, vaux(ip_mean_les),
! $        txc(1,1,1,1), txc(1,1,1,2), txc(1,1,1,3), 
! $        txc(1,1,1,4), txc(1,1,1,5), txc(1,1,1,6), 
! $        txc(1,1,1,7), txc(1,1,1,8), txc(1,1,1,9),
! $        wrk1d, wrk2d, wrk3d,
! $        vaux(vindex(VA_LES_FLT0X)), vaux(vindex(VA_LES_FLT0Y)), vaux(vindex(VA_LES_FLT0Z)), 
! $        vaux(vindex(VA_LES_FLT1X)), vaux(vindex(VA_LES_FLT1Y)), vaux(vindex(VA_LES_FLT1Z)),
! $        vaux(vindex(VA_LES_FLT2X)), vaux(vindex(VA_LES_FLT2Y)), vaux(vindex(VA_LES_FLT2Z)))


! #######################################################################
! Reactive terms 
! #######################################################################
! No model
!  IF ( iles_type_chem .EQ. LES_CHEM_NONE ) THEN
!! ARM model for Burke-Schumann (just statistics)
!  ELSE IF ( iles_type_chem .EQ. LES_CHEM_STAT_BS ) THEN
!     CALL SGS_ARM_CHEM_BS(istep, statflg, x, y, z, dx, dy, dz, &
!          rho, z1, &
!          txc(1,1,1,1), txc(1,1,1,2), txc(1,1,1,3), txc(1,1,1,4), &
!          txc(1,1,1,5), txc(1,1,1,6), vaux(vindex(VA_MEAN_WRK)),&
!          vaux(vindex(VA_MEAN_WRK)+&
!          (MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*(inb_scal-1))*&
!          nstatavg*jmax),&
!          vaux(ip_mean_les+LA_MOMENTUM_SIZE*jmax*nstatavg),&
!          vaux(ip_mean_les+(LA_MOMENTUM_SIZE+&
!          LS_SCALAR_SIZE*inb_scal)*jmax*nstatavg),&
!          wrk1d, wrk2d, txc(1,1,1,7), &
!          vaux(vindex(VA_ARM_WRK)), vaux(vindex(VA_ARM_C0)),&
!          vaux(vindex(VA_LES_FLT0X)), vaux(vindex(VA_LES_FLT0Y)), &
!          vaux(vindex(VA_LES_FLT0Z)), messageles)
!! ARM model for passive scalar (just statistics)
!  ELSE IF ( iles_type_chem .EQ. LES_CHEM_STAT_PS ) THEN
!     CALL SGS_ARM_CHEM_PS(istep, statflg, x, y, z, dx, dy, dz, &
!          rho, z1, &
!          txc(1,1,1,1), txc(1,1,1,2), txc(1,1,1,3), txc(1,1,1,4), &
!          txc(1,1,1,5), txc(1,1,1,6), vaux(vindex(VA_MEAN_WRK)),&
!          vaux(vindex(VA_MEAN_WRK)+&
!          (MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*(inb_scal-1))*&
!          nstatavg*jmax),&
!          vaux(ip_mean_les+LA_MOMENTUM_SIZE*jmax*nstatavg),&
!          vaux(ip_mean_les+(LA_MOMENTUM_SIZE+&
!          LS_SCALAR_SIZE*inb_scal)*jmax*nstatavg),&
!          wrk1d, wrk2d, txc(1,1,1,7), &
!          vaux(vindex(VA_ARM_WRK)), vaux(vindex(VA_ARM_C0)),&
!          vaux(vindex(VA_LES_FLT0X)), vaux(vindex(VA_LES_FLT0Y)), &
!          vaux(vindex(VA_LES_FLT0Z)), messageles)
!! Burke-Schumann (active)
!  ELSE IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
!     CALL SGS_CHEM_QUASIBS(x, y, z, dx, dy, dz, rho, &
!          u, p, z1, h4, txc(1,1,1,1), txc(1,1,1,2), txc(1,1,1,3), &
!          txc(1,1,1,ip_chi), wrk1d, wrk2d, wrk3d,&
!          vaux(vindex(VA_LES_FLT0X)), vaux(vindex(VA_LES_FLT0Y)), &
!          vaux(vindex(VA_LES_FLT0Z)),&
!          vaux(vindex(VA_LES_FDF_BS)))
!  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LES_RHS' )
#endif

  RETURN
END SUBROUTINE LES_RHS
