!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2001/11/25 - J.P. Mellado
!#              Created
!# 2008/11/17 - J.P.Mellado
!#              Recovered
!#
!########################################################################
!# DESCRIPTION
!#
!# Runnning statistics for LES terms. Careful with passed array txc.
!#
!########################################################################
!# ARGUMENTS 
!# 
!########################################################################
SUBROUTINE LES_SAVE_AVGIJ(iterms, dx,dy,dz, u,v,w, txc, mean1d_les, wrk1d,wrk2d,wrk3d)

#include "types.h"
#include "les_avgij_map.h"
#include "dns_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

  TINTEGER iterms

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL u(imax,jmax,kmax)
  TREAL v(imax,jmax,kmax)
  TREAL w(imax,jmax,kmax)
  TREAL txc(imax,jmax,kmax,*)

  TREAL mean1d_les(nstatavg,jmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)

  TINTEGER i, j, n, ij, ij2, NNstat
  TREAL r13, r23

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_SAVE_AVGIJ' )
#endif

  r13 = C_1_R/C_3_R
  r23 = C_2_R/C_3_R
  NNstat = nstatavg*jmax

  IF ( imode_sim .NE. DNS_MODE_SPATIAL ) RETURN

! #######################################################################
! Smagorinsky
! #######################################################################
! Dynamic coefficient and eddy viscosity
! wrk2d(1) - 2*Cs
! txc(1)   - 2*mu_s

  IF ( iterms .EQ. 1 ) THEN

     DO j = 1,jmax
        DO n = 1,nstatavg
           i = statavg(n)
           ij = (j-1)*imax + i
           ij2 = (j-1)*nstatavg + n
           LA_SMG_CS_U(ij2) = LA_SMG_CS_U(ij2) + kmax_total*wrk2d(ij,1)*C_05_R
        ENDDO
     ENDDO

     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )

     DO j = 1,NNstat
        LA_SMG_RNU(j)=LA_SMG_RNU(j)+C_05_R*wrk2d(j,2)
     ENDDO

  ENDIF

! tmp4 is tau_xy, tmp1 is Sxy
  IF ( iterms .EQ. 2 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TAUxy(j)=LA_SMG_TAUxy(j)+wrk2d(j,1)
        LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+C_2_R*wrk2d(j,2)
     ENDDO
  ENDIF

! tmp4 is tau_xz, tmp2 is Sxz
  IF ( iterms .EQ. 3 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TAUxz(j)=LA_SMG_TAUxz(j)+wrk2d(j,1)
        LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+C_2_R*wrk2d(j,2)
     ENDDO
  ENDIF

! tmp4 is tau_yz, tmp3 is Syz
  IF ( iterms .EQ. 4 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,3), txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )

     DO j = 1,NNstat
        LA_SMG_TAUyz(j)=LA_SMG_TAUyz(j)+wrk2d(j,1)
        LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+C_2_R*wrk2d(j,2)
     ENDDO

  ENDIF

! tmp1 is dudx, tmp2 is dvdy, tmp3 is dwdz,  tmp4 is tau_xx
  IF ( iterms .EQ. 5 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TAUxx(j)=LA_SMG_TAUxx(j)+wrk2d(j,1)
     ENDDO
     IF ( sgs_devsmag .EQ. 1 ) THEN
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,1), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,2), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,3), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,3), wrk2d(1,5) )
        DO j = 1,NNstat
           LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+r23*wrk2d(j,1)-r13*(wrk2d(j,2)+wrk2d(j,3))
        ENDDO
     ELSE
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,1), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
        DO j = 1,NNstat
           LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+wrk2d(j,1)
        ENDDO
     ENDIF
  ENDIF

! tmp1 is dudx, tmp2 is dvdy, tmp3 is dwdz, tmp4 is tau_yy
  IF ( iterms .EQ. 6 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TAUyy(j)=LA_SMG_TAUyy(j)+wrk2d(j,1)
     ENDDO
     IF ( sgs_devsmag .EQ. 1 ) THEN
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,1), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,2), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,3), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,3), wrk2d(1,5) )
        DO j = 1,NNstat
           LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+r23*wrk2d(j,2)-r13*(wrk2d(j,1)+wrk2d(j,3))
        ENDDO
     ELSE
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,2), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
        DO j = 1,NNstat
           LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+wrk2d(j,1)
        ENDDO
     ENDIF
  ENDIF

! tmp1 is dudx, tmp2 is dvdy, tmp3 is dwdz, tmp4 is tau_zz
  IF ( iterms .EQ. 7 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TAUzz(j)=LA_SMG_TAUzz(j)+wrk2d(j,1)
     ENDDO
     IF ( sgs_devsmag .EQ. 1 ) THEN
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,1), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,2), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,3), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,3), wrk2d(1,5) )
        DO j = 1,NNstat
           LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+r23*wrk2d(j,3)-r13*(wrk2d(j,1)+wrk2d(j,2))
        ENDDO
     ELSE
        CALL REDUCE_MUL(imax,jmax,kmax, txc(1,1,1,3), txc(1,1,1,4), nstatavg, statavg, wrk3d )
        CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
        DO j = 1,NNstat
           LA_SMG_EPS_U(j)=LA_SMG_EPS_U(j)+wrk2d(j,1)
        ENDDO
     ENDIF
  ENDIF

! #######################################################################
! Scale similarity part
! #######################################################################
! txc1 tau_xx, txc2 dudx
  IF ( iterms .EQ. 8 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TAUxx(j)=LA_SSM_TAUxx(j)+wrk2d(j,1)
        LA_SSM_EPS_U(j)=LA_SSM_EPS_U(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 tau_yy, txc2 dvdy
  IF ( iterms .EQ. 9 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TAUyy(j)=LA_SSM_TAUyy(j)+wrk2d(j,1)
        LA_SSM_EPS_U(j)=LA_SSM_EPS_U(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 tau_zz, txc2 dwdz
  IF ( iterms .EQ. 10 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TAUzz(j)=LA_SSM_TAUzz(j)+wrk2d(j,1)
        LA_SSM_EPS_U(j)=LA_SSM_EPS_U(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 tau_xy, txc2 dvdx, txc3 dudy
  IF ( iterms .EQ. 11 ) THEN
     CALL REDUCE_SUM( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,3), nstatavg, statavg, wrk3d )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, txc(1,1,1,2) )
     DO ij = 1,NNstat*kmax
        wrk3d(ij,1,1) = wrk3d(ij,1,1)*txc(ij,1,1,2)
     ENDDO
     CALL SUM_K_V( NNstat, kmax, txc(1,1,1,2), wrk2d(1,1), wrk2d(1,5) )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TAUxy(j)=LA_SSM_TAUxy(j)+wrk2d(j,1)
        LA_SSM_EPS_U(j)=LA_SSM_EPS_U(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 tau_xz, txc2 dwdx, txc3 dudz
  IF ( iterms .EQ. 12 ) THEN
     CALL REDUCE_SUM( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,3), nstatavg, statavg, wrk3d )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, txc(1,1,1,2) )
     DO ij = 1,NNstat*kmax
        wrk3d(ij,1,1) = wrk3d(ij,1,1)*txc(ij,1,1,2)
     ENDDO
     CALL SUM_K_V( NNstat, kmax, txc(1,1,1,2), wrk2d(1,1), wrk2d(1,5) )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TAUxz(j)=LA_SSM_TAUxz(j)+wrk2d(j,1)
        LA_SSM_EPS_U(j)=LA_SSM_EPS_U(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 tau_yz, txc2 dvdz, txc3 dwdy
  IF ( iterms .EQ. 13 ) THEN
     CALL REDUCE_SUM( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,3), nstatavg, statavg, wrk3d )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, txc(1,1,1,2) )
     DO ij = 1,NNstat*kmax
        wrk3d(ij,1,1) = wrk3d(ij,1,1)*txc(ij,1,1,2)
     ENDDO
     CALL SUM_K_V( NNstat, kmax, txc(1,1,1,2), wrk2d(1,1), wrk2d(1,5) )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TAUyz(j)=LA_SSM_TAUyz(j)+wrk2d(j,1)
        LA_SSM_EPS_U(j)=LA_SSM_EPS_U(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! #######################################################################
! Terms new for v1.1
! #######################################################################

! txc1 is d/dy(tauxy)    
  IF ( iterms .EQ. 14 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), u, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_FORCE_U(j)=LA_SMG_FORCE_U(j)-&
             wrk2d(j,1)
        LA_SMG_TRA_U(j)=LA_SMG_TRA_U(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 is d/dx(tauxy)    
  IF ( iterms .EQ. 15 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), v, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_FORCE_V(j)=LA_SMG_FORCE_V(j)-&
             wrk2d(j,1)
        LA_SMG_TRA_U(j)=LA_SMG_TRA_U(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! txc5 is d/dx(tauxx)    
  IF ( iterms .EQ. 16 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,5), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,5), u, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_FORCE_U(j)=LA_SMG_FORCE_U(j)-&
             wrk2d(j,1)
        LA_SMG_CON_Ux(j)=LA_SMG_CON_Ux(j)+&
             wrk2d(j,1)
        LA_SMG_TRA_U(j)=LA_SMG_TRA_U(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! txc5 is d/dy(tauyy)    
  IF ( iterms .EQ. 17 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,5), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,5), v, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_FORCE_V(j)=LA_SMG_FORCE_V(j)-&
             wrk2d(j,1)
        LA_SMG_CON_Uy(j)=LA_SMG_CON_Uy(j)+&
             wrk2d(j,1)
        LA_SMG_TRA_U(j)=LA_SMG_TRA_U(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 is d/dz(tauxz) and txc2 is d/dx(tauzx)
  IF ( iterms .EQ. 18 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), u, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), w, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TRA_U(j)=LA_SMG_TRA_U(j)+&
             wrk2d(j,1)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 is d/dz(tauyz) and txc2 is d/dy(tauzy)
  IF ( iterms .EQ. 19 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), v, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), w, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TRA_U(j)=LA_SMG_TRA_U(j)+&
             wrk2d(j,1)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc5 is d/dz(tauzz)    
  IF ( iterms .EQ. 20 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,5), w, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TRA_U(j)=LA_SMG_TRA_U(j)+&
             wrk2d(j,1)
     ENDDO
  ENDIF

! ############
! # SSM part #
! ############

! txc1 is d/dx(tauxx)    
  IF ( iterms .EQ. 21 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), u, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_FORCE_U(j)=LA_SSM_FORCE_U(j)-&
             wrk2d(j,1)
        LA_SSM_CON_Ux(j)=LA_SSM_CON_Ux(j)+&
             wrk2d(j,1)
        LA_SSM_TRA_U(j)=LA_SSM_TRA_U(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 is d/dy(tauyy)    
  IF ( iterms .EQ. 22 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), v, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_FORCE_V(j)=LA_SSM_FORCE_V(j)-&
             wrk2d(j,1)
        LA_SSM_CON_Uy(j)=LA_SSM_CON_Uy(j)+&
             wrk2d(j,1)
        LA_SSM_TRA_U(j)=LA_SSM_TRA_U(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 is d/dz(tauzz)    
  IF ( iterms .EQ. 23 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), w, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TRA_U(j)=LA_SSM_TRA_U(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 is d/dx(tauyx) and txc2 is d/dy(tauxy)    
  IF ( iterms .EQ. 24 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,2), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), v, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,3), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), u, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,4), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_FORCE_V(j)=LA_SSM_FORCE_V(j)-&
             wrk2d(j,1)
        LA_SSM_FORCE_U(j)=LA_SSM_FORCE_U(j)-&
             wrk2d(j,2)
        LA_SSM_TRA_U(j)=LA_SSM_TRA_U(j)+&
             wrk2d(j,3)+wrk2d(j,4)
     ENDDO
  ENDIF

! txc1 is d/dz(tauxz) and txc2 is d/dx(tauzx)    
  IF ( iterms .EQ. 25 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), u, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), w, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TRA_U(j)=LA_SSM_TRA_U(j)+&
             wrk2d(j,1)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 is d/dz(tauyz) and txc2 is d/dy(tauzy)    
  IF ( iterms .EQ. 26 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), v, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), w, &
          nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TRA_U(j)=LA_SSM_TRA_U(j)+&
             wrk2d(j,1)+wrk2d(j,2)
     ENDDO
  ENDIF

! #######################################################################
! Terms new for v1.2
! #######################################################################

! txc5 is d/dy(tauxx)    
  IF ( iterms .EQ. 27 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,5), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_CON_Uy(j)=LA_SMG_CON_Uy(j)+&
             wrk2d(j,1)
     ENDDO
  ENDIF

! txc5 is d/dx(tauyy)    
  IF ( iterms .EQ. 28 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,5), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_CON_Ux(j)=LA_SMG_CON_Ux(j)+&
             wrk2d(j,1)
     ENDDO
  ENDIF

! txc1 is d/dx(tauzz), txc2 is d/dy(tauzz)    
  IF ( iterms .EQ. 29 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,2), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_CON_Ux(j)=LA_SMG_CON_Ux(j)+&
             wrk2d(j,1)
        LA_SMG_CON_Uy(j)=LA_SMG_CON_Uy(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! ############
! # SSM part #
! ############

! txc1 is d/dy(tauxx)    
  IF ( iterms .EQ. 30 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_CON_Uy(j)=LA_SSM_CON_Uy(j)+&
             wrk2d(j,1)
     ENDDO
  ENDIF


! txc1 is d/dx(tauyy)    
  IF ( iterms .EQ. 31 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_CON_Ux(j)=LA_SSM_CON_Ux(j)+&
             wrk2d(j,1)
     ENDDO
  ENDIF

! txc1 is d/dx(tauzz)
  IF ( iterms .EQ. 32 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_CON_Ux(j)=LA_SSM_CON_Ux(j)+&
             wrk2d(j,1)
     ENDDO
  ENDIF

! txc1 is d/dy(tauzz)    
  IF ( iterms .EQ. 33 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_CON_Uy(j)=LA_SSM_CON_Uy(j)+&
             wrk2d(j,1)
     ENDDO
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LES_SAVE_AVGIJ' )
#endif

  RETURN
END SUBROUTINE LES_SAVE_AVGIJ
