!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2002/02/25 - J.P. Mellado
!#              Created
!# 2008/11/17 - J.P.Mellado
!#              Addapted for the energy terms
!#
!########################################################################
!# DESCRIPTION
!#
!# Runnning statistics for LES terms. Careful with passed array txc.
!# Originally written for the pressure, now taken for the energy.
!#
!########################################################################
!# ARGUMENTS 
!# 
!########################################################################
SUBROUTINE LES_SAVE_AVGPIJ(iterms, dx,dy,dz, e, txc, mean1d_les, wrk1d,wrk2d,wrk3d)
  
#include "types.h"
#include "les_avgij_map.h"
#include "dns_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

  TINTEGER iterms

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL e(imax,jmax,kmax)
  TREAL txc(imax,jmax,kmax,*)

  TREAL mean1d_les(nstatavg,jmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)

! -----------------------------------------------------------------------
  TINTEGER i, j, n, ij, ij2, NNstat

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_SAVE_AVGPIJ' )
#endif

  NNstat = nstatavg*jmax

  IF ( imode_sim .NE. DNS_MODE_SPATIAL ) RETURN

! #######################################################################
! Smagorinsky
! #######################################################################
! Dynamic coefficient and eddy viscosity
! wrk2d(1) - CsP
! txc(1)   - rho*alpha_s
  IF ( iterms .EQ. 1 ) THEN
     DO j = 1,jmax
        DO n = 1,nstatavg
           i = statavg(n)
           ij = (j-1)*imax + i
           ij2 = (j-1)*nstatavg + n
           LA_SMG_CS_P(ij2) =  LA_SMG_CS_P(ij2) + kmax_total*wrk2d(ij,1)
        ENDDO
     ENDDO

     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_RALPHA(j)=LA_SMG_RALPHA(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! tmp2 dedx, tmp1 -q_x
  IF ( iterms .EQ. 2 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_QPx(j)  =LA_SMG_QPx(j)   - wrk2d(j,1)
        LA_SMG_EPS_P(j)=LA_SMG_EPS_P(j) - wrk2d(j,2)
     ENDDO
  ENDIF

! tmp2 dedy, tmp1 -q_y
  IF ( iterms .EQ. 3 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_QPy(j)  =LA_SMG_QPy(j)   - wrk2d(j,1)
        LA_SMG_EPS_P(j)=LA_SMG_EPS_P(j) - wrk2d(j,2)
     ENDDO
  ENDIF

! tmp2 dedz, tmp1 -q_z
  IF ( iterms .EQ. 4 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_QPz(j)   =LA_SMG_QPz(j)   - wrk2d(j,1)
        LA_SMG_EPS_P(j) =LA_SMG_EPS_P(j) - wrk2d(j,2)
     ENDDO
  ENDIF

! #######################################################################
! Scale similarity part
! #######################################################################
! txc1 q_x, txc2 is de/dx
  IF ( iterms .EQ. 5 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1),nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_QPx(j)  =LA_SSM_QPx(j)  +wrk2d(j,1)
        LA_SSM_EPS_P(j)=LA_SSM_EPS_P(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 q_y, txc2 is de/dy
  IF ( iterms .EQ. 6 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_QPy(j)  =LA_SSM_QPy(j)  +wrk2d(j,1)
        LA_SSM_EPS_P(j)=LA_SSM_EPS_P(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 q_z, txc is de/dz
  IF ( iterms .EQ. 7 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_QPz(j)  =LA_SSM_QPz(j)  +wrk2d(j,1)
        LA_SSM_EPS_P(j)=LA_SSM_EPS_P(j)+wrk2d(j,2)
     ENDDO
  ENDIF

! #######################################################################
! Terms new for v1.1
! #######################################################################
! tmp2 -d/dx(q_x) or -d/dy(q_y) 
  IF ( iterms .EQ. 8 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, e, txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_FORCE_P(j)=LA_SMG_FORCE_P(j) + wrk2d(j,1)
        LA_SMG_TRA_P(j)  =LA_SMG_TRA_P(j)   - wrk2d(j,2)
     ENDDO
  ENDIF

! tmp2 -d/dz(q_z)
  IF ( iterms .EQ. 9 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, e, txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SMG_TRA_P(j)=LA_SMG_TRA_P(j)- wrk2d(j,2)
     ENDDO
  ENDIF

! Scale similarity part
! ---------------------

! tmp1 d/dx(q_x) or d/dy(q_y) 
  IF ( iterms .EQ. 10 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg,  statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, e, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_FORCE_P(j)=LA_SSM_FORCE_P(j)- wrk2d(j,1)
        LA_SSM_TRA_P(j)=LA_SSM_TRA_P(j)    + wrk2d(j,2)
     ENDDO
  ENDIF

! tmp1 d/dz(q_z)
  IF ( iterms .EQ. 11 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, e, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LA_SSM_TRA_P(j)=LA_SSM_TRA_P(j)+ wrk2d(j,2)
     ENDDO
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LES_SAVE_AVGPIJ' )
#endif
  
  RETURN
END SUBROUTINE LES_SAVE_AVGPIJ
