!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2001/11/25 - J.P. Mellado
!#              Created
!# 2008/11/17 - J.P.Mellado
!#              Addapted for the energy terms
!#
!########################################################################
!# DESCRIPTION
!#
!# Runnning statistics for LES terms. Careful with passed array txc.
!#
!########################################################################
!# ARGUMENTS 
!# 
!########################################################################
SUBROUTINE LES_SAVE_SCBDGIJ(iterms, dx,dy,dz, z1, txc, mean1d_les_sc, wrk1d,wrk2d,wrk3d )

#include "types.h"
#include "les_avgij_map.h"
#include "dns_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

  TINTEGER iterms

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL z1(imax,jmax,kmax)
  TREAL txc(imax,jmax,kmax,*)

  TREAL mean1d_les_sc(nstatavg,jmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)

! -----------------------------------------------------------------------
  TINTEGER i, j, n, ij, ij2, NNstat

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_SAVE_SCBDGIJ' )
#endif

  NNstat = nstatavg*jmax

  IF ( imode_sim .NE. DNS_MODE_SPATIAL ) RETURN

! #######################################################################
! Smagorinsky
! #######################################################################
! wrk2d(1) - CsZ
! txc(1)   - rho*D_s
  IF ( iterms .EQ. 1 ) THEN
     DO j = 1,jmax
        DO n = 1,nstatavg
           i = statavg(n)
           ij = (j-1)*imax + i
           ij2 = (j-1)*nstatavg + n
           LS_SMG_CS_Z(ij2) =  LS_SMG_CS_Z(ij2) + kmax_total*wrk2d(ij,1)
        ENDDO
     ENDDO

     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SMG_RD(j)=LS_SMG_RD(j)+ wrk2d(j,2)
     ENDDO

  ENDIF

! tmp2 dz1dx, tmp1 -q_x
  IF ( iterms .EQ. 2 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SMG_Qx(j)   =LS_SMG_Qx(j)   - wrk2d(j,1)
        LS_SMG_EPS_Z(j)=LS_SMG_EPS_Z(j)- wrk2d(j,2)
     ENDDO
  ENDIF

! tmp2 dz1dy, tmp1 -q_y
  IF ( iterms .EQ. 3 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SMG_Qy(j)   =LS_SMG_Qy(j)   - wrk2d(j,1)
        LS_SMG_EPS_Z(j)=LS_SMG_EPS_Z(j)- wrk2d(j,2)
     ENDDO
  ENDIF

! tmp2 dz1dz, tmp1 -q_z
  IF ( iterms .EQ. 4 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,2), txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SMG_Qz(j)   =LS_SMG_Qz(j)   - wrk2d(j,1)
        LS_SMG_EPS_Z(j)=LS_SMG_EPS_Z(j)- wrk2d(j,2)
     ENDDO
  ENDIF

! #######################################################################
! Scale similarity part
! #######################################################################
! txc1 q_x, txc2 dz1dx
  IF ( iterms .EQ. 5 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SSM_Qx(j)   =LS_SSM_Qx(j)   + wrk2d(j,1)
        LS_SSM_EPS_Z(j)=LS_SSM_EPS_Z(j)+ wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 q_y, txc2 dz1dy
  IF ( iterms .EQ. 6 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SSM_Qy(j)   =LS_SSM_Qy(j)   + wrk2d(j,1)
        LS_SSM_EPS_Z(j)=LS_SSM_EPS_Z(j)+ wrk2d(j,2)
     ENDDO
  ENDIF

! txc1 q_z, txc2 dz1dz
  IF ( iterms .EQ. 7 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, txc(1,1,1,1), txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SSM_Qz(j)   =LS_SSM_Qz(j)   + wrk2d(j,1)
        LS_SSM_EPS_Z(j)=LS_SSM_EPS_Z(j)+ wrk2d(j,2)
     ENDDO
  ENDIF

! #######################################################################
! Terms new for v1.1
! #######################################################################
! tmp2 -d/dx(q_x) or -d/dy(q_y) 
  IF ( iterms .EQ. 8 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, z1, txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SMG_FORCE_Z(j)=LS_SMG_FORCE_Z(j)+ wrk2d(j,1)
        LS_SMG_TRA_Z(j)=LS_SMG_TRA_Z(j)    - wrk2d(j,2)
     ENDDO
  ENDIF

! tmp2 -d/dz(q_z)
  IF ( iterms .EQ. 9 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, z1, txc(1,1,1,2), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SMG_TRA_Z(j)=LS_SMG_TRA_Z(j)- wrk2d(j,2)
     ENDDO
  ENDIF

! Scale similarity part
! ---------------------

! tmp1 d/dx(q_x) or d/dy(q_y) 
  IF ( iterms .EQ. 10 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg,  statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), wrk2d(1,5) )
     CALL REDUCE_MUL( imax, jmax, kmax, z1, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SSM_FORCE_Z(j)=LS_SSM_FORCE_Z(j)- wrk2d(j,1)
        LS_SSM_TRA_Z(j)=LS_SSM_TRA_Z(j)    + wrk2d(j,2)
     ENDDO
  ENDIF

! tmp1 d/dz(q_z)
  IF ( iterms .EQ. 11 ) THEN
     CALL REDUCE_MUL( imax, jmax, kmax, z1, txc(1,1,1,1), nstatavg, statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), wrk2d(1,5) )
     DO j = 1,NNstat
        LS_SSM_TRA_Z(j)=LS_SSM_TRA_Z(j)+ wrk2d(j,2)
     ENDDO
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LES_SAVE_SCBDGIJ' )
#endif

  RETURN
END SUBROUTINE LES_SAVE_SCBDGIJ
