      SUBROUTINE LESSCBDG_V1_0(is, itxc, x, y, mean1d, mean1d_sc, &
           mean1d_les_sc, mean1d_les_ch,&
           stat, wrk1d, wrk2d)

! ###############################################################
! # Postprocessing of averages for the LES quantities
! #
! # mean1d_les - array with the accumalated statistics for the les
! # mrho    - accumulated rho
! # mu      - accumulated rho*u
! # mz1     - accumulated rho*z1
! # stat    - tmp array of size itxc
! #
! # Juan Pedro Mellado, May 2002
! ################################################################

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL
  USE NFILE,     ONLY : efile

      IMPLICIT NONE

      TINTEGER is,itxc
      TREAL x(imax)
      TREAL y(jmax)
      TREAL mean1d(nstatavg,jmax,*)
      TREAL mean1d_sc(nstatavg,jmax,*)
      TREAL mean1d_les_sc(nstatavg,jmax,*)
      TREAL mean1d_les_ch(nstatavg,jmax,*)
      TREAL stat(nstatavg,jmax,*)
      TREAL wrk1d(nstatavg,*)
      TREAL wrk2d(nstatavg,jmax,5)

      TINTEGER i, j, n, NNstat, l1, l2, ll1, ll2
      TREAL U2, DU, fU_05, S2, DS, fS_05  
      TREAL pts, armpts, y_center, SimVar2
      CHARACTER*32 name
      CHARACTER*250 line1
      CHARACTER*500 line2
      TINTEGER i0, i1, i23

#define smg_cs_Z(A,B)    stat(A,B,1)
#define smg_mu_Z(A,B)    stat(A,B,2)
#define smg_qx(A,B)      stat(A,B,3)
#define smg_qy(A,B)      stat(A,B,4) 
#define smg_qz(A,B)      stat(A,B,5)
#define smg_dis(A,B)     stat(A,B,6)

#define ssm_qx(A,B)      stat(A,B,7)
#define ssm_qy(A,B)      stat(A,B,8) 
#define ssm_qz(A,B)      stat(A,B,9)
#define ssm_dis(A,B)     stat(A,B,10)

!#define arm_c0(A,B)      stat(A,B,11)
!#define arm_Z2_sg(A,B)   stat(A,B,12)
!#define arm_Z4_sg(A,B)   stat(A,B,13)
!#define arm_Z8_sg(A,B)   stat(A,B,14)
!#define arm_Z2_r(A,B)    stat(A,B,15)
!#define arm_Z4_r(A,B)    stat(A,B,16)
!#define arm_Z8_r(A,B)    stat(A,B,17)
!#define arm_A1_sg(A,B)   stat(A,B,18)
!#define arm_A1_r(A,B)    stat(A,B,19)
!#define arm_A2(A,B)      stat(A,B,20)

#define L_AVGMAX 20

#ifdef TRACE_ON
      CALL IO_WRITE_ASCII(tfile, 'ENTERING LESSCBDG_V1_0' )
#endif

      i0 = 0
      i1 = 1
      i23 = 23

      IF ( itxc .LT. nstatavg*jmax*L_AVGMAX) THEN
         CALL IO_WRITE_ASCII(efile,'LESSCBDG_V1_0: Not enough space in stat')
         CALL DNS_STOP(LES_ERROR_AVGTMP)
      ENDIF
      if ( nstatavg_points .EQ. 0 ) then
         CALL IO_WRITE_ASCII(efile,'LESSCBDG_V1_0: Zero number of points')
         CALL DNS_STOP(LES_ERROR_AVGPTS)
      ELSE
         pts = C_1_R/M_REAL(nstatavg_points)
      endif

      IF ( iarmavg_pts .GT. 0 ) THEN
         armpts = C_1_R/M_REAL(iarmavg_pts)
      ELSE
         armpts = C_0_R
      ENDIF

      NNstat = jmax*nstatavg
      y_center = y(1) + C_05_R*scaley
      U2 = mean_u - C_05_R*delta_u
      S2 = mean_z - C_05_R*delta_z

      DO j = 1,NNstat

         wrk2d(j,1,1) = MA_RU(j)/MA_R(j)
         wrk2d(j,1,2) = MS_RS(j)/MA_R(j)

         smg_cs_Z(j,1)  = LS_SMG_CS_Z(j)*pts
         smg_mu_Z(j,1)  = LS_SMG_RD(j)*pts
         smg_qx(j,1)    = LS_SMG_Qx(j)*pts
         smg_qy(j,1)    = LS_SMG_Qy(j)*pts
         smg_qz(j,1)    = LS_SMG_Qz(j)*pts
         smg_dis(j,1)   = C_2_R*LS_SMG_EPS_Z(j)*pts
         
         ssm_qx(j,1)    = LS_SSM_Qx(j)*pts
         ssm_qy(j,1)    = LS_SSM_Qy(j)*pts
         ssm_qz(j,1)    = LS_SSM_Qz(j)*pts
         ssm_dis(j,1)   = C_2_R*LS_SSM_EPS_Z(j)*pts

!     arm_c0(j,1)    = LC_ARM_c0(j)*armpts
!     arm_Z2_sg(j,1) = LC_ARM_Z2_SG(j)*armpts
!     arm_Z4_sg(j,1) = LC_ARM_Z4_SG(j)*armpts
!     arm_Z8_sg(j,1) = LC_ARM_Z8_SG(j)*armpts
!     arm_Z2_r(j,1)  = LC_ARM_Z2_R(j)*armpts
!     arm_Z4_r(j,1)  = LC_ARM_Z4_R(j)*armpts
!     arm_Z8_r(j,1)  = LC_ARM_Z8_R(j)*armpts
!     arm_A1_r(j,1)  = LC_ARM_A1_R(j)*armpts
!     arm_A1_sg(j,1) = LC_ARM_A1_SG(j)*armpts
!     arm_A2(j,1)    = LC_ARM_A2(j)*armpts
         
      ENDDO

! ######################################
! # Favre-averaged profiles of U and Z #
! ######################################
    
      IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN

      ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
! Velocity Half width
         DO n = 1,nstatavg
            DU = wrk2d(n,jmax/2,1) - U2
            fU_05 = U2 + C_05_R*DU
            DO j = 1,jmax/2
               IF ( wrk2d(n,j  ,1) .LE. fU_05 .AND. &
                    wrk2d(n,j+1,1) .GT. fU_05 ) THEN
                  wrk1d(n,1) = y(j) + &
                       (fU_05-wrk2d(n,j,1))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,1)-wrk2d(n,j,1))
               ENDIF
            ENDDO
            wrk1d(n,1) = y_center - wrk1d(n,1)
            DU = wrk2d(n,jmax/2+1,1) - U2
            fU_05 = U2 + C_05_R*DU
            DO j = jmax/2+1,jmax
               IF ( wrk2d(n,j  ,1) .GT. fU_05 .AND. &
                    wrk2d(n,j+1,1) .LE. fU_05 ) THEN
                  wrk1d(n,2) = y(j) + &
                       (fU_05-wrk2d(n,j,1))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,1)-wrk2d(n,j,1))
               ENDIF
            ENDDO
            wrk1d(n,2) = wrk1d(n,2) - y_center
            wrk1d(n,1) = C_05_R*(wrk1d(n,1)+wrk1d(n,2))
         ENDDO
! Scalar Half width
         DO n = 1,nstatavg
            DS = wrk2d(n,jmax/2,2) - S2
            fS_05 = S2 + C_05_R*DS
            DO j = 1,jmax/2
               IF ( wrk2d(n,j  ,2) .LE. fS_05 .AND. &
                    wrk2d(n,j+1,2) .GT. fS_05 ) THEN
                  wrk1d(n,2) = y(j) + &
                       (fS_05-wrk2d(n,j,2))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,2)-wrk2d(n,j,2))
               ENDIF
            ENDDO
            wrk1d(n,2) = y_center - wrk1d(n,2)
            DS = wrk2d(n,jmax/2+1,2) - S2
            fS_05 = S2 + C_05_R*DS
            DO j = jmax/2+1,jmax
               IF ( wrk2d(n,j  ,2) .GT. fS_05 .AND. &
                    wrk2d(n,j+1,2) .LE. fS_05 ) THEN
                  wrk1d(n,3) = y(j) + &
                       (fS_05-wrk2d(n,j,2))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,2)-wrk2d(n,j,2))
               ENDIF
            ENDDO
            wrk1d(n,3) = wrk1d(n,3) - y_center
            wrk1d(n,2) = C_05_R*(wrk1d(n,2)+wrk1d(n,3))
         ENDDO
         
      ENDIF

! ############################
! # Scaling of the quatities #
! ############################

      DO n = 1,nstatavg

         IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN
         ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
            DU = C_05_R*(wrk2d(n,jmax/2,1)+wrk2d(n,jmax/2+1,1)) - U2
            DS = C_05_R*(wrk2d(n,jmax/2,2)+wrk2d(n,jmax/2+1,2)) - S2
         ENDIF

         DO j = 1,jmax
            smg_dis(n,j) = smg_dis(n,j)/(DS*DS*DU)*wrk1d(n,2)
            ssm_dis(n,j) = ssm_dis(n,j)/(DS*DS*DU)*wrk1d(n,2)
         ENDDO

      ENDDO

! ####################################
! # Saving the data in TkStat format #
! ####################################

      name = 'jls                            '
      CALL CONCATI(name, is)
      CALL CONCAT(name, 'scbdg')
      CALL CONCATI(name,itime)

#ifdef USE_RECLEN
      OPEN(UNIT=i23,RECL=550,FILE=name,STATUS='unknown')
#else
      OPEN(UNIT=i23,FILE=name,STATUS='unknown')
#endif     

      WRITE(i23,*) '#################'
      WRITE(i23,*) '#  Version 1.0  #'
      WRITE(i23,*) '#################'
      WRITE(i23,*) 'RTIME = ', rtime

      line2='I J X Y SS'
      CALL CLIP(line2,ll1,ll2)

      line1 = 'Cs_Z rho*D_sg smg_qx smg_qy smg_qz smg_Chi '
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP 1 SMG ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='ssm_qx ssm_qy ssm_qz ssm_Chi '
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP 2 SSM ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='arm_c0 arm_Z2_sg arm_Z4_sg arm_Z8_sg '&
           //'arm_Z2_r arm_Z4_r arm_Z8_r arm_A1_sg arm_A1_r arm_A(Zm)'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP 3 ARM ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      WRITE(i23,*) line2(ll1:ll2)
   
      DO n = 1,nstatavg
         i = statavg(n)
         DO j = 1,jmax
            IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
            ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN
               SimVar2 = (y(j) - y_center)/wrk1d(n,2)
               WRITE(i23,1300) i, j, x(i), y(j), SimVar2,&
                    (stat(n,j,l1),l1=1,L_AVGMAX)
            ENDIF
         ENDDO
      ENDDO

      CLOSE(i23)

#ifdef TRACE_ON
      CALL IO_WRITE_ASCII(tfile, 'LEAVING LESSCBDG_V1_0' )
#endif

 1300 FORMAT(I3,1X,I3,4(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))

      RETURN
      END
