SUBROUTINE LESSCBDG_V1_1(is, itxc, x, y, mean1d, mean1d_sc, &
     mean1d_les_sc, stat, wrk1d, wrk2d)

! ###############################################################
! # Postprocessing of averages for the LES quantities
! #
! # mean1d_les_sc - array with the accumalated statistics for the scalar les
! # mean1d        - array with the accumalated statistics for base flow
! # mean1d_sc     - array with the accumalated statistics for base scalar flow
! # stat          - tmp array of size itxc
! #
! # 05/01/02 - Version 1.0
! # 07/19/03 - Version 1.1. Adding terms for the momentum and 
! #            energy balance equations
! # 11/06/03 - Version 1.1. Removing the ARM terms
! #
! ################################################################

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL
  USE NFILE,     ONLY : efile

  IMPLICIT NONE

  TINTEGER is,itxc
  TREAL x(imax)
  TREAL y(jmax)
  TREAL mean1d(nstatavg,jmax,*)
  TREAL mean1d_sc(nstatavg,jmax,*)
  TREAL mean1d_les_sc(nstatavg,jmax,*)
  TREAL stat(nstatavg,jmax,*)
  TREAL wrk1d(nstatavg,*)
  TREAL wrk2d(nstatavg,jmax,5)

  TINTEGER i, j, n, NNstat, l1, l2, ll1, ll2
  TREAL U2, DU, S2, DS, r05
  TREAL pts, y_center, SimVar2, favZx, favZy, favZ, reyZ
  TREAL SIMPSON_NU
  CHARACTER*32 name
  CHARACTER*250 line1
  CHARACTER*500 line2
  TINTEGER i0, i1, i23

#define smg_cs_Z(A,B)    stat(A,B,1)
#define smg_mu_Z(A,B)    stat(A,B,2)
#define smg_qx(A,B)      stat(A,B,3)
#define smg_qy(A,B)      stat(A,B,4) 
#define smg_qz(A,B)      stat(A,B,5)
#define smg_dis(A,B)     stat(A,B,6)
#define smg_trans(A,B)   stat(A,B,7)
#define smg_force(A,B)   stat(A,B,8)

#define ssm_qx(A,B)      stat(A,B,9)
#define ssm_qy(A,B)      stat(A,B,10) 
#define ssm_qz(A,B)      stat(A,B,11)
#define ssm_dis(A,B)     stat(A,B,12)
#define ssm_trans(A,B)   stat(A,B,13)
#define ssm_force(A,B)   stat(A,B,14)

#define tot_dis(A,B)     stat(A,B,15)
#define tot_trans(A,B)   stat(A,B,16)
#define tot_mean(A,B)    stat(A,B,17)
#define tot_prod(A,B)    stat(A,B,18)

#define tot_force(A,B)   stat(A,B,19)

#define tot_qx(A,B)      stat(A,B,20)
#define tot_qy(A,B)      stat(A,B,21)
#define tot_qz(A,B)      stat(A,B,22)

#define L_AVGMAX 22
#define LAST_INDEX 23

#define TotMom(A)        stat(A,1,LAST_INDEX)

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LESSCBDG_V1_1' )
#endif

  i0 = 0
  i1 = 1
  i23 = 23

  r05 = C_05_R

  IF ( itxc .LT. nstatavg*jmax*LAST_INDEX) THEN
     CALL IO_WRITE_ASCII(efile,'LESSCBDG_V1_1: Not enough space in stat')
     CALL DNS_STOP(LES_ERROR_AVGTMP)
  ENDIF
  if ( nstatavg_points .EQ. 0 ) then
     CALL IO_WRITE_ASCII(efile,'LESSCBDG_V1_1: Zero number of points')
     CALL DNS_STOP(LES_ERROR_AVGPTS)
  ELSE
     pts = C_1_R/M_REAL(nstatavg_points)
  endif

  NNstat = jmax*nstatavg
  y_center = y(1) + C_05_R*scaley
  U2 = mean_u - C_05_R*delta_u
  S2 = mean_z - C_05_R*delta_z

  DO j = 1,NNstat

     wrk2d(j,1,1) = MA_RU(j)/MA_R(j)
     wrk2d(j,1,2) = MS_RS(j)/MA_R(j)

     smg_cs_Z(j,1)  = LS_SMG_CS_Z(j)*pts
     smg_mu_Z(j,1)  = LS_SMG_RD(j)*pts
     smg_qx(j,1)    = LS_SMG_Qx(j)*pts
     smg_qy(j,1)    = LS_SMG_Qy(j)*pts
     smg_qz(j,1)    = LS_SMG_Qz(j)*pts
     smg_dis(j,1)   = C_2_R*LS_SMG_EPS_Z(j)*pts
     smg_trans(j,1) = C_2_R*LS_SMG_TRA_Z(j)*pts
     smg_force(j,1) = LS_SMG_FORCE_Z(j)*pts

     ssm_qx(j,1)    = LS_SSM_Qx(j)*pts
     ssm_qy(j,1)    = LS_SSM_Qy(j)*pts
     ssm_qz(j,1)    = LS_SSM_Qz(j)*pts
     ssm_dis(j,1)   = C_2_R*LS_SSM_EPS_Z(j)*pts
     ssm_trans(j,1) = C_2_R*LS_SSM_TRA_Z(j)*pts
     ssm_force(j,1) = LS_SSM_FORCE_Z(j)*pts

     tot_force(j,1) = smg_force(j,1)+&
          ssm_force(j,1)

     tot_qx(j,1) = (smg_qx(j,1)+&
          ssm_qx(j,1))/&
          (MA_R(j)*pts)
     tot_qy(j,1) = (smg_qy(j,1)+&
          ssm_qy(j,1))/&
          (MA_R(j)*pts)
     tot_qz(j,1) = (smg_qz(j,1)+&
          ssm_qz(j,1))/&
          (MA_R(j)*pts)

     favZx = (MS_RSx(j)+MS_SRx(j)-&
          MA_Rx(j)*wrk2d(j,1,2))/MA_R(j)
     favZy = (MS_RSy(j)+MS_SRy(j)-&
          MA_Ry(j)*wrk2d(j,1,2))/MA_R(j)
     favZ  = wrk2d(j,1,2)
     reyZ  = MS_S(j)*pts

     tot_dis(j,1) = C_2_R*(LS_SMG_EPS_Z(j)+&
          LS_SSM_EPS_Z(j)-&
          (LS_SMG_Qx(j)+LS_SSM_Qx(j))*favZx-&
          (LS_SMG_Qy(j)+LS_SSM_Qy(j))*favZy)/&
          MA_R(j)

! watch out the sign of LS_FORCE !!
     tot_trans(j,1)=-tot_dis(j,1)-C_2_R*(&
          LS_SMG_TRA_Z(j)+&
          LS_SSM_TRA_Z(j)+&
          (LS_SMG_FORCE_Z(j)+LS_SSM_FORCE_Z(j))*&
          favZ)/MA_R(j)

     tot_mean(j,1) = C_2_R*(reyZ-favZ)*&
          tot_force(j,1)/&
          (MA_R(j)*pts)

     tot_prod(j,1) =-(&
          tot_qx(j,1)*favZx+&
          tot_qy(j,1)*favZy)

! tot_force
     tot_force(j,1) = tot_force(j,1)/&
          (MA_R(j)*pts)
  ENDDO

! ######################################
! # Favre-averaged profiles of U and Z #
! ######################################

  IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN

  ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
! Velocity Half width
     CALL DELTA_X(nstatavg, jmax, y, wrk2d(1,1,1), wrk1d(1,1),&
          wrk1d(1,2), wrk1d(1,3), U2, r05)
! Scalar Half width
     CALL DELTA_X(nstatavg, jmax, y, wrk2d(1,1,2), wrk1d(1,2),&
          wrk1d(1,3), wrk1d(1,4), S2, r05)
!     
  ENDIF

! ###########################
! # Integral quantities jet #
! ###########################

  IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
! Excess of momentum
     l1 = ibuffer_ht-ibuffer_hb+1
     DO n = 1,nstatavg
        DO j = ibuffer_hb,ibuffer_ht
           wrk2d(j,1,3) = smg_qx(n,j)+&
                ssm_qx(n,j)
        ENDDO
        TotMom(n) = SIMPSON_NU(l1, wrk2d(ibuffer_hb,1,3), &
             y(ibuffer_hb))
     ENDDO
  ENDIF

! ############################
! # Scaling of the quatities #
! ############################

  DO n = 1,nstatavg

     IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN
     ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
        DU = C_05_R*(wrk2d(n,jmax/2,1)+wrk2d(n,jmax/2+1,1)) - U2
        DS = C_05_R*(wrk2d(n,jmax/2,2)+wrk2d(n,jmax/2+1,2)) - S2
     ENDIF

     DO j = 1,jmax
        tot_dis(n,j)   = tot_dis(n,j)&
             /(DS*DS*DU)*wrk1d(n,1)
        tot_trans(n,j) = tot_trans(n,j)&
             /(DS*DS*DU)*wrk1d(n,1)
        tot_mean(n,j)  = tot_mean(n,j)&
             /(DS*DS*DU)*wrk1d(n,1)
        tot_prod(n,j)  = tot_prod(n,j)&
             /(DS*DS*DU)*wrk1d(n,1)

        tot_force(n,j) = tot_force(n,j)&
             /(DS*DU)*wrk1d(n,1)

        tot_qx(n,j) = tot_qx(n,j)/(DS*DU)
        tot_qy(n,j) = tot_qy(n,j)/(DS*DU)
        tot_qz(n,j) = tot_qz(n,j)/(DS*DU)

     ENDDO

  ENDDO

! ####################################
! # Saving the data in TkStat format #
! ####################################

  name = 'jlsavg '
  CALL CONCATI(name, is)
  CALL CONCAT(name, 's')
  CALL CONCATI(name,itime)

#ifdef USE_RECLEN
  OPEN(UNIT=i23,RECL=400,FILE=name,STATUS='unknown')
#else
  OPEN(UNIT=i23,FILE=name,STATUS='unknown')
#endif     

  WRITE(i23,1010) '#################'
  WRITE(i23,1010) '#  Version 1.1  #'
  WRITE(i23,1010) '#################'
  WRITE(i23,'(A8,E14.7E3)') 'RTIME = ', rtime

  line2='I J X Y SS'
  CALL CLIP(line2,ll1,ll2)

  line1 = 'Cs_Z rho*D_sg smg_qx smg_qy smg_qz smg_Chi '&
       //'smg_trans smg_force'
  CALL CLIP(line1,l1,l2)
  WRITE(i23,1010) 'GROUP = SMG '//line1(l1:l2)
  line2=line2(ll1:ll2)//' '//line1(l1:l2)
  CALL CLIP(line2,ll1,ll2)

  line1 ='ssm_qx ssm_qy ssm_qz ssm_Chi ssm_trans ssm_force'
  CALL CLIP(line1,l1,l2)
  WRITE(i23,1010) 'GROUP = SSM '//line1(l1:l2)
  line2=line2(ll1:ll2)//' '//line1(l1:l2)
  CALL CLIP(line2,ll1,ll2)

  line1 ='Diss Tran MnFl Prod'
  CALL CLIP(line1,l1,l2)
  WRITE(i23,1010) 'GROUP = Rss_Eqn '//line1(l1:l2)
  line2=line2(ll1:ll2)//' '//line1(l1:l2)
  CALL CLIP(line2,ll1,ll2)

  line1 ='Reyn_s'
  CALL CLIP(line1,l1,l2)
  WRITE(i23,1010) 'GROUP = S_Eqn '//line1(l1:l2)
  line2=line2(ll1:ll2)//' '//line1(l1:l2)
  CALL CLIP(line2,ll1,ll2)

  line1 ='Rus Rvu Rws'
  CALL CLIP(line1,l1,l2)
  WRITE(i23,1010) 'GROUP = Reynolds_stresses '//line1(l1:l2)
  line2=line2(ll1:ll2)//' '//line1(l1:l2)
  CALL CLIP(line2,ll1,ll2)

  IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
     line1 = 'Int_mom_RuZ'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = 1D_Quantities '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)
  ENDIF

  WRITE(i23,1010) line2(ll1:ll2)

1010 FORMAT(A)

  DO n = 1,nstatavg
     i = statavg(n)
     DO j = 1,jmax
        SimVar2 = (y(j) - y_center)/wrk1d(n,2)
        IF ( j .EQ. jmax/2 ) THEN
           IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
           ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN
              WRITE(i23,1400) i, j, x(i), y(j), SimVar2,&
                   (stat(n,j,l1),l1=1,L_AVGMAX),&
                   TotMom(n)
           ENDIF
        ELSE
           IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
           ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN
              WRITE(i23,1300) i, j, x(i), y(j), SimVar2,&
                   (stat(n,j,l1),l1=1,L_AVGMAX)
           ENDIF
        ENDIF
     ENDDO
  ENDDO

  CLOSE(i23)

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LESSCBDG_V1_1' )
#endif

1300 FORMAT(I5,1X,I5,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))
1400 FORMAT(I5,1X,I5,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3),1X,E12.5E3)

  RETURN
END SUBROUTINE LESSCBDG_V1_1
