      SUBROUTINE SGS_DYN_CROSS2(idir1, idir2, ssmflg, n0, n1,  
     $     dx, dy, dz,
     $     rho, u, v, sij, barsij, barvs, frc, shij, txc, wrk1d, 
     $     wrk2d, wrk3d, cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, 
     $     cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_vf, carm_vh)

c ###############################################################
c # frc1 is LHSH
c # frc2 is NFSH
c # frc3 is NHSH
c #
c # txc - required size of 3 if ssmflg is 0
c #       required size of 4 if ssmflg is 1
c #
c # shij stores in reality 2shij for the case of the velocity because
c #   the off-diagonal terms appear twice
c #
c # 07/29/02 - Introducing barvs and eliminating FILT_FAV to
c #            save time
c # 09/25/04 - Profiling and preparing for ARM: 2 more arrays.
c ###############################################################

      IMPLICIT NONE

#include "types.h"
#include "common.h"

      TINTEGER idir1, idir2, ssmflg, n0, n1

      TREAL dx(imax)
      TREAL dy(jmax)
      TREAL dz(kmax_total)
      TREAL rho(imax,jmax,kmax)
      TREAL u(imax,jmax,kmax)
      TREAL v(imax,jmax,kmax)

      TREAL sij(imax,jmax,kmax)
      TREAL barsij(imax,jmax,kmax)
      TREAL barvs(imax,jmax,kmax,*)
      TREAL frc(imax,jmax,kmax,3)
      TREAL shij(imax,jmax,kmax)
      TREAL txc(imax,jmax,kmax,*)

      TREAL wrk1d(ijkmax,*)
      TREAL wrk2d(ijmax,*)
      TREAL wrk3d(imax,jmax,kmax)
      TREAL cf0x(*), cf0y(*), cf0z(*)
      TREAL cf1x(*), cf1y(*), cf1z(*)
      TREAL cf2x(*), cf2y(*), cf2z(*)
      TREAL carm_uf(*), carm_uh(*)
      TREAL carm_vf(*), carm_vh(*)

      TINTEGER ij, k, i, j
      TINTEGER i0, i1, i2, i3
      TREAL mean, AVG_IK

      i0 = 0
      i1 = 1
      i2 = 2
      i3 = 3

#ifdef TRACE_ON
      CALL OUTPUT(trfile, 'ENTERING SGS_DYN_CROSS')
#endif

#define LHSH(i,j,k)  frc(i,j,k,1)
#define NFSH(i,j,k)  frc(i,j,k,2)
#define NHSH(i,j,k)  frc(i,j,k,3)

c ############################
c # Calculate Sij at level H #
c ############################
      DO k = 1,kmax
         DO ij = 1,NN
            txc(ij,1,k,1) = rho(ij,1,k)*v(ij,1,k)
         ENDDO
      ENDDO
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n1, cf1x, cf1y, cf1z, txc(1,1,1,1), txc(1,1,1,2), wrk3d)
      DO k = 1,kmax
         DO ij = 1,NN
            txc(ij,1,k,2) = txc(ij,1,k,2)*barvs(ij,1,k,1)
         ENDDO
      ENDDO
      IF ( idir1 .EQ. 1 ) THEN
         CALL PARTIAL_X(isord, imax, jmax, kmax, i1bc,
     $        dx, txc(1,1,1,2), shij, i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ELSE IF ( idir1 .EQ. 2 ) THEN
         CALL PARTIAL_Y(isord, imax, jmax, kmax, j1bc,
     $        dy, txc(1,1,1,2), shij, i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ELSE
         CALL PARTIAL_Z(isord, imax, jmax, kmax, k1bc,
     $        dz, txc(1,1,1,2), shij, i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ENDIF

      IF ( idir2 .NE. 0 ) THEN
c     Velocity case required the symmetric term
         DO k = 1,kmax
            DO ij = 1,NN
               txc(ij,1,k,2) = rho(ij,1,k)*u(ij,1,k)
            ENDDO
         ENDDO
         CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $        iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $        n1, cf1x, cf1y, cf1z, txc(1,1,1,2), txc(1,1,1,3), wrk3d)
         DO k = 1,kmax
            DO ij = 1,NN
               txc(ij,1,k,3) = txc(ij,1,k,3)*barvs(ij,1,k,1)
            ENDDO
         ENDDO
         IF ( idir2 .EQ. 1 ) THEN
            CALL PARTIAL_X(isord, imax, jmax, kmax, i1bc,
     $           dx, txc(1,1,1,3), txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ELSE IF (idir2 .EQ. 2 ) THEN
            CALL PARTIAL_Y(isord, imax, jmax, kmax, j1bc,
     $           dy, txc(1,1,1,3), txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ELSE
            CALL PARTIAL_Z(isord, imax, jmax, kmax, k1bc,
     $           dz, txc(1,1,1,3), txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ENDIF
c     SHij stores in reality 2*SHij for the case of the velocity becasue
c     the off-diagonal terms appear twice
         DO k=1,kmax
            DO ij=1,NN
               shij(ij,1,k) = shij(ij,1,k)+txc(ij,1,k,2)
            ENDDO
         ENDDO
      ENDIF

c ####################
c # Calculate NijSij #
c ####################
c -------------------------------------------------------------
c                                         level F, bar(N_F):S_H
c -------------------------------------------------------------
      IF ( idir1 .EQ. 1 ) THEN
         CALL PARTIAL_X(isord, imax, jmax, kmax, i1bc,
     $        dx, v, txc(1,1,1,1), i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ELSE IF ( idir1 .EQ. 2 ) THEN
         CALL PARTIAL_Y(isord, imax, jmax, kmax, j1bc,
     $        dy, v, txc(1,1,1,1), i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ELSE
         CALL PARTIAL_Z(isord, imax, jmax, kmax, k1bc,
     $        dz, v, txc(1,1,1,1), i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ENDIF
      IF ( idir2 .NE. 0 ) THEN
c     velocity case
         IF ( idir2 .EQ. 1 ) THEN
            CALL PARTIAL_X(isord, imax, jmax, kmax, i1bc,
     $           dx, u, txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ELSE IF ( idir2 .EQ. 2 ) THEN
            CALL PARTIAL_Y(isord, imax, jmax, kmax, j1bc,
     $           dy, u, txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ELSE
            CALL PARTIAL_Z(isord, imax, jmax, kmax, k1bc,
     $           dz, u, txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ENDIF
         DO k=1,kmax
            DO ij=1,NN
               txc(ij,1,k,1) = C_05_R*(txc(ij,1,k,1)+txc(ij,1,k,2))*
     $              sij(ij,1,k)
            ENDDO
         ENDDO
      ELSE
c     Scalar case
         DO k=1,kmax
            DO ij=1,NN
               txc(ij,1,k,1) = txc(ij,1,k,1)*sij(ij,1,k)
            ENDDO
         ENDDO
      ENDIF

      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n1, cf1x, cf1y, cf1z, txc(1,1,1,1), txc(1,1,1,2), wrk3d)

      DO j = 1,jmax
         mean = AVG_IK(imax, jmax, kmax, j, txc(1,1,1,2), dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               NFSH(i,j,k) = NFSH(i,j,k) + 
     $              (txc(i,j,k,2)-mean)*shij(i,j,k)
            ENDDO
         ENDDO
      ENDDO      
c      DO k=1,kmax
c         DO ij=1,NN
c            NFSH(ij,1,k) = NFSH(ij,1,k) + 
c     $           txc(ij,1,k,2)*shij(ij,1,k)
c         ENDDO
c      ENDDO

c -------------------------------------------------------------
c                                                       level H
c -------------------------------------------------------------
      DO k = 1,kmax
         DO ij = 1,NN
            txc(ij,1,k,1) = rho(ij,1,k)*v(ij,1,k)
         ENDDO
      ENDDO
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n1, cf1x, cf1y, cf1z, txc(1,1,1,1), txc(1,1,1,2), wrk3d)
      DO k = 1,kmax
         DO ij = 1,NN
            txc(ij,1,k,2) = txc(ij,1,k,2)*barvs(ij,1,k,1)
         ENDDO
      ENDDO
      IF ( idir1 .EQ. 1 ) THEN
         CALL PARTIAL_X(isord, imax, jmax, kmax, i1bc,
     $        dx, txc(1,1,1,2), txc(1,1,1,1), i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ELSE IF ( idir1 .EQ. 2 ) THEN
         CALL PARTIAL_Y(isord, imax, jmax, kmax, j1bc,
     $        dy, txc(1,1,1,2), txc(1,1,1,1), i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ELSE
         CALL PARTIAL_Z(isord, imax, jmax, kmax, k1bc,
     $        dz, txc(1,1,1,2), txc(1,1,1,1), i0, i0,
     $        wrk1d, wrk2d, wrk3d)
      ENDIF

      IF ( idir2 .NE. 0 ) THEN
c     Velocity case
         DO k = 1,kmax
            DO ij = 1,NN
               txc(ij,1,k,2) = rho(ij,1,k)*u(ij,1,k)
            ENDDO
         ENDDO
         CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $        iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $        n1, cf1x, cf1y, cf1z, txc(1,1,1,2), txc(1,1,1,3), wrk3d)
         DO k = 1,kmax
            DO ij = 1,NN
               txc(ij,1,k,3) = txc(ij,1,k,3)*barvs(ij,1,k,1)
            ENDDO
         ENDDO
         IF ( idir2 .EQ. 1 ) THEN
            CALL PARTIAL_X(isord, imax, jmax, kmax, i1bc,
     $           dx, txc(1,1,1,3), txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ELSE IF (idir2 .EQ. 2 ) THEN
            CALL PARTIAL_Y(isord, imax, jmax, kmax, j1bc,
     $           dy, txc(1,1,1,3), txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ELSE
            CALL PARTIAL_Z(isord, imax, jmax, kmax, k1bc,
     $           dz, txc(1,1,1,3), txc(1,1,1,2), i0, i0,
     $           wrk1d, wrk2d, wrk3d)
         ENDIF

         DO k=1,kmax
            DO ij=1,NN
               txc(ij,1,k,1) = C_05_R*(txc(ij,1,k,1)+txc(ij,1,k,2))*
     $              barsij(ij,1,k)
            ENDDO
         ENDDO
      ELSE
c     Scalar case
         DO k=1,kmax
            DO ij=1,NN
               txc(ij,1,k,1) = txc(ij,1,k,1)*barsij(ij,1,k)
            ENDDO
         ENDDO

      ENDIF

      DO j = 1,jmax
         mean = AVG_IK(imax, jmax, kmax, j, txc(1,1,1,1), dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               NHSH(i,j,k) = NHSH(i,j,k) + 
     $              (txc(i,j,k,1)-mean)*shij(i,j,k)
            ENDDO
         ENDDO
      ENDDO      

c ####################      
c # Calculate LijSij #
c ####################
c -------------------------------------------------------------
c                                                       level 0
c -------------------------------------------------------------
      DO k=1,kmax
         DO ij=1,NN
            txc(ij,1,k,1) = u(ij,1,k)*v(ij,1,k)*rho(ij,1,k)
         ENDDO
      ENDDO
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n1, cf1x, cf1y, cf1z, txc(1,1,1,1), txc(1,1,1,2), wrk3d)
      DO k=1,kmax
         DO ij=1,NN
            LHSH(ij,1,k) = LHSH(ij,1,k) + 
     $           txc(ij,1,k,2)*shij(ij,1,k)
         ENDDO
      ENDDO

c -------------------------------------------------------------
c                                                       level 1
c -------------------------------------------------------------
      DO k=1,kmax
         DO ij=1,NN
            txc(ij,1,k,2) = rho(ij,1,k)*u(ij,1,k)
         ENDDO
      ENDDO
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n1, cf1x, cf1y, cf1z, txc(1,1,1,2), txc(1,1,1,1), wrk3d)
      DO k=1,kmax
         DO ij=1,NN
            txc(ij,1,k,3) = rho(ij,1,k)*v(ij,1,k)
         ENDDO
      ENDDO
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n1, cf1x, cf1y, cf1z, txc(1,1,1,3), txc(1,1,1,2), wrk3d)
      DO k=1,kmax
         DO ij=1,NN
            LHSH(ij,1,k) = LHSH(ij,1,k) - 
     $           txc(ij,1,k,1)*txc(ij,1,k,2)*shij(ij,1,k)*
     $           barvs(ij,1,k,1)
         ENDDO
      ENDDO

c ####################
c # Calculate HijSij #
c ####################
      IF ( ssmflg .EQ. 1 ) THEN
         CALL SGS_DYN_HIJ(i0, n0, n1, rho, u, v, barvs, LHSH(1,1,1), 
     $        shij, txc, 
     $        wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, 
     $        cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_vf, carm_vh)
      ENDIF

#ifdef TRACE_ON
      CALL OUTPUT(trfile, 'LEAVING SGS_DYN_CROSS')
#endif

#undef LHSH
#undef NFSH
#undef NHSH

      RETURN
      END
