SUBROUTINE SGS_CHEM_QUASIBS(x, y, z, dx, dy, dz, rho, &
     u, p, z1, h4, T, gama, tmp, chi, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z, fdf)
  
! txc size required is 8
  
#include "types.h"
  
  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL u(imax,jmax,kmax)
  TREAL rho(imax,jmax,kmax)
  TREAL p(imax,jmax,kmax)
  TREAL z1(imax,jmax,kmax)
  TREAL T(imax,jmax,kmax)
  TREAL h4(imax,jmax,kmax)
  TREAL gama(imax,jmax,kmax)

  TREAL chi(imax,jmax,kmax)
  TREAL tmp(imax,jmax,kmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)
  TREAL cf0x(*), cf0y(*), cf0z(*)
  TREAL fdf(*)

  TINTEGER k, ij, p2, p3

#ifdef PARALLEL
  TREAL dtmp
#endif

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_CHEM_QUASIBS')
#endif

! gamma
!      CALL CHEM(z1, p, rho, T, gama)

#define OMEGAP(i,j,k)  tmp(i,j,k,1)
#define TMP_LOC(i,j,k) tmp(i,j,k,2)

  p2 = i1 + les_fdf_bs_maxmean
  p3 = p2 + les_fdf_bs_maxmean*les_fdf_bs_maxvar
  CALL SGS_HEAT_BS(x, y, z, dx, dy, dz, rho, &
       u, z1, gama, OMEGAP(1,1,1), &
       TMP_LOC(1,1,1), chi, wrk1d, wrk2d, wrk3d,&
       cf0x, cf0y, cf0z, fdf(1), fdf(p2), fdf(p3))

  DO k = 1,kmax
     DO ij = 1,imax*jmax
        h4(ij,1,k) = h4(ij,1,k) + OMEGAP(ij,1,k)
        TGFM = MAX(TGFM, ABS(OMEGAP(ij,1,k)))
     ENDDO
  ENDDO

! Distribute TGFM across processors
#ifdef PARALLEL
  CALL MPI_ALLREDUCE(TGFM, dtmp, 1, MPI_REAL8, MPI_MAX, &
       MPI_COMM_WORLD, ims_err)
  TGFM = dtmp
#endif

#undef OMEGAP
#undef TMP_LOC

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_CHEM_QUASIBS' )
#endif

  RETURN
END SUBROUTINE SGS_CHEM_QUASIBS
