!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2001/11/30 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# sij     = rho * delta_F^2 * SQRT(2SijSij)
!# barsij  = rho * delta_f^2 * SQRT(2*bar(Sij)*bar(Sij))
!#
!# txc1 is bar1(bar0(u_sg) |
!# txc2 is bar1(bar0(v_sg)  > use in dilatation
!# txc3 is bar1(bar0(w_sg) |
!#
!# barvs(1) = 1/bar1(bar0(rho))          <--- Always.
!# barvs(2) = 1/bar0(bar0(rho))             | Only
!# barvs(3) = 1/bar1(bar0(bar1(bar0(rho)))) | for Hij.
!#
!# txc     In   size is 6, if ssmflg is 0
!#              size is 7, if ssmflg is 1
!#
!########################################################################
SUBROUTINE SGS_DYN_Z(ssmflg, coefdelta, n0, n1, x, y, z, dx, dy, dz, &
     rho, u, v, w, z1, u_sg, v_sg, w_sg, z1_sg, &
     sij, barsij, barvs, txc, sgscz, wrk1d, wrk2d, wrk3d, &
     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_zf, carm_zh)

#include "types.h"
#include "dns_const.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER ssmflg, n1, n0
  TREAL coefdelta

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: u, v, w, rho, z1, u_sg, v_sg, w_sg, z1_sg
  TREAL, DIMENSION(imax,jmax,kmax,*) :: txc

  TREAL sij(imax,jmax,kmax)
  TREAL barsij(imax,jmax,kmax)
  TREAL barvs(imax,jmax,kmax,*)

  TREAL, DIMENSION(*) :: cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z
  TREAL, DIMENSION(*) :: carm_uf, carm_uh, carm_zf, carm_zh, sgscz

  TREAL, DIMENSION(*) :: wrk1d, wrk2d, wrk3d

! -----------------------------------------------------------------------
  TREAL r23, r13

  TREAL, DIMENSION(:,:,:,:), POINTER : FRACTION

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_DYN_Z')
#endif

  r23 = C_2_R/C_3_R
  r13 = C_1_R/C_3_R

! -----------------------------------------------------------------------
! Define the base fields over which the dynamic procedure 
! applies (i.e. remove mean if so specified)
! -----------------------------------------------------------------------
! txc_size=1
  CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
       cf0x, cf0y, cf0z, rho, z1, z1_sg, txc, wrk2d, wrk3d)
! ------------------------------------------------------------
!                            ARM coefficients (txc_size=5)
! ------------------------------------------------------------
  IF ( iles_type_tran .EQ. LES_TRAN_ARMRANS .OR.&
       iles_type_tran .EQ. LES_TRAN_ARMSPTL .OR.&
       iles_type_tran .EQ. LES_TRAN_ARMINVS ) THEN
! not yet implemented in new version
!         CALL SGS_DYN_ARM_Z(n0, n1, x, y, z, dx, dy, dz, &
!              rho, z1_sg, barvs, txc, wrk1d, wrk2d, wrk3d, &
!              cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, &
!              carm_zf, carm_zh)
  ENDIF

#define MIJ(i,j,k)     txc(i,j,k,1)

  FRACTION => txc(:,:,:,2:)

  FRACTION(:,:,:,1:2) = C_0_R

#define TMP_LOC(i,j,k) txc(i,j,k,4)

! #########################
! #  Lx, Hy and Mx terms  #
! #########################
  CALL SGS_DYN_CROSS(i1, i0, ssmflg, n0, n1, coefdelta, dx, dy, dz, rho, u, z1, u_sg, z1_sg, &
       sij, barsij, barvs, FRACTION(1,1,1), MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_zf, carm_zh)
!  CALL SGS_DYN_CROSS(i1, i0, ssmflg, n0, n1, coefdelta, &
!       dx, dy, dz, rho, u_sg, z1_sg, u_sg, z1_sg,&
!       sij, barsij, barvs, FRACTION(1,1,1), &
!       MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
!       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!       carm_uf, carm_uh, carm_zf, carm_zh)

! #########################
! #  Ly, Hy and My terms  #
! #########################
  CALL SGS_DYN_CROSS(i2, i0, ssmflg, n0, n1, coefdelta, dx, dy, dz, rho, v, z1, v_sg, z1_sg, &
       sij, barsij, barvs, FRACTION(1,1,1), MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_zf, carm_zh)
!  CALL SGS_DYN_CROSS(i2, i0, ssmflg, n0, n1, coefdelta, &
!       dx, dy, dz, rho, v_sg, z1_sg, v_sg, z1_sg, &
!       sij, barsij, barvs, FRACTION(1,1,1), &
!       MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
!       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!       carm_uf, carm_uh, carm_zf, carm_zh)

! #########################
! #  Lz, Hy and Mz terms  #
! #########################
  IF ( kmax_total .GT. 1 ) THEN
     CALL SGS_DYN_CROSS(i3, i0, ssmflg, n0, n1, coefdelta, dx, dy, dz, rho, w, z1, w_sg, z1_sg, &
          sij, barsij, barvs, FRACTION(1,1,1), MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf,carm_uh,carm_zf,carm_zh)
!     CALL SGS_DYN_CROSS(i3, i0, ssmflg, n0, n1, coefdelta, &
!          dx, dy, dz, rho, w_sg, z1_sg, w_sg, z1_sg,&
!          sij, barsij, barvs, FRACTION(1,1,1), &
!          MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
!          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!          carm_uf, carm_uh, carm_zf, carm_zh)
  ENDIF

#undef MIJ
#undef TMP_LOC

! #######################################################################
! Calculate spatial average of CsZ; Clip it to positive values for stability
! #######################################################################
  CALL SGS_DYNCF(x, dx, dy, dz, FRACTION(1,1,1), sgscz, wrk1d, wrk2d, wrk3d)

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_DYN_Z' )
#endif

  NULLIFY(FRACTION)

  RETURN
END SUBROUTINE SGS_DYN_Z

