      SUBROUTINE SGS_DYN_HIJ(inormal,&
           n0, n1, dx, dy, dz, rho, u, v, barvs, numerator, mij, &
           txc, wrk1d, wrk2d, wrk3d, &
           cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
           carm_uf, carm_uh, carm_vf, carm_vh)

! #####################################################################
! # numerator - numerator of the fraccion, i.e., -H_ij*M_ij, therefore
! #             there are four terms to be computed. 
! # Flag inormal indicates if it is diagonal term
! # - if yes (1), txc size is 3
! # - if no  (0), txc size is 4
! #
! # FILTH_FAV is not used to save 1 division and 1 multiplication.
! #
! # barvs(1) = 1/bar1(bar0(rho))          
! # barvs(2) = 1/bar0(bar0(rho))             
! # barvs(3) = 1/bar1(bar0(bar1(bar0(rho)))) 
! # 
! # Modification
! # ------------
! # 09/25/04 - Reducing operations by adding 2 array. Preparing for ARM. 
! #            Introducing density arrays.
! # 10/25/04 - Introducing ARM
! #####################################################################

#include "types.h"
#include "les_const.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

      IMPLICIT NONE

#include "integers.h"

      TINTEGER inormal
      TINTEGER n0, n1

      TREAL dx(imax)
      TREAL dy(jmax)
      TREAL dz(kmax_total)

      TREAL rho(imax,jmax,kmax)
      TREAL u(imax,jmax,kmax)
      TREAL v(imax,jmax,kmax)

      TREAL barvs(imax,jmax,kmax,*)
      TREAL numerator(imax,jmax,kmax)
      TREAL mij(imax,jmax,kmax)
      TREAL txc(imax,jmax,kmax,*)

      TREAL wrk1d(isize_wrk1d,*)
      TREAL wrk2d(*)
      TREAL wrk3d(imax,jmax,kmax)
      TREAL cf0x(*), cf0y(*), cf0z(*)
      TREAL cf1x(*), cf1y(*), cf1z(*)
      TREAL cf2x(*), cf2y(*), cf2z(*)
      TREAL carm_uf(*), carm_uh(*)
      TREAL carm_vf(*), carm_vh(*)

      TINTEGER ij, k

#define RHOU_M(i,j,k)   txc(i,j,k,1)
#define TMP_LOC1(i,j,k) txc(i,j,k,2)
#define TMP_LOC2(i,j,k) txc(i,j,k,3)
#define RHOV_M(i,j,k)   txc(i,j,k,4)

! ######################
! filter level 0 (F)
! ######################
      IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
         IF ( inormal .EQ. 1 ) THEN
            DO k=1,kmax
               DO ij=1,imax*jmax
                  RHOU_M(ij,1,k) = u(ij,1,k)*rho(ij,1,k)
               ENDDO
            ENDDO
         ELSE
            DO k=1,kmax
               DO ij=1,imax*jmax
                  RHOU_M(ij,1,k) = u(ij,1,k)*rho(ij,1,k)
                  RHOV_M(ij,1,k) = v(ij,1,k)*rho(ij,1,k)
               ENDDO
            ENDDO
         ENDIF
      ELSE 
! ARM formulation
! not yet implemented in new version
!         IF ( inormal .EQ. 1 ) THEN
!            DO k=1,kmax
!               DO ij=1,imax*jmax
!                  TMP_LOC1(ij,1,k) = u(ij,1,k)*rho(ij,1,k)
!               ENDDO
!            ENDDO
!            CALL SGS_ARM_UM(n0, dx, dy, dz, &
!                 cf0x, cf0y, cf0z, carm_uf, TMP_LOC1(1,1,1), &
!                 RHOU_M(1,1,1), wrk1d, wrk3d)
!         ELSE
!            DO k=1,kmax
!               DO ij=1,imax*jmax
!                  TMP_LOC1(ij,1,k) = u(ij,1,k)*rho(ij,1,k)
!                  TMP_LOC2(ij,1,k) = v(ij,1,k)*rho(ij,1,k)
!               ENDDO
!            ENDDO
!            CALL SGS_ARM_UM(n0, dx, dy, dz, &
!                 cf0x, cf0y, cf0z, carm_uf, TMP_LOC1(1,1,1), &
!                 RHOU_M(1,1,1), wrk1d, wrk3d)
!            CALL SGS_ARM_UM(n0, dx, dy, dz, &
!                 cf0x, cf0y, cf0z, carm_vf, TMP_LOC2(1,1,1), &
!                 RHOV_M(1,1,1), wrk1d, wrk3d)
!         ENDIF
      ENDIF

! -----------------------------------------------------------------
!                                                    First term
! -----------------------------------------------------------------
      IF ( inormal .EQ. 1 ) THEN
         DO k=1,kmax
            DO ij=1,imax*jmax
               TMP_LOC1(ij,1,k) = &
                    RHOU_M(ij,1,k)*RHOU_M(ij,1,k)/rho(ij,1,k)
            ENDDO
         ENDDO
      ELSE
         DO k=1,kmax
            DO ij=1,imax*jmax
               TMP_LOC1(ij,1,k) = &
                    RHOU_M(ij,1,k)*RHOV_M(ij,1,k)/rho(ij,1,k)
            ENDDO
         ENDDO
      ENDIF
      CALL FILTH2(i3, imax, jmax, kmax, kmax_total,&
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
           n0, n1, cf2x, cf2y, cf2z, TMP_LOC1(1,1,1), &
           TMP_LOC2(1,1,1), wrk3d)
      DO k=1,kmax
         DO ij=1,imax*jmax
            numerator(ij,1,k) = numerator(ij,1,k) +&
                 TMP_LOC2(ij,1,k)*mij(ij,1,k)
         ENDDO
      ENDDO

! -----------------------------------------------------------------
!                                                   Second term
! -----------------------------------------------------------------
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
           n0, cf0x, cf0y, cf0z, RHOU_M(1,1,1), &
           TMP_LOC1(1,1,1), wrk3d)
      IF ( inormal .EQ. 1 ) THEN
         DO k=1,kmax
            DO ij=1,imax*jmax
               TMP_LOC1(ij,1,k) = TMP_LOC1(ij,1,k)*&
                    TMP_LOC1(ij,1,k)*barvs(ij,1,k,2)
            ENDDO
         ENDDO
      ELSE
         CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
              iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
              n0, cf0x, cf0y, cf0z, RHOV_M(1,1,1), &
              TMP_LOC2(1,1,1), wrk3d)
         DO k=1,kmax
            DO ij=1,imax*jmax
               TMP_LOC1(ij,1,k) = TMP_LOC1(ij,1,k)*&
                    TMP_LOC2(ij,1,k)*barvs(ij,1,k,2)
            ENDDO
         ENDDO
      ENDIF
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
           n1, cf1x, cf1y, cf1z, TMP_LOC1(1,1,1), &
           TMP_LOC2(1,1,1), wrk3d)
      DO k=1,kmax
         DO ij=1,imax*jmax
            numerator(ij,1,k) = numerator(ij,1,k) -&
                 TMP_LOC2(ij,1,k)*mij(ij,1,k)
         ENDDO
      ENDDO

! ######################
! filter level 1 (H)
! ######################
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
           n1, cf1x, cf1y, cf1z, RHOU_M(1,1,1), &
           TMP_LOC1(1,1,1), wrk3d)
      IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
         DO k=1,kmax
            DO ij=1,imax*jmax
               RHOU_M(ij,1,k) = TMP_LOC1(ij,1,k)
            ENDDO
         ENDDO
      ELSE 
! ARM formulation
! not yet implemented in new version
!         CALL SGS_ARM_UM2(n0, n1, dx, dy, dz, &
!              cf2x, cf2y, cf2z, carm_uh, TMP_LOC1(1,1,1), &
!              RHOU_M(1,1,1), wrk1d, wrk3d)
      ENDIF

      IF ( inormal .EQ. 1 ) THEN
      ELSE
         CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
              iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
              n1, cf1x, cf1y, cf1z, RHOV_M(1,1,1), &
              TMP_LOC1(1,1,1), wrk3d)
         IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
            DO k=1,kmax
               DO ij=1,imax*jmax
                  RHOV_M(ij,1,k) = TMP_LOC1(ij,1,k)
               ENDDO
            ENDDO
         ELSE 
! ARM formulation
! not yet impllemented in new version
!            CALL SGS_ARM_UM2(n0, n1, dx, dy, dz, &
!                 cf2x, cf2y, cf2z, carm_vh, TMP_LOC1(1,1,1), &
!                 RHOV_M(1,1,1), wrk1d, wrk3d)
         ENDIF

      ENDIF

! -----------------------------------------------------------------
!                                                    First term
! -----------------------------------------------------------------
      IF ( inormal .EQ. 1 ) THEN
         DO k=1,kmax
            DO ij=1,imax*jmax
               TMP_LOC1(ij,1,k) = &
                    RHOU_M(ij,1,k)*RHOU_M(ij,1,k)*barvs(ij,1,k,1)
            ENDDO
         ENDDO
      ELSE
         DO k=1,kmax
            DO ij=1,imax*jmax
               TMP_LOC1(ij,1,k) = &
                    RHOU_M(ij,1,k)*RHOV_M(ij,1,k)*barvs(ij,1,k,1)
            ENDDO
         ENDDO
      ENDIF
      CALL FILTH2(i3, imax, jmax, kmax, kmax_total,&
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
           n0, n1, cf2x, cf2y, cf2z, TMP_LOC1(1,1,1), &
           TMP_LOC2(1,1,1), wrk3d)
      DO k=1,kmax
         DO ij=1,imax*jmax
            numerator(ij,1,k) = numerator(ij,1,k) -&
                 TMP_LOC2(ij,1,k)*mij(ij,1,k)
         ENDDO
      ENDDO

! -----------------------------------------------------------------
!                                                   Second term
! -----------------------------------------------------------------
      CALL FILTH2(i3, imax, jmax, kmax, kmax_total,&
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
           n0, n1, cf2x, cf2y, cf2z, RHOU_M(1,1,1), &
           TMP_LOC1(1,1,1), wrk3d)
      IF ( inormal .EQ. 1 ) THEN
         DO k=1,kmax
            DO ij=1,imax*jmax
               numerator(ij,1,k) = numerator(ij,1,k) +&
                    TMP_LOC1(ij,1,k)*TMP_LOC1(ij,1,k)*mij(ij,1,k)*&
                    barvs(ij,1,k,3)
            ENDDO
         ENDDO
         DO k = 1,kmax
            DO ij = 1,imax*jmax
               TMP_LOC1(ij,1,k) = &
                    TMP_LOC1(ij,1,k)*TMP_LOC1(ij,1,k)*barvs(ij,1,k,3)
            ENDDO
         ENDDO
      ELSE
         CALL FILTH2(i3, imax, jmax, kmax, kmax_total,&
              iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
              n0, n1, cf2x, cf2y, cf2z, RHOV_M(1,1,1), &
              TMP_LOC2(1,1,1), wrk3d)
         DO k=1,kmax
            DO ij=1,imax*jmax
               numerator(ij,1,k) = numerator(ij,1,k) +&
                    TMP_LOC1(ij,1,k)*TMP_LOC2(ij,1,k)*mij(ij,1,k)*&
                    barvs(ij,1,k,3)
            ENDDO
         ENDDO
      ENDIF

#undef RHOU_M
#undef RHOV_M
#undef TMP_LOC1
#undef TMP_LOC2

      RETURN
      END
