SUBROUTINE SGS_HEAT_BS(x, y, z, dx, dy, dz, rho, &
     u, z1, gama, omegap, txc, chi, wrk1d, wrk2d, wrk3d,&
     cf0x, cf0y, cf0z, fdf_mean, fdf_var, fdf_XF)

! txc size is 7

#include "types.h"
#include "les_const.h"

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL u(imax,jmax,kmax)
  TREAL rho(imax,jmax,kmax)
  TREAL z1(imax,jmax,kmax)
  TREAL gama(imax,jmax,kmax)
  TREAL omegap(imax,jmax,kmax)

  TREAL chi(imax,jmax,kmax)
  TREAL txc(imax,jmax,kmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)
  TREAL cf0x(*), cf0y(*), cf0z(*)
  TREAL fdf_mean(les_fdf_bs_maxmean)
  TREAL fdf_var(les_fdf_bs_maxvar,les_fdf_bs_maxmean)
  TREAL fdf_XF(les_fdf_bs_maxvar,les_fdf_bs_maxmean)

  TINTEGER i, j, k, ij, n0, l1, l2
  TREAL AVG_IK, SPL2V2, ERFINV, SIMPSON_NU
  TREAL cda, chi_s,  delta_w, r23
  TREAL dtmp, r0
  TREAL zmin, zmax

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_HEAT_BS')
#endif

  r23 = C_2_R/C_3_R
  r0 = C_0_R

  n0 = isgs_f0size

! prefactor
  cda = LES_CHW_Qp/((gama0-C_1_R)*mach*mach)

! #################
! #  SGS VARIANCE #
! #################
#define C0(j)            wrk1d(j,1)
#define WRK1D_LOC(j)     wrk1d(j,2)

#define TMP1_LOC(i,j,k)  txc(i,j,k,1) 
#define BARVS(i,j,k)     txc(i,j,k,5)
#define RHOZ_M(i,j,k)    txc(i,j,k,6)
#define ZSG(i,j,k)       txc(i,j,k,7)
! Computing c0 and intermediate field
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, rho, BARVS(1,1,1), wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        BARVS(ij,1,k) = C_1_R/BARVS(ij,1,k)
     ENDDO
  ENDDO
! small scale field
  CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
       cf0x, cf0y, cf0z, rho, z1, ZSG(1,1,1), TMP1_LOC(1,1,1), wrk2d, wrk3d)
  IF ( iles_type_recchem .EQ. LES_RECCHEM_SSM ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
!           RHOZ_M(ij,1,k) = rho(ij,1,k)*z1(ij,1,k)
           RHOZ_M(ij,1,k) = rho(ij,1,k)*ZSG(ij,1,k)
        ENDDO
     ENDDO
  ELSE 
! tmp1_loc size is 4
!     CALL SGS_ARM_Z(iles_type_recchem, 
! $        n0, delta_w, x, y, z, dx, dy, dz, 
! $        rho, z1, BARVS(1,1,1), C0(1), RHOZ_M(1,1,1), 
! $        TMP1_LOC(1,1,1), WRK1D_LOC(1), wrk2d, wrk3d,
! $        cf0x, cf0y, cf0z)
     CALL SGS_ARM_Z(iles_type_recchem, &
          n0, delta_w, x, y, z, dx, dy, dz, &
          rho, ZSG(1,1,1), BARVS(1,1,1), C0(1), RHOZ_M(1,1,1), &
          TMP1_LOC(1,1,1), WRK1D_LOC(1), wrk2d, wrk3d,&
          cf0x, cf0y, cf0z)
  ENDIF

#undef TMP1_LOC
#undef ZSG

#define Z2SG(i,j,k)       txc(i,j,k,1)
#define BARRZ_M(i,j,k)    txc(i,j,k,2)
#define TMP1_LOC(i,j,k)   txc(i,j,k,3) 

! Computing the variance
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, RHOZ_M(1,1,1), BARRZ_M(1,1,1), wrk3d)
  CALL SGS_RECTN_TAUIJ(n0, rho, RHOZ_M(1,1,1), RHOZ_M(1,1,1), &
       BARVS(1,1,1), BARRZ_M(1,1,1), BARRZ_M(1,1,1), &
       Z2SG(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)

  DO k = 1,kmax
     DO ij = 1,imax*jmax
        Z2SG(ij,1,k) = Z2SG(ij,1,k)/rho(ij,1,k)
! assuming a positive-definite filter
        dtmp = z1(ij,1,k)*(C_1_R-z1(ij,1,k))
        Z2SG(ij,1,k) = MAX(MIN(Z2SG(ij,1,k),dtmp),C_0_R)
     ENDDO
  ENDDO

#undef BARRZ_M
#undef TMP1_LOC
#undef BARVS
#undef RHOZ_M

#undef WRK1D_LOC

! #####################################
! # scalar dissipation and total term #
! #####################################
#define FDF_Zs(i,j,k)    txc(i,j,k,2)
#define TMP1_LOC(i,j,k)  txc(i,j,k,3) 
#define TMP2_LOC(i,j,k)  txc(i,j,k,4) 
#define TMP3_LOC(i,j,k)  txc(i,j,k,5) 

#define fU_y(j)        wrk1d(j,2)
#define rR(j)          wrk1d(j,3)
#define fU(j)          wrk1d(j,4)
#define WRK1D_LOC1(j)  wrk1d(j,5)
#define WRK1D_LOC2(j)  wrk1d(j,6)
  IF ( iles_type_disZchem .EQ. LES_DISZCHEM_MEAN ) THEN
! --------------------------------------------------------------------
!                                          Model based on the mean
! --------------------------------------------------------------------
! simplified case. No table required.
     CALL CLIP(les_fdf_bs_file, l1, l2)
     IF ( les_fdf_bs_file(l1:l2) .EQ. 'simplified' ) THEN

        DO j = 1,jmax
           rR(j) = AVG_IK(imax, jmax, kmax, j, rho, &
                dx, dz, area)
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = rho(i,j,k)*u(i,j,k)
              ENDDO
           ENDDO
           fU(j) = AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, &
                area)/rR(j)
        ENDDO
        CALL PARTIAL_Y(imode_fdm, i1, jmax, i1, j1bc, dy, fU(1), &
             fU_y(1), i0, i0, WRK1D_LOC1(1), wrk2d, wrk3d)

        delta_w = C_1_R/MAXVAL(ABS(fU_y(1:jmax)))
        chi_s = 0.009/delta_w

! complete case using table
     ELSE

        CALL FILTERED_CHI(n0, x, y, z, dx, dy, dz, z1, chi,&
             TMP1_LOC(1,1,1), TMP2_LOC(1,1,1), TMP3_LOC(1,1,1), &
             WRK1D_LOC1(1), wrk2d, wrk3d,&
             cf0x, cf0y, cf0z)

        DO j = 1,jmax
           rR(j) = AVG_IK(imax, jmax, kmax, j, rho, &
                dx, dz, area)
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = rho(i,j,k)*z1(i,j,k)
                 wrk3d(i,2,k) = wrk3d(i,1,k)*z1(i,j,k)
                 wrk3d(i,3,k) = rho(i,j,k)*Z2SG(i,j,k)
              ENDDO
           ENDDO
           fU(j) = AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, &
                area)/rR(j)
           fU_y(j) = AVG_IK(imax, jmax, kmax, i2, wrk3d, dx, dz, &
                area)/rR(j)-fU(j)*fU(j)
           fU_y(j) = AVG_IK(imax, jmax, kmax, i3, wrk3d, dx, dz, &
                area)/rR(j)+fU_y(j)
           WRK1D_LOC1(j) = AVG_IK(imax, jmax, kmax, j, chi, &
                dx, dz, area)
        ENDDO

        zmin = fdf_mean(2)
        zmax = fdf_mean(les_fdf_bs_maxmean-1)
        DO j = 1,jmax
           IF ( fU(j) .LE. zmin .OR.&
                fU(j) .GT. zmax ) THEN
              WRK1D_LOC2(j) = C_0_R
           ELSE
              WRK1D_LOC2(j) = rR(j)*&
                   SPL2V2(les_fdf_bs_maxvar, &
                   fdf_var, les_fdf_bs_maxmean, fdf_mean,&
                   fdf_XF, fU_y(j), fU(j))
           ENDIF
        ENDDO

        chi_s = SIMPSON_NU(jmax, WRK1D_LOC1(1), y)/&
             SIMPSON_NU(jmax, WRK1D_LOC2(1), y)

     ENDIF

! Impose exactly same chi_s in all PEs
#ifdef PARALLEL
     CALL MPI_BCAST(chi_s, 1, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
#endif

! Computing the beta FDF
     CALL FDF_BETA_STO(imax, jmax, kmax, XIST, z1, Z2SG(1,1,1), &
          FDF_Zs(1,1,1))

! heat release term
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           omegap(ij,1,k) = cda*(gama(ij,1,k)-C_1_R)*rho(ij,1,k)*&
                chi_s*FDF_Zs(ij,1,k)
        ENDDO
     ENDDO

  ELSE IF ( iles_type_disZchem .EQ. LES_DISZCHEM_ONEDIM ) THEN
! ----------------------------------------------------------------------
!                                 Model based on the laminar 1D case 
! ----------------------------------------------------------------------
     CALL FILTERED_CHI(n0, x, y, z, dx, dy, dz, z1, chi,&
          TMP1_LOC(1,1,1), TMP2_LOC(1,1,1), TMP3_LOC(1,1,1), &
          WRK1D_LOC1(1), wrk2d, wrk3d,&
          cf0x, cf0y, cf0z)

! simplified case. No table required.
     CALL CLIP(les_fdf_bs_file, l1, l2)
     IF ( les_fdf_bs_file(l1:l2) .EQ. 'simplified' ) THEN
! Computing the beta FDF
        CALL FDF_BETA_STO(imax, jmax, kmax, XIST, z1, &
             Z2SG(1,1,1), FDF_Zs(1,1,1))
        chi_s = EXP(-C_2_R*(ERFINV(C_1_R-C_2_R*XIST))**2)
        zmin = C_1EM2_R
        zmax = C_1_R-C_1EM2_R
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              dtmp = MAX(zmin,MIN(zmax,z1(ij,1,k)))
              FDF_Zs(ij,1,k) =chi_s*FDF_Zs(ij,1,k)/&
                   EXP(-C_2_R*(ERFINV(C_1_R-C_2_R*dtmp))**2)
! heat release term
              omegap(ij,1,k) = cda*(gama(ij,1,k)-C_1_R)*&
                   chi(ij,1,k)*FDF_Zs(ij,1,k)
           ENDDO
        ENDDO
! full case. Table.
     ELSE
! zmin = C_1EM2_R
! zmax = C_1_R-C_1EM2_R
        zmin = fdf_mean(2)
        zmax = fdf_mean(les_fdf_bs_maxmean-1)
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              IF ( z1(ij,1,k) .LE. zmin .OR.&
                   z1(ij,1,k) .GT. zmax ) THEN
                 FDF_Zs(ij,1,k) = C_0_R
                 omegap(ij,1,k) = C_0_R
              ELSE
                 FDF_Zs(ij,1,k) = SPL2V2(les_fdf_bs_maxvar, &
                      fdf_var, les_fdf_bs_maxmean, fdf_mean,&
                      fdf_XF, Z2SG(ij,1,k), z1(ij,1,k))
! heat release term
                 omegap(ij,1,k) = cda*(gama(ij,1,k)-C_1_R)*&
                      chi(ij,1,k)*FDF_Zs(ij,1,k)
              ENDIF
           ENDDO
        ENDDO
     ENDIF
  ENDIF

#undef C0
#undef fU_y
#undef rR
#undef fU
#undef WRK1D_LOC1
#undef WRK1D_LOC2

#undef FDF_Zs
#undef Z2SG
#undef TMP1_LOC
#undef TMP2_LOC
#undef TMP3_LOC

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_HEAT_BS' )
#endif

  RETURN
END SUBROUTINE SGS_HEAT_BS
