!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# barvs is the specific volume 1/bar0(bar0(rho))
!#
!# Written in terms of momentum per unit volume, m_i=rho*u_i:
!# barmu_m is bar0((rho*u)_model)
!# barmv_m is bar0((rho*v)_model)
!# barmw_m is bar0((rho*w)_model)
!#
!# txc size is 5
!########################################################################
SUBROUTINE SGS_RECTN(statflg, x,y,z,dx,dy,dz, q,z1, &
     u_sg,v_sg, w_sg,aux_sg, h1,h2,h3,h4,zh1, mean1d_les, &
     mu_m,mv_m,mw_m, barvs, barmu_m,barmv_m,barmw_m, txc, chi, wrk1d,wrk2d,wrk3d, &
     cf0x,cf0y,cf0z, carm_uf,carm_pf,carm_zf)

#include "types.h"
#include "les_avgij_map.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER statflg

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: u_sg, v_sg, w_sg, aux_sg, h1, h2, h3, h4, chi
  TREAL, DIMENSION(imax,jmax,kmax)   :: mu_m, mv_m, mw_m, barmu_m, barmv_m, barmw_m, barvs
  TREAL, DIMENSION(imax,jmax,kmax,*) :: q, z1, zh1, txc
  TREAL, DIMENSION(nstatavg,jmax,*)  :: mean1d_les
  TREAL, DIMENSION(*)                :: wrk1d, wrk2d, wrk3d

  TREAL, DIMENSION(*) :: cf0x, cf0y, cf0z
  TREAL, DIMENSION(*) :: carm_uf, carm_pf, carm_zf

  TARGET :: q

! -----------------------------------------------------------------------
  TINTEGER ij, k, n0, is, statflg_loc

! Pointers to existing allocated space
  TREAL, DIMENSION(:,:,:),   POINTER :: u, v, w, e, rho

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_RECTN' )
#endif

! Define pointers
  u   => q(:,:,:,1)
  v   => q(:,:,:,2)
  w   => q(:,:,:,3)
  e   => q(:,:,:,4)
  rho => q(:,:,:,5)

  n0 = isgs_f0size

  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, rho, barvs, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        barvs(ij,1,k) = C_1_R/barvs(ij,1,k)
     ENDDO
  ENDDO

! #######################################################################
! Defining the intermediate fields
! #######################################################################
! ------------------------------------------------------------
!                              small resolved scale fields
! ------------------------------------------------------------
! txc_size=1
!  CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!       cf0x, cf0y, cf0z, rho, u, u_sg, txc, wrk2d, wrk3d)
!  CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!       cf0x, cf0y, cf0z, rho, v, v_sg, txc, wrk2d, wrk3d)
!  CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!       cf0x, cf0y, cf0z, rho, w, w_sg, txc, wrk2d, wrk3d)

  IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
!           mu_m(ij,1,k) = rho(ij,1,k)*u_sg(ij,1,k)
!           mv_m(ij,1,k) = rho(ij,1,k)*v_sg(ij,1,k)
!           mw_m(ij,1,k) = rho(ij,1,k)*w_sg(ij,1,k)
           mu_m(ij,1,k) = rho(ij,1,k)*u(ij,1,k)
           mv_m(ij,1,k) = rho(ij,1,k)*v(ij,1,k)
           mw_m(ij,1,k) = rho(ij,1,k)*w(ij,1,k)
        ENDDO
     ENDDO
  ELSE 
! txc size is 4
! not yet impllemented in new version
!         CALL SGS_ARM_U(n0, x, y, z, dx, dy, dz, &
!              rho, u_sg, v_sg, w_sg, barvs, carm_uf, mu_m, mv_m, mw_m,&
!              txc, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z) 
  ENDIF

  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, mu_m, barmu_m, wrk3d)
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, mv_m, barmv_m, wrk3d)
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, mw_m, barmw_m, wrk3d)

! #######################################################################
! Momentum equation
! #######################################################################
  CALL SGS_RECTN_U(n0, statflg, dx, dy, dz, rho, u, v, w, &
       mu_m, mv_m, mw_m, barvs, barmu_m, barmv_m, barmw_m,&
       h1, h2, h3, h4, mean1d_les, &
       txc, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z)

! #######################################################################
! Energy equation
! #######################################################################
  statflg_loc =-statflg
  CALL SGS_RECTN_Z(n0, statflg_loc, x, y, z, dx, dy, dz, rho, e, aux_sg, &
       mu_m, mv_m, mw_m, barvs, barmu_m, barmv_m, barmw_m, h4, &
       mean1d_les, &
       txc, chi, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z, carm_pf)

! #######################################################################
! Scalar equation
! #######################################################################
  IF ( icalc_scal .EQ. 1 ) THEN
     DO is = 1,inb_scal
        CALL SGS_RECTN_Z(n0, statflg, x, y, z, dx, dy, dz, rho, z1(1,1,1,is), aux_sg, &
             mu_m, mv_m, mw_m, barvs, barmu_m, barmv_m, barmw_m, zh1(1,1,1,is), &
             mean1d_les(1,1,1+LA_MOMENTUM_SIZE+(is-1)*LS_SCALAR_SIZE),&
             txc, chi, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z, carm_zf)
     ENDDO
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_RECTN' )
#endif

  RETURN
END SUBROUTINE SGS_RECTN
