!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2001/11/30 - J.P. Mellado
!#              Created
!# 2008/11/16 - J.P. Mellado
!#              New code version based on energy
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# barvs is the specific volume 1/bar0(bar0(rho))
!#
!# Written in terms of momentum per unit volume, m_i=rho*u_i:
!# barmu_m is bar0((rho*u)_model)
!# barmv_m is bar0((rho*v)_model)
!# barmw_m is bar0((rho*w)_model)
!# 
!# txc size is 4 if SSM
!# txc size is 5 if ARM
!#
!########################################################################
SUBROUTINE SGS_RECTN_Z(n0, statflg, x, y, z, dx, dy, dz, rho, z1, z1_sg, &
     mu_m, mv_m, mw_m, barvs, barmu_m, barmv_m, barmw_m, zh1, mean1d_les_sc, &
     txc, chi, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z, carm_zf)

#include "types.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER n0, statflg

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: rho, z1, z1_sg, zh1
  TREAL, DIMENSION(imax,jmax,kmax)   :: mu_m, mv_m, mw_m, barmu_m, barmv_m, barmw_m, barvs
  TREAL, DIMENSION(imax,jmax,kmax,*) :: txc

  TREAL mean1d_les_sc(nstatavg,jmax,*)

  TREAL chi(imax,jmax,kmax)

  TREAL, DIMENSION(*) :: wrk1d, wrk2d, wrk3d
  
  TREAL, DIMENSION(*) :: cf0x, cf0y, cf0z
  TREAL, DIMENSION(*) :: carm_zf

! -----------------------------------------------------------------------
  TREAL r1
  TINTEGER ij, k, idissflg

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_RECTN_Z' )
#endif

  r1 = C_1_R

  IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS .OR. statflg .NE. 0 ) THEN
     idissflg = 1
  ELSE
     idissflg = 0
  ENDIF

#define RZ_M(i,j,k)     txc(i,j,k,1)
#define TMP1_LOC(i,j,k) txc(i,j,k,2)

! #######################################################################
! Defining the model fields
! #######################################################################
! ------------------------------------------------------------
!                              small resolved scale fields
! ------------------------------------------------------------
! txc_size=1
!  CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!       cf0x, cf0y, cf0z, rho, z1, z1_sg, txc, wrk2d, wrk3d)

  IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           RZ_M(ij,1,k) = rho(ij,1,k)*z1(ij,1,k)
!           RZ_M(ij,1,k) = rho(ij,1,k)*z1_sg(ij,1,k)
        ENDDO
     ENDDO
  ELSE 
! tmp1_loc size is 4
! not yet implemented in new version
!         CALL SGS_ARM_Z(iles_type_tran,&
!              n0, r1, x, y, z, dx, dy, dz, &
!              rho, z1_sg, barvs, carm_zf, RZ_M(1,1,1), &
!              TMP1_LOC(1,1,1), wrk1d, wrk2d, wrk3d,&
!              cf0x, cf0y, cf0z)
  ENDIF

#undef TMP1_LOC

#define BARRZ_M(i,j,k)  txc(i,j,k,2)
#define Q_M(i,j,k)      txc(i,j,k,3)
#define TMP1_LOC(i,j,k) txc(i,j,k,4)
! !!!! this last two have to be contiguous for LES_SAVE_SCBDGIJ

  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, RZ_M(1,1,1), BARRZ_M(1,1,1), wrk3d)

! #######################################################################
! SGS flux vector
! #######################################################################
! -----------------------------------------------------------------    
!                                             qx = (Z_m*U_m)_sg
! -----------------------------------------------------------------    
  CALL SGS_RECTN_TAUIJ(n0, rho, RZ_M(1,1,1), mu_m, &
       barvs, BARRZ_M(1,1,1), barmu_m, Q_M(1,1,1),&
       TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
! Dissipation, -q_x*dz1/dx
  IF ( idissflg .EQ. 1 ) THEN
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
          z1, TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              chi(ij,1,k) = chi(ij,1,k) - Q_M(ij,1,k)*TMP1_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_SCBDGIJ( i5, dx, dy, dz, z1, Q_M(1,1,1), &
             mean1d_les_sc, wrk1d, wrk2d, wrk3d )
     ELSE IF ( statflg .EQ. -1 ) THEN
        CALL LES_SAVE_AVGPIJ( i5, dx, dy, dz, z1, Q_M(1,1,1), &
             mean1d_les_sc, wrk1d, wrk2d, wrk3d )
     ENDIF
  ENDIF
! Calculate  - d/dx(qx)                (Z equation)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
       Q_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  DO k=1, kmax
     DO ij=1,imax*jmax
        zh1(ij,1,k) = zh1(ij,1,k) - sgs_alpha*TMP1_LOC(ij,1,k)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i10, dx, dy, dz, z1, &
          TMP1_LOC(1,1,1), mean1d_les_sc, wrk1d, wrk2d, wrk3d)
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i10, dx, dy, dz, z1, &
          TMP1_LOC(1,1,1), mean1d_les_sc, wrk1d, wrk2d, wrk3d)
  ENDIF

! -----------------------------------------------------------------    
!                                             qy = (Z_m*V_m)_sg
! -----------------------------------------------------------------    
  CALL SGS_RECTN_TAUIJ(n0, rho, RZ_M(1,1,1), mv_m, &
       barvs, BARRZ_M(1,1,1), barmv_m, Q_M(1,1,1),&
       TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
! Statistics
  IF ( idissflg .EQ. 1 ) THEN
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
          z1, TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              chi(ij,1,k) = chi(ij,1,k) - Q_M(ij,1,k)*TMP1_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_SCBDGIJ( i6, dx, dy, dz, z1, Q_M(1,1,1), &
             mean1d_les_sc, wrk1d, wrk2d, wrk3d )
     ELSE IF ( statflg .EQ. -1 ) THEN
        CALL LES_SAVE_AVGPIJ( i6, dx, dy, dz, z1, Q_M(1,1,1), &
             mean1d_les_sc, wrk1d, wrk2d, wrk3d )
     ENDIF
  ENDIF
! Calculate  - d/dy(qy)                (Z equation)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
       Q_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  DO k=1, kmax
     DO ij=1,imax*jmax
        zh1(ij,1,k) = zh1(ij,1,k) - sgs_alpha*TMP1_LOC(ij,1,k)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i10, dx, dy, dz, z1, &
          TMP1_LOC(1,1,1), mean1d_les_sc, wrk1d, wrk2d, wrk3d)
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i10, dx, dy, dz, z1, &
          TMP1_LOC(1,1,1), mean1d_les_sc, wrk1d, wrk2d, wrk3d)
  ENDIF

  IF ( kmax_total .GT. 1 ) THEN
! -----------------------------------------------------------------    
!                                             qz = (Z_m*W_m)_sg
! -----------------------------------------------------------------    
     CALL SGS_RECTN_TAUIJ(n0, rho, RZ_M(1,1,1), mw_m, &
          barvs, BARRZ_M(1,1,1), barmw_m, Q_M(1,1,1),&
          TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
! Statistics
     IF ( idissflg .EQ. 1 ) THEN
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
             z1, TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
           DO k = 1,kmax
              DO ij = 1,imax*jmax
                 chi(ij,1,k) = chi(ij,1,k) - Q_M(ij,1,k)*TMP1_LOC(ij,1,k)
              ENDDO
           ENDDO
        ENDIF
        IF ( statflg .EQ. 1 ) THEN
           CALL LES_SAVE_SCBDGIJ( i7, dx, dy, dz, z1, Q_M(1,1,1), &
                mean1d_les_sc, wrk1d, wrk2d, wrk3d )
        ELSE IF ( statflg .EQ. -1 ) THEN
           CALL LES_SAVE_AVGPIJ( i7, dx, dy, dz, z1, Q_M(1,1,1), &
                mean1d_les_sc, wrk1d, wrk2d, wrk3d )
        ENDIF
     ENDIF
! Calculate  - d/dz(qz)                (Z equation)
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
          Q_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1, kmax
        DO ij=1,imax*jmax
           zh1(ij,1,k) = zh1(ij,1,k) - sgs_alpha*TMP1_LOC(ij,1,k)
        ENDDO
     ENDDO
! Statistics
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_SCBDGIJ(i11, dx, dy, dz, z1, &
             TMP1_LOC(1,1,1), mean1d_les_sc, wrk1d, wrk2d, wrk3d)
     ELSE IF ( statflg .EQ. -1 ) THEN
        CALL LES_SAVE_AVGPIJ(i11, dx, dy, dz, z1, &
             TMP1_LOC(1,1,1), mean1d_les_sc, wrk1d, wrk2d, wrk3d)
     ENDIF
  ENDIF

#undef RZ_M
#undef BARRZ_M
#undef Q_M
#undef TMP1_LOC

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_RECTN_Z' )
#endif

  RETURN
END SUBROUTINE SGS_RECTN_Z
