!# ##################################################################
!#
!#  SUBROUTINE KXY2D_EXECUTE
!#  Purpose:   Execute fftw_calls for 2d spectral transformation 
!#             in 2 steps (1D, transposed 1D)
!#
!#  Arguments: 
#include "types.h"

SUBROUTINE KXZ2D_EXECUTE(direction,nx,ny,nz,ikmax,i,j,k,&
     u,fft_dataR,fft_dataC,fft_plan)

#ifdef USE_OPENMP
  USE OMP_LIB
#endif

  IMPLICIT NONE

#include "integers.h"
  TINTEGER,  INTENT(IN) :: j 
  TINTEGER,  INTENT(IN) :: nx,ny,nz,ikmax
  TINTEGER,  INTENT(IN) :: direction
  INTEGER(8),INTENT(IN) :: fft_plan

  TCOMPLEX, INTENT(INOUT),DIMENSION(2*(ikmax+2))   :: fft_dataC
  TREAL,    INTENT(INOUT),DIMENSION(2*nx)          :: fft_dataR
  TINTEGER, INTENT(INOUT)   :: i,k 


  TREAL, INTENT(INOUT)  :: u(nx,ny,nz)
  TREAL, PARAMETER :: SQRT_C_05_R = SQRT(C_05_R)

  IF ( direction .EQ. i1 ) THEN
    DO k = 1,nz
        DO i = 1,nx
           fft_dataR(i) = u(i,j,k)
        ENDDO

        CALL dfftw_execute_dft_r2c(fft_plan, fft_dataR, fft_dataC)
        
        u(1,j,k) = SQRT_C_05_R * REAL(fft_dataC(1))
        u(2,j,k) = C_0_R
        DO i=2,nx/2
           u(2*i-1,j,k) = REAL(fft_dataC(i))
           u(2*i,  j,k) = AIMAG(fft_dataC(i)) 
        ENDDO
     ENDDO

  ELSE IF ( direction .EQ. i3 ) THEN
     DO i=1,nx/2
        DO k=1,nz
           fft_dataC(k) = CMPLX(u(2*i-1,j,k),u(2*i,j,k))
        ENDDO
        
        CALL dfftw_execute_dft(fft_plan, fft_dataC, fft_dataC)
        
        DO k=1,nz
           u(2*i-1,j,k)  = ABS(fft_dataC(k))
           u(2*i,j,k)    = ATAN2(AIMAG(fft_dataC(k)), REAL(fft_dataC(k)))
        ENDDO
     ENDDO
     
  ENDIF

  
END SUBROUTINE KXZ2D_EXECUTE

!# ###################################################################
!# E N D    O F
!# S U B R O U T I N E   K X Y 2 D _ E X E C U T E
!# ###################################################################

!# ##################################################################
!#
!#  SUBROUTINE KXZ2D_PSD
!#  Purpose:   Calculate Power spectral density and phase from 
!#             complex 2D spectrum
!#             No Normalization !
!#  Arguments: 
#include "types.h"

SUBROUTINE KXZ2D_PSD(iwrite,jwrite,kwrite,nv,j,j_out,jmax,nblock,iv,&
     wrk3d,pow_out,phase_out)

  IMPLICIT NONE

! Input 
  TINTEGER                       :: iwrite, jwrite, kwrite, nv, nblock, j,j_out,jmax
  TREAL,    DIMENSION(2*iwrite,jmax,kwrite) :: wrk3d

! Output
  TREAL,    DIMENSION(kwrite,jwrite,iwrite,nv) :: pow_out, phase_out

! Local Variables
  TINTEGER  i,k,iv
  TINTEGER  offset


  offset = kwrite/2
  
  DO k = 1,kwrite/2
     DO i = 1,iwrite  
        pow_out  (k+offset,j_out,i,iv) = &
             wrk3d(2*i-1,j,k) + pow_out  (k+offset,j_out,i,iv)
        phase_out(k+offset,j_out,i,iv) = &
             wrk3d(2*i,  j,k) + phase_out(k+offset,j_out,i,iv)
     ENDDO
  ENDDO
     
  DO k = kwrite/2+1,kwrite
     DO i = 1,iwrite
        pow_out  (k-offset,j_out,i,iv) = &
             wrk3d(2*i-1,j,k) + pow_out  (k-offset,j_out,i,iv)
        phase_out(k-offset,j_out,i,iv) = &
             wrk3d(2*i,  j,k) + phase_out(k-offset,j_out,i,iv)
     ENDDO
  ENDDO
  
END SUBROUTINE KXZ2D_PSD

!# ###################################################################
!# E N D    O F
!# S U B R O U T I N E   K X Z 2 D _ P S D
!# ###################################################################

SUBROUTINE KXZ2D_MERGE_FIELDS(in1, in2, kwrite,jwrite,iwrite,&
  out, imax,jmax,kmax) 
  
  IMPLICIT NONE 

  TINTEGER :: iwrite,jwrite,kwrite,imax,jmax,kmax
  TINTEGER :: i,j,k,add1,add2,npage,nline
  TREAL, DIMENSION(kwrite,jwrite,iwrite) :: in1,in2
  TREAL, DIMENSION(imax*jmax*kmax) :: out

  npage = imax*jmax
  nline = imax 

  DO i=1,iwrite
     DO j=1,jwrite
        DO k=1,kwrite
           add1 = (k-1) * npage + (j-1) * nline + 2*i-1
           add2 = add1+1
           out(add1) = in1(k,j,i) 
           out(add2) = in2(k,j,i) 
        ENDDO
     ENDDO
  ENDDO

END SUBROUTINE KXZ2D_MERGE_FIELDS

SUBROUTINE KXZ2D_SPLIT_FIELDS(in, o1, o2, imax,jmax,kmax, iwrite, jwrite, kwrite) 

  IMPLICIT NONE

  TINTEGER, INTENT(IN)                                  :: iwrite,jwrite,kwrite, imax,jmax,kmax 
  TREAL,    INTENT(IN), DIMENSION(imax*jmax*kmax)       :: in 
  TREAL,    INTENT(OUT),DIMENSION(iwrite,jwrite,kwrite) :: o1,o2

  TINTEGER i,j,k,add1,add2,npage, nline
  
  npage = imax*jmax
  nline = imax
  DO k=1,kwrite
     DO j=1,jwrite
        DO i=1,iwrite
           add1 = (k-1) * npage + (j-1)*nline + 2*i-1 
           add2 = add1+1
           o1(i,j,k) = in(add1) 
           o2(i,j,k) = in(add2) 
        ENDDO
     ENDDO
  ENDDO

END SUBROUTINE KXZ2D_SPLIT_FIELDS
