!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 1999/01/01 - C. Pantano
!#              Created
!# 2008/04/17 - J.P. Mellado
!#              Introducing FFTW
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# spectrum  Out  Power spectral density
!# fft_data  In   Array with FFT information, size 2*nx+15 in old fft
!# fft_plan  In   Pointer to FFTW information
!#
!########################################################################
#include "types.h"

SUBROUTINE KZ1D_EXECUTE(nx,ny,nz, j, u, fft_plan, fft_data, spectrum)
  
  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER nx, ny, nz
  TREAL u(nx,ny,nz)
#ifdef USE_FFTW
  TREAL fft_data(nz+2,*)
#else
  TREAL fft_data(2*nz+15,*)
#endif
  TREAL spectrum(nz/2)
  INTEGER(8) fft_plan

! -----------------------------------------------------------------------
  TINTEGER i, j, k

#ifdef PARALLEL
  INTEGER ims_err, itmp
  TINTEGER nx_total
#endif

! #######################################################################
  spectrum(:) = C_0_R

! #######################################################################
! Loop over spanwise direction
! #######################################################################
  DO i = 1,nx
     fft_data(1:nz,1) = u(i,j,1:nz)

#ifdef USE_FFTW
! Perform 1D fft
     CALL dfftw_execute_dft_r2c(fft_plan, fft_data(1,1), fft_data(1,2))
! power spectral density
     spectrum(1) = spectrum(1) + (fft_data(1,2)**2)
     DO k = 2,nz/2
        spectrum(k) = spectrum(k) + fft_data(2*k-1,2)**2 + fft_data(2*k,2)**2
     ENDDO

#else
! Perform 1D fft; fft_data(1,1) is over-written, fft_data(1,2) aux array of size 2*nz+15
     CALL RFFTF(nz, fft_data(1,1), fft_data(1,2))
! power spectral density
     spectrum(1) = spectrum(1) + (fft_data(1,1)**2)
     DO k = 2, nz/2
        spectrum(k) = spectrum(k) + fft_data(2*k-2,1)**2 + fft_data(2*k-1,1)**2
     ENDDO
#endif

  ENDDO

! -----------------------------------------------------------------------
! Final average operation
! -----------------------------------------------------------------------
#ifdef PARALLEL
  CALL MPI_ALLREDUCE(nx, nx_total, 1, MPI_INTEGER4, MPI_SUM, MPI_COMM_WORLD, ims_err)
  itmp = nz/2
  CALL MPI_REDUCE(spectrum, fft_data, itmp, MPI_REAL8, MPI_SUM, 0, MPI_COMM_WORLD, ims_err)

  spectrum(1:nz/2) = fft_data(1:nz/2,1)/M_REAL(nx_total*nz)

#else
  spectrum(1:nz/2) = spectrum(1:nz/2)/M_REAL(nz*nx)

#endif

  RETURN
END SUBROUTINE KZ1D_EXECUTE
