#include "types.h"
#include "dns_error.h"
#define NVARS_LOC 12

!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 1999/01/01 - C. Pantano
!#              Created
!# 2008/04/17 - J.P. Mellado
!#              Introducing FFTW
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# fft_data    In    Array for old FFT data, size 2*nx+15
!#
!########################################################################
SUBROUTINE KZ2D_N(prefix, varname, nx,ny,nz, itime, rtime, nvar, data, pow, fft_data)

  USE DNS_TYPES,  ONLY : pointers_structure
  USE DNS_GLOBAL, ONLY : efile

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif
#ifdef USE_FFTW
#include "fftw3.f"
#endif

  CHARACTER*(*) prefix
  CHARACTER*32 varname(nvar)
  TINTEGER nx,ny,nz, itime, nvar
  TREAL rtime
  TREAL pow(nz/2,nvar) 
#ifdef USE_FFTW
  TREAL fft_data(nz+2,2)
#else
  TREAL fft_data(2*nz+15,2)
#endif

  TYPE(pointers_structure), DIMENSION(nvar) :: data

! -----------------------------------------------------------------------
  TINTEGER j, iv, n
  CHARACTER*32 fname
  CHARACTER*512 line1
  INTEGER(8) fft_plan

#ifdef PARALLEL
  INTEGER ims_pro, ims_err
#endif

! #######################################################################
  IF ( NVARS_LOC .LT. nvar ) THEN
     CALL IO_WRITE_ASCII(efile, 'KX2D_N: Format length too sort.')
     CALL DNS_STOP(DNS_ERROR_WRKSIZE)
  ENDIF

  IF ( nz .EQ. 1 ) RETURN

! -----------------------------------------------------------------------
! FFTW
! -----------------------------------------------------------------------
#ifdef USE_FFTW
  CALL dfftw_plan_dft_r2c_1d(fft_plan,nz,fft_data(1,1),fft_data(1,2),FFTW_ESTIMATE)
! -----------------------------------------------------------------------
! old fft
! -----------------------------------------------------------------------
#else
  CALL RFFTI(nz, fft_data)
#endif

! -------------------------------------------------------------------
! TkStat file; header
! -------------------------------------------------------------------
#ifdef PARALLEL
  CALL MPI_COMM_RANK(MPI_COMM_WORLD,ims_pro,ims_err)

  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(fname,*) itime; fname = TRIM(ADJUSTL(prefix))//TRIM(ADJUSTL(fname))

     OPEN(unit=23,file=fname)
     WRITE(23, '(A8,E14.7E3)') 'RTIME = ', rtime
     WRITE(23, '(A7,I8)') 'IMAX = ', i1
     WRITE(23, '(A7,I8)') 'JMAX = ', ny
     WRITE(23, '(A7,I8)') 'IVAR = ', i1
     line1 = 'I J Kz '
     DO iv = 1,nvar
        line1 = TRIM(ADJUSTL(line1))//' '//TRIM(ADJUSTL(varname(iv)))
     ENDDO
     WRITE(23,'(A)') TRIM(ADJUSTL(line1))

#ifdef PARALLEL
  ENDIF
#endif

! ###################################################################
! Spectra calculation
! ###################################################################
  DO j = 1,ny
     DO iv = 1,nvar
        CALL KZ1D_EXECUTE(nx,ny,nz, j, data(iv)%field(1),  fft_plan, fft_data, pow(1,iv))
     ENDDO

! -------------------------------------------------------------------
! TkStat output
! -------------------------------------------------------------------
#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        DO n = 1,nz/2
           WRITE(23,1020) 1, j, n, (pow(n,iv),iv=1,nvar)
        ENDDO
#ifdef PARALLEL
     ENDIF
#endif

  ENDDO

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CLOSE(23)
#ifdef PARALLEL
  ENDIF
#endif

! ###################################################################
#ifdef USE_FFTW
  CALL dfftw_destroy_plan(fft_plan)
#endif

  RETURN

1020 FORMAT(I5,2(1X,I5),NVARS_LOC(1X,G_FORMAT_R))

END SUBROUTINE KZ2D_N

