!########################################################################
!# Tool DNS
!#
!########################################################################
!# HISTORY
!#
!# 2007/06/09 - J.P. Mellado
!#              Created
!# 2007/06/20 - J.P. Mellado
!#              AIRWATER case included.
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate rho from Cp from composition and gama
!#
!# C_p = \gamma/(\gamma-1)*R
!#
!# and nondimensional
!#
!# C_p = \gamma/(\gamma-1)*(R^0/(C_{p,0}W_0))\sum Y_i/W_i
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################















		   



			   
               	           







! File formats

! Flow Types

! Flow Mode

! Equations mode

! Equation terms

 





! Finite-differences method


! Runge-Kutta method

! Boundary conditions


! Filters

! Mean profiles

! Chemistry Constants


! Chemistry Type

! Mixture Type


SUBROUTINE THERMO_CP(nx,ny,nz, s,gama, cp)

  USE THERMO_GLOBAL, ONLY : imixture, GRATIO
  USE THERMO_GLOBAL, ONLY : NSP, WGHT_INV

  IMPLICIT NONE

  INTEGER(4) nx,ny,nz
  REAL(8), DIMENSION(nx*ny*nz),   INTENT(IN)  :: gama
  REAL(8), DIMENSION(nx*ny*nz,*), INTENT(IN)  :: s
  REAL(8), DIMENSION(nx*ny*nz),   INTENT(OUT) :: cp

! -------------------------------------------------------------------
  INTEGER(4) ij, is
  REAL(8) WMEAN_INV

! ###################################################################
! Single species
! ###################################################################
  IF ( imixture .EQ. 0 ) THEN
     DO ij = 1,nx*ny*nz
        cp(ij) = 1.0d0
     ENDDO

! ###################################################################
! Mixture defined by a conserved scalar Z, Y_i=f_i(Z)
! ###################################################################
  ELSE IF ( imixture .EQ. 1          .OR.&
            imixture .EQ. 5 .OR.&
            imixture .EQ. 8    ) THEN
!     DO ij = 1,nx*ny*nz
!
!#define MACRO_ZINPUT s(ij,inb_scal)
!#include "dns_chem_mass.h"
!
!        cp(ij) = gama(ij)*GRATIO/((gama(ij)-1.0d0)*WMEAN)
!
!     ENDDO

! ###################################################################
! mixture 10. s(1,2) contains liquid mass fraction
! ###################################################################
  ELSE IF ( imixture .EQ. 10 ) THEN
     DO ij = 1,nx*ny*nz
        WMEAN_INV = WGHT_INV(2) + s(ij,1)*(WGHT_INV(1)-WGHT_INV(2)) - s(ij,2)*WGHT_INV(1)
        cp(ij) = gama(ij)*GRATIO*WMEAN_INV/(gama(ij)-1.0d0)
     ENDDO

! ###################################################################
! General mixture of NSP species. s contains NSP-1 mass fractions
! ###################################################################
  ELSE 
     DO ij = 1,nx*ny*nz

! Compute mean molecular weight of mayor species
        WMEAN_INV = 0.0d0
        DO is = 1,NSP-1
           WMEAN_INV = WMEAN_INV + s(ij,is)*(WGHT_INV(is)-WGHT_INV(NSP))
        ENDDO
        WMEAN_INV = WMEAN_INV + WGHT_INV(NSP)

        cp(ij) = gama(ij)*GRATIO*WMEAN_INV/(gama(ij)-1.0d0)

     ENDDO

  ENDIF

  RETURN
END SUBROUTINE THERMO_CP
