#include "types.h"
#include "dns_const.h"
#include "dns_error.h"

!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2008/01/20 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Writing data in Ensight Gold Format
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
PROGRAM PLOT_MAIN

  USE DNS_GLOBAL
  USE THERMO_GLOBAL, ONLY : imixture
  USE THERMO_GLOBAL, ONLY : NSP, THERMO_SPNAME
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif
#ifdef USE_FFTW
#include "fftw3.f"
#endif

! -------------------------------------------------------------------
! Grid and associated arrays
  TREAL,   DIMENSION(:),   ALLOCATABLE, SAVE :: x,y,z, dx,dy,dz

! Fields
  TREAL,   DIMENSION(:,:), ALLOCATABLE, SAVE :: txc, s, txc_poisson

! Work arrays
  TREAL,   DIMENSION(:),   ALLOCATABLE, SAVE :: wrk1d,wrk2d,wrk3d, mean2d
  TREAL,   DIMENSION(:,:), ALLOCATABLE, SAVE :: surface
  TREAL,   DIMENSION(:),   ALLOCATABLE, SAVE :: tmp_mpi

! Flow partition
  INTEGER(1), ALLOCATABLE, SAVE :: gate(:), MaskFile(:)

  TINTEGER ij, is, isize_wrk3d, inb_poisson_txc, iscal_offset, iread_scal, iheader, idummy
  TINTEGER iread_gate, gate_nmax, igate, gate_levels(16), ierr
  TINTEGER MaskSize
  TREAL diff, umin, umax, dummy
  CHARACTER*32 flow_file, scal_file, aux_file, plot_file, time_str
  CHARACTER*64 str, line
  TINTEGER subdomain(6)

  TINTEGER itime_size_max, itime_size, it
  PARAMETER(itime_size_max=256)
  TINTEGER itime_vec(itime_size_max)

  TINTEGER ivar_size_max, ivar_size, iv
  PARAMETER(ivar_size_max=32)
  TINTEGER ivar_vec(ivar_size_max)

  CHARACTER*512 sRes
  CHARACTER*32 inifile, bakfile

#ifdef PARALLEL
  TINTEGER itmp_mpi
  INTEGER icount
#endif

! ###################################################################
  inifile = 'dns.ini'

  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL(inifile)

#ifdef PARALLEL
  CALL DNS_MPI_INITIALIZE
#endif

  MaskSize = 6
  IF ( idiffusion .EQ. EQNS_NONE ) THEN; diff = C_0_R
  ELSE;                                  diff = visc/schmidt(inb_scal); ENDIF

  isize_wrk3d = isize_txc_field

  bakfile = TRIM(ADJUSTL(inifile))//'.bak'

! -------------------------------------------------------------------
! Allocating memory space
! -------------------------------------------------------------------
  ALLOCATE(x(imax_total))
  ALLOCATE(y(jmax_total))
  ALLOCATE(z(kmax_total))
  ALLOCATE(dx(imax_total*inb_grid))
  ALLOCATE(dy(jmax_total*inb_grid))
  ALLOCATE(dz(kmax_total*inb_grid))

  ALLOCATE(wrk1d(isize_wrk1d*inb_wrk1d))
  ALLOCATE(mean2d(isize_wrk1d*5))

! -------------------------------------------------------------------
! File names
! -------------------------------------------------------------------
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CALL SCANINICHAR(bakfile, inifile, 'PostProcessing', 'Files', '-1', sRes)
     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) 'Integral Iterations ?'
        READ(*,'(A512)') sRes
     ENDIF
     itime_size = itime_size_max
     CALL LIST_INTEGER(sRes, itime_size, itime_vec)
#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST(itime_size, 1,      MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  icount = itime_size
  CALL MPI_BCAST(itime_vec,  icount, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

! -------------------------------------------------------------------
! Read local options
! -------------------------------------------------------------------
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif

     CALL SCANINICHAR(bakfile, inifile, 'PostProcessing', 'Subdomain', '-1', sRes)
     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) 'Subdomain limits ?'
        READ(*,'(A64)') sRes
     ENDIF
     is = 6
     CALL LIST_INTEGER(sRes, is, subdomain)
     IF ( is .LT. 6 ) THEN
        subdomain(1) = 1; subdomain(2) = imax_total
        subdomain(3) = 1; subdomain(4) = jmax_total
        subdomain(5) = 1; subdomain(6) = kmax_total
     ENDIF
        
     igate = 0
     CALL SCANINICHAR(bakfile, inifile, 'PostProcessing', 'Partition', '-1', sRes)
     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) 'Intermittency conditioning ?'
        WRITE(*,*) ' 0. None'
        WRITE(*,*) ' 1. Based on scalar'
        WRITE(*,*) ' 2. Based on vorticity'
        WRITE(*,*) ' 3. Based on scalar gradient'
        WRITE(*,*) ' 4. Based on external file'
        READ(*,'(A64)') sRes
     ENDIF
     READ(sRes,*) igate
     IF ( igate .EQ. 1 .OR. igate .EQ. 2 .OR. igate .EQ. 3 ) THEN
        CALL OUTPUT(efile,'ENSIGHT. Conditioning undeveloped.')
        CALL DNS_STOP(DNS_ERROR_UNDEVELOP)

     ELSE IF ( igate .EQ. 4 ) THEN
        WRITE(*,'(A)') 'Gate levels to be shown ?'
        READ(*,'(A64)') sRes
        igate = 16
        CALL LIST_INTEGER(sRes, igate, gate_levels)

     ENDIF

     iheader = 1
     CALL SCANINICHAR(bakfile, inifile, 'PostProcessing', 'Format', '-1', sRes)
     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) 'File Format ?'
        WRITE(*,*) ' 0. General restart format'
        WRITE(*,*) ' 1. Ensight'
        WRITE(*,*) ' 2. Raw, single precision, no header'
        READ(*,'(A64)') sRes
     ENDIF
     READ(sRes,*) iheader
     
     IF    ( imixture .EQ. MIXT_TYPE_NONE ) THEN; iscal_offset = 9    ! define iscal_offset
     ELSE;                                        iscal_offset = 9+NSP; ENDIF



     CALL SCANINICHAR(bakfile, inifile, 'PostProcessing', 'ParamEnsight', '-1', sRes)
     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,'(A)') ' '
        WRITE(*,'(A)') 'Tool Ensight v1.0'
        WRITE(*,'(A)') '----------------'
        WRITE(*,'(A)') ' '
        WRITE(*,'(A)') ' 0. Grid'
        WRITE(*,'(A)') ' 1. VelocityX'
        WRITE(*,'(A)') ' 2. VelocityY'
        WRITE(*,'(A)') ' 3. VelocityZ'
        WRITE(*,'(A)') ' 4. VelocityVector'
        WRITE(*,'(A)') ' 5. Velocity V_iV_i'
        WRITE(*,'(A)') ' 6. Density'
        WRITE(*,'(A)') ' 7. Temperature'
        WRITE(*,'(A)') ' 8. Pressure'
        WRITE(*,'(A)') ' 9. Scalars'
        IF ( imixture .NE. MIXT_TYPE_NONE ) THEN
           DO is = 1,NSP
              WRITE(*,'(I2,A)') 9+is,'. '//THERMO_SPNAME(is)
           ENDDO
        ENDIF
       
        WRITE(*,'(I2,A)') iscal_offset+ 1,'. ScalarGradientVector'
        WRITE(*,'(I2,A)') iscal_offset+ 2,'. ScalarGradient G_iG_i (Ln)'
        WRITE(*,'(I2,A)') iscal_offset+ 3,'. ScalarGradientEquation'
        WRITE(*,'(I2,A)') iscal_offset+ 4,'. VorticityVector'
        WRITE(*,'(I2,A)') iscal_offset+ 5,'. Enstrophy W_iW_i (Ln)'
        WRITE(*,'(I2,A)') iscal_offset+ 6,'. EnstrophyEquation'
        WRITE(*,'(I2,A)') iscal_offset+ 7,'. StrainTensor (not yet)'
        WRITE(*,'(I2,A)') iscal_offset+ 8,'. Strain 2S_ijS_ij (Ln)'
        WRITE(*,'(I2,A)') iscal_offset+ 9,'. StrainEquation'
        WRITE(*,'(I2,A)') iscal_offset+10,'. VelocityGradientInvariants'
        WRITE(*,'(I2,A)') iscal_offset+11,'. Space partition'
        WRITE(*,'(I2,A)') iscal_offset+12,'. Buoyancy'
        WRITE(*,'(I2,A)') iscal_offset+13,'. Envelopes'
        WRITE(*,'(I2,A)') iscal_offset+14,'. NaturalLog BuoyancySource'
        WRITE(*,'(I2,A)') iscal_offset+15,'. HorizontalDivergence'
        WRITE(*,'(I2,A)') iscal_offset+16,'. Turbulent quantities'
         IF (  iradiation .EQ. EQNS_RAD_BULK1D ) THEN
               WRITE(*,'(I2,A)') iscal_offset+17,'. Radiative forcing'
         ENDIF
        READ(*,'(A512)') sRes        

     ENDIF
     
     ivar_size = ivar_size_max
     CALL LIST_INTEGER(sRes, ivar_size, ivar_vec)

#ifdef PARALLEL
  ENDIF

  CALL MPI_BCAST(iscal_offset, 1,      MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  CALL MPI_BCAST(ivar_size,    1,      MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  icount = ivar_size
  CALL MPI_BCAST(ivar_vec,     icount, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  icount = 6
  CALL MPI_BCAST(subdomain,    icount, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  CALL MPI_BCAST(igate,        1,      MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  icount = 16
  CALL MPI_BCAST(gate_levels,  icount, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  CALL MPI_BCAST(iheader,      1,      MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

! -------------------------------------------------------------------
! Further allocation of memory space
! -------------------------------------------------------------------
  inb_txc         = 0
  inb_poisson_txc = 0
  DO iv = 1,ivar_size
     IF ( ivar_vec(iv) .GE. 1 .AND. ivar_vec(iv) .LE. 8 ) inb_txc=MAX(inb_txc,2)
     IF ( ivar_vec(iv) .EQ. 4 .OR.  ivar_vec(iv) .EQ. 5 ) inb_txc=MAX(inb_txc,4)
     IF ( ivar_vec(iv) .EQ. 7 ) inb_txc=MAX(inb_txc,3)
     IF ( ivar_vec(iv) .EQ. 8 ) THEN
        IF ( imode_eqns .EQ. DNS_EQNS_INCOMPRESSIBLE .OR. &
             imode_eqns .EQ. DNS_EQNS_ANELASTIC           ) THEN
           inb_txc        =MAX(inb_txc,5)
           inb_poisson_txc=4
        ELSE
           inb_txc        =MAX(inb_txc,3)
        ENDIF
     ENDIF
     IF ( ivar_vec(iv) .EQ. 11 .AND. imixture.EQ.MIXT_TYPE_AIRWATER ) inb_txc=MAX(inb_txc,3)

     IF ( ivar_vec(iv) .EQ. iscal_offset+1 ) inb_txc=MAX(inb_txc,3)
     IF ( ivar_vec(iv) .EQ. iscal_offset+2 ) inb_txc=MAX(inb_txc,3)
     IF ( ivar_vec(iv) .EQ. iscal_offset+3 ) inb_txc=MAX(inb_txc,9)
     IF ( ivar_vec(iv) .EQ. iscal_offset+4 ) inb_txc=MAX(inb_txc,7)
     IF ( ivar_vec(iv) .EQ. iscal_offset+5 ) inb_txc=MAX(inb_txc,6)
     IF ( ivar_vec(iv) .EQ. iscal_offset+6 ) inb_txc=MAX(inb_txc,9)

     IF ( ivar_vec(iv) .EQ. iscal_offset+8 ) inb_txc=MAX(inb_txc,6)
     IF ( ivar_vec(iv) .EQ. iscal_offset+9 ) inb_txc=MAX(inb_txc,9)
     IF ( ivar_vec(iv) .EQ. iscal_offset+10) inb_txc=MAX(inb_txc,9)
     IF ( ivar_vec(iv) .EQ. iscal_offset+12) inb_txc=MAX(inb_txc,2)
     IF ( ivar_vec(iv) .EQ. iscal_offset+14) inb_txc=MAX(inb_txc,3)
     IF ( ivar_vec(iv) .EQ. iscal_offset+15) inb_txc=MAX(inb_txc,3)
     IF ( ivar_vec(iv) .EQ. iscal_offset+16) inb_txc=MAX(inb_txc,8)
  ENDDO
  IF ( inb_txc .GT. 0 ) THEN
     WRITE(str,*) inb_txc;        line = 'Allocating array txc. Size '//TRIM(ADJUSTL(str))//'x'
     WRITE(str,*) isize_field; line = TRIM(ADJUSTL(line))//TRIM(ADJUSTL(str))
     CALL OUTPUT(lfile,line)
     ALLOCATE(txc(isize_field,inb_txc),stat=ierr)
     IF ( ierr .NE. 0 ) THEN
        CALL OUTPUT(efile,'DNS. Not enough memory for txc.')
        CALL DNS_STOP(DNS_ERROR_ALLOC)
     ENDIF
  ENDIF
  IF ( inb_poisson_txc .GT. 0 ) THEN
     WRITE(str,*) inb_poisson_txc; line = 'Allocating array poisson. Size '//TRIM(ADJUSTL(str))//'x'
     WRITE(str,*) isize_txc_field; line = TRIM(ADJUSTL(line))//TRIM(ADJUSTL(str))
     CALL OUTPUT(lfile,line)
     ALLOCATE(txc_poisson(isize_txc_field,inb_poisson_txc),stat=ierr)
     IF ( ierr .NE. 0 ) THEN
        CALL OUTPUT(efile,'DNS. Not enough memory for poisson.')
        CALL DNS_STOP(DNS_ERROR_ALLOC)
     ENDIF
  ENDIF

  iread_scal = 0
  DO iv = 1,ivar_size
     IF ( ivar_vec(iv) .EQ. 7 ) iread_scal = 1
     IF ( ivar_vec(iv) .EQ. 8 ) iread_scal = 1
     IF ( ivar_vec(iv) .GE. 9 .AND. ivar_vec(iv) .LE. iscal_offset ) iread_scal = 1
     IF ( ivar_vec(iv) .EQ. iscal_offset+1 ) iread_scal = 1
     IF ( ivar_vec(iv) .EQ. iscal_offset+2 ) iread_scal = 1
     IF ( ivar_vec(iv) .EQ. iscal_offset+3 ) iread_scal = 1
     IF ( ivar_vec(iv) .EQ. iscal_offset+14) iread_scal = 1
     IF ( ivar_vec(iv) .EQ. iscal_offset+17) iread_scal = 1
  ENDDO
  IF ( iread_scal .EQ. 1 ) THEN
     WRITE(str,*) inb_scal_array; line = 'Allocating array scal. Size '//TRIM(ADJUSTL(str))//'x'
     WRITE(str,*) isize_field; line = TRIM(ADJUSTL(line))//TRIM(ADJUSTL(str))
     CALL OUTPUT(lfile,line)
     ALLOCATE(s(isize_field,inb_scal_array),stat=ierr)
     IF ( ierr .NE. 0 ) THEN
        CALL OUTPUT(efile,'DNS. Not enough memory for scal.')
        CALL DNS_STOP(DNS_ERROR_ALLOC)
     ENDIF
  ENDIF

  iread_gate = 0
  DO iv = 1,ivar_size
     IF ( ivar_vec(iv) .EQ. iscal_offset+11 ) iread_gate = 1
  ENDDO
  IF ( igate .GT. 0 ) iread_gate = 1
  IF ( iread_gate .EQ. 1 ) THEN
     ALLOCATE(gate(isize_field))
  ENDIF
  IF ( igate .GT. 0 ) ALLOCATE(MaskFile(isize_field))

  DO iv = 1,ivar_size
     IF ( ivar_vec(iv) .EQ. iscal_offset+13 ) idummy = 1 ! envelopes/superlayers
  ENDDO
  IF ( idummy .EQ. 1 ) ALLOCATE(surface(imax*kmax,inb_scal))

  inb_wrk2d = 1
  IF ( ifourier .EQ. 1 ) inb_wrk2d = MAX(inb_wrk2d,2)

  ALLOCATE(wrk2d(isize_wrk2d*inb_wrk2d))

! Array wrk3d; I need it in MPI_IO...
#ifndef PARALLEL
  IF ( inb_txc .GT. 0 .OR. iread_scal .GT. 0 .OR. iread_gate .GT. 0 ) THEN
#endif
  WRITE(str,*) isize_wrk3d; line = 'Allocating array wrk3d. Size '//TRIM(ADJUSTL(str))
  CALL OUTPUT(lfile,line)
  ALLOCATE(wrk3d(isize_wrk3d),stat=ierr)
  IF ( ierr .NE. 0 ) THEN
     CALL OUTPUT(efile,'DNS. Not enough memory for wrk3d.')
     CALL DNS_STOP(DNS_ERROR_ALLOC)
  ENDIF
#ifndef PARALLEL
  ENDIF
#endif

! auxiliar array for in parallel mode
#ifdef PARALLEL
  itmp_mpi=isize_field*2
  WRITE(str,*) itmp_mpi; line = 'Allocating array tmp_mpi. Size '//TRIM(ADJUSTL(str))
  CALL OUTPUT(lfile,line)
  ALLOCATE(tmp_mpi(itmp_mpi),stat=ierr)
  IF ( ierr .NE. 0 ) THEN
     CALL OUTPUT(efile,'DNS. Not enough memory for tmp_mpi.')
     CALL DNS_STOP(DNS_ERROR_ALLOC)
  ENDIF
#endif

! -------------------------------------------------------------------
! Read the grid 
! -------------------------------------------------------------------
#include "dns_read_grid.h"

! -------------------------------------------------------------------
! Initialize Poisson solver
! -------------------------------------------------------------------
  IF ( inb_poisson_txc .GT. 0 .AND. ifourier .EQ. 1 ) THEN
     CALL OPR_FOURIER_INITIALIZE(txc_poisson, wrk1d,wrk2d,wrk3d)
  ENDIF
 
! ###################################################################
! Grid
! ###################################################################
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     DO iv = 1,ivar_size
        IF ( ivar_vec(iv) .EQ. 0 ) THEN
           CALL ENSIGHT_GRID('grid.ensight', imax,jmax,kmax_total, subdomain, x,y,z)
        ENDIF
     ENDDO
#ifdef PARALLEL
  ENDIF
#endif

! ###################################################################
! Postprocess given list of files
! ###################################################################
  DO it=1, itime_size

     itime = itime_vec(it)

     WRITE(sRes,*) itime; sRes = 'Processing iteration It'//TRIM(ADJUSTL(sRes))
     CALL OUTPUT(lfile,sRes)

! main variables
     WRITE(flow_file,*) itime; flow_file=TRIM(ADJUSTL(tag_flow))//TRIM(ADJUSTL(flow_file))

! scalar variables
     WRITE(scal_file,*) itime; scal_file=TRIM(ADJUSTL(tag_scal))//TRIM(ADJUSTL(scal_file))

! gate file for partition and create mask if needed
     IF ( iread_gate .EQ. 1 ) THEN
        WRITE(aux_file,*) itime; aux_file='par'//TRIM(ADJUSTL(aux_file))
        CALL DNS_READ_GATE(aux_file, itime, rtime, imax, jmax, kmax, kmax_total, &
             gate_nmax, isize_wrk3d, gate, wrk3d)

        IF ( igate .GT. 0 ) THEN
           DO ij = 1,isize_field
              MaskFile(ij) = 0
              DO is=1,igate
                 IF ( gate(ij) .EQ. gate_levels(is) ) MaskFile(ij) = 1
              ENDDO
           ENDDO
        ENDIF
        
     ENDIF

! define time string
     DO ij=MaskSize,1,-1
        time_str(ij:ij)='0'
     ENDDO
     WRITE(plot_file,'(I10)') itime
     time_str(MaskSize-LEN_TRIM(ADJUSTL(plot_file))+1:Masksize)=TRIM(ADJUSTL(plot_file))

     DO iv = 1,ivar_size

! ###################################################################
! Velocities
! ###################################################################
        IF ( ivar_vec(iv) .EQ. 1 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i1, isize_wrk3d, txc(1,1),wrk3d)

           plot_file = 'VelocityX'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

        ELSE IF ( ivar_vec(iv) .EQ. 2 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i2, isize_wrk3d, txc(1,1),wrk3d)

           plot_file = 'VelocityY'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

        ELSE IF ( ivar_vec(iv) .EQ. 3 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i3, isize_wrk3d, txc(1,1),wrk3d)

           plot_file = 'VelocityZ'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

        ELSE IF ( ivar_vec(iv) .EQ. 4 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i0, isize_wrk3d, txc(1,1),wrk3d)

           plot_file = 'VelocityVector'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i1, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

        ELSE IF ( ivar_vec(iv) .EQ. 5 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i0, isize_wrk3d, txc(1,1),wrk3d)

           DO ij = 1,isize_field
              txc(ij,1) = txc(ij,1)*txc(ij,1) + txc(ij,2)*txc(ij,2) + txc(ij,3)*txc(ij,3)
           ENDDO

           plot_file = 'VelocityMagnitude'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

! ###################################################################
! Thermodynamic state
! ###################################################################
        ELSE IF ( ivar_vec(iv) .EQ. 6 .OR. ivar_vec(iv) .EQ. 7 .OR. ivar_vec(iv) .EQ. 8 ) THEN
! -------------------------------------------------------------------
! Incompressible
! -------------------------------------------------------------------
           IF ( imode_eqns .EQ. DNS_EQNS_INCOMPRESSIBLE .OR. &
                imode_eqns .EQ. DNS_EQNS_ANELASTIC           ) THEN
! density 
              IF ( ivar_vec(iv) .EQ. 6 ) THEN
                 CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, i1,i0, isize_wrk3d, txc(1,1),wrk3d)
                 wrk1d(1:jmax) = C_0_R
                 CALL FLOW_BUOYANCY(ibodyforce, imax,jmax,kmax, body_param, txc(1,1), txc(1,2), wrk1d)
                 dummy = C_1_R/froude
                 txc(1:isize_field,2) = txc(1:isize_field,2)*dummy + C_1_R

                 plot_file = 'Density'//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,2), tmp_mpi)

!Temperature (only MixAirwater, ALberto)
              ELSE IF ( ivar_vec(iv) .EQ. 7 ) THEN

                  IF ( imixture .EQ. MIXT_TYPE_AIRWATER)  THEN
           	     !Read the scalar fields
                     CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, inb_scal,i0, isize_wrk3d, s,wrk3d)
 
                     !ADD CALL TO CALCLUALTE LIQUID
                     CALL THERMO_THERMAL_DENSITY_HP_ALWATER(i1,i1,i1, mean_i(2),mean_i(1),p_init,mean_rho)
                     CALL THERMO_AIRWATER_PHAL(imax,jmax,kmax, s(:,2), p_init, s(:,1))    
 		     !Temperature in txc(1,1) (energy in txc(1,2), not needed)
		     CALL THERMO_CALORIC_TEMPERATURE(imax,jmax,kmax, s, txc(1,2), txc(1,3), txc(1,1), wrk3d)

                      plot_file = 'Temperature'//time_str(1:MaskSize)
                      CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

                  ELSEIF ( imixture .EQ. MIXT_TYPE_SUPSAT) THEN  !In this case the liquid is already in the third scalar
                     !Read the scalar fields
                     CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, inb_scal,i0, isize_wrk3d, s,wrk3d)
                     !Temperature in txc(1,1) (energy in txc(1,2), not needed)
		     CALL THERMO_CALORIC_TEMPERATURE(imax,jmax,kmax, s, txc(1,2), txc(1,3), txc(1,1), wrk3d)
                      txc(:,1) = (txc(:,1) - txc(1,1))/(txc(isize_field,1) - txc(1,1)) 
                      !Plot the file
                      plot_file = 'Temperature'//time_str(1:MaskSize)
                      CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)
                      
!Calculate Buoyancy
                       CALL THERMO_THERMAL_DENSITY_HP_ALWATER(imax,jmax,kmax, s(1,2),s(1,1),p_init,txc(1,1))
                      ! txc(:,1) = (txc(:,1) - txc(1,1))/(txc(isize_field,1) - txc(1,1)) 
                      txc(:,1) = body_vector(2)*(txc(:,1) - mean_rho)/mean_rho
 

                      !Plot the file
                      plot_file = 'Buoyancy'//time_str(1:MaskSize)
                      CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)
!Calculate supersaturated liquid
                       txc(:,1:2) = s(:,1:2)
                       CALL THERMO_AIRWATER_PHAL(imax,jmax,kmax, txc(:,2), p_init, txc(:,1))
                       txc(:,3) = (s(:,3)-txc(:,3))/s(1,3)
                       !Plot the file
                      plot_file = 'Supsat'//time_str(1:MaskSize)
                      CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,3), tmp_mpi)

                  ENDIF


! pressure
              ELSE IF ( ivar_vec(iv) .EQ. 8 ) THEN
                 CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i0, isize_wrk3d, txc(1,1),wrk3d)
                 IF ( icalc_scal .EQ. 1 ) THEN
                 CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, i1,i0, isize_wrk3d, txc(1,4),wrk3d)
                 ENDIF

                 CALL OUTPUT(lfile,'Computing pressure...')
                 CALL FI_PRESSURE_BOUSSINESQ(y,dx,dy,dz, txc(1,1),txc(1,2),txc(1,3),txc(1,4), &
                      txc_poisson(1,1),txc_poisson(1,2),txc_poisson(1,3),txc_poisson(1,4), wrk1d,wrk2d,wrk3d)

                 plot_file = 'Pressure'//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc_poisson(1,1), tmp_mpi)
              ENDIF

! -------------------------------------------------------------------
! compressible
! -------------------------------------------------------------------
           ELSE
              CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i5,i5, isize_wrk3d, txc(1,1),wrk3d)
! density in array txc1
              IF ( ivar_vec(iv) .EQ. 6 ) THEN
                 plot_file = 'Density'//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)
                 
! temperature in array txc3
              ELSE IF ( ivar_vec(iv) .EQ. 7 ) THEN
                 CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i4,i4, isize_wrk3d, txc(1,2),wrk3d)
                 IF ( imixture .NE. MIXT_TYPE_NONE ) THEN
                    CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, inb_scal,i0, isize_wrk3d, s,wrk3d)
                 ENDIF
                 CALL THERMO_CALORIC_TEMPERATURE(imax,jmax,kmax, s, txc(1,2), txc(1,1), txc(1,3), wrk3d)

                 plot_file = 'Temperature'//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,3), tmp_mpi)
                 
! pressure in array txc2 (energy overwritten)
              ELSE IF ( ivar_vec(iv) .EQ. 8 ) THEN
                 CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i4,i4, isize_wrk3d, txc(1,2),wrk3d)
                 IF ( imixture .NE. MIXT_TYPE_NONE ) THEN
                    CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, inb_scal,i0, isize_wrk3d, s,wrk3d)
                 ENDIF
                 CALL THERMO_CALORIC_TEMPERATURE&
                      (imax,jmax,kmax, s, txc(1,2), txc(1,1), txc(1,3), wrk3d)
                 CALL THERMO_THERMAL_PRESSURE&
                      (imax,jmax,kmax, s, txc(1,1), txc(1,3), txc(1,2))

                 plot_file = 'Pressure'//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,2), tmp_mpi)
              ENDIF
              
           ENDIF
        ENDIF

! ###################################################################
! N Scalars
! ###################################################################
    
        IF ( ivar_vec(iv) .GE. 9 .AND. ivar_vec(iv) .LE. iscal_offset) THEN
           CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, inb_scal,i0, isize_wrk3d, s,wrk3d)

! Transported scalar fields
           IF      ( ivar_vec(iv) .EQ. 9 ) THEN
              DO is = 1,inb_scal
                 WRITE(str,*) is; str = 'Scalar'//TRIM(ADJUSTL(str))

                 plot_file = TRIM(ADJUSTL(str))//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, s(1,is), tmp_mpi)
              ENDDO

           ELSE

! -------------------------------------------------------------------
! mixture MIXT_TYPE_AIRWATER. s(1,inb_scal+1) contains liquid mass fraction
! -------------------------------------------------------------------
              IF ( imixture .EQ. MIXT_TYPE_AIRWATER ) THEN

              IF ( imode_eqns .EQ. DNS_EQNS_INCOMPRESSIBLE .OR. &
                   imode_eqns .EQ. DNS_EQNS_ANELASTIC      ) THEN
                 CALL THERMO_AIRWATER_PHAL(i1, i1,i1, mean_i(2), p_init, mean_i(1))
                 CALL THERMO_THERMAL_DENSITY_HP_ALWATER(i1,i1,i1, mean_i(2),mean_i(1),p_init,mean_rho)
                 CALL THERMO_AIRWATER_PHAL(imax,jmax,kmax, s(:,2), p_init, s(:,1))    

              ELSE ! Compressible
                 CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i5,i5, isize_wrk3d, txc(1,2),wrk3d)
                 CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i4,i4, isize_wrk3d, txc(1,1),wrk3d)

                 CALL THERMO_CALORIC_TEMPERATURE(imax,jmax,kmax, s, txc(1,1), txc(1,2), txc(1,3), wrk3d)

              ENDIF

              IF      ( ivar_vec(iv) .EQ. 10 ) THEN ! vapor water mass fraction
                 s(:,1) = s(:,inb_scal) - s(:,inb_scal+1)

                 plot_file = TRIM(ADJUSTL(THERMO_SPNAME(1)))//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, s, tmp_mpi)

              ELSE IF ( ivar_vec(iv) .EQ. 11 ) THEN ! air mass fraction
                 s(:,1) = C_1_R - s(:,inb_scal) 

                 plot_file = TRIM(ADJUSTL(THERMO_SPNAME(2)))//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, s, tmp_mpi)

              ELSE IF ( ivar_vec(iv) .EQ. 12 ) THEN ! liquid mass fraction

                 plot_file = TRIM(ADJUSTL(THERMO_SPNAME(3)))//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, s(1,inb_scal+1), tmp_mpi)

              ENDIF

! -------------------------------------------------------------------
! General mixture of NSP species. Plot the chosen species
! -------------------------------------------------------------------
              ELSE
                 is = ivar_vec(iv) - 9
                 plot_file = TRIM(ADJUSTL(THERMO_SPNAME(is)))//time_str(1:MaskSize)
                 CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, s(1,is), tmp_mpi)

              ENDIF

           ENDIF

        ENDIF


! ###################################################################
! Scalar Derivatives
! ###################################################################
        IF ( ivar_vec(iv) .GE. iscal_offset+1 .AND. ivar_vec(iv) .LE. iscal_offset+3 ) THEN
           CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, inb_scal,i0, isize_wrk3d, s,wrk3d) 

           DO is = 1,inb_scal
           WRITE(str,*) is; str = 'Scalar'//TRIM(ADJUSTL(str))

! Scalar gradient vector
           IF ( ivar_vec(iv) .EQ. iscal_offset+1 ) THEN
              CALL OUTPUT(lfile,'Computing scalar gradient vector...')
              CALL PARTIAL_X(imode_fdm, imax,jmax,kmax, i1bc, dx, s(1,is), txc(1,1), i0,i0, wrk1d,wrk2d,wrk3d)
              CALL PARTIAL_Y(imode_fdm, imax,jmax,kmax, j1bc, dy, s(1,is), txc(1,2), i0,i0, wrk1d,wrk2d,wrk3d)
              CALL PARTIAL_Z(imode_fdm, imax,jmax,kmax, k1bc, dz, s(1,is), txc(1,3), i0,i0, wrk1d,wrk2d,wrk3d)
              
              plot_file = TRIM(ADJUSTL(str))//'GradientVector'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i1, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)
           ENDIF

! Scalar gradient
           IF ( ivar_vec(iv) .EQ. iscal_offset+2 .OR. ivar_vec(iv) .EQ. iscal_offset+3 ) THEN
              CALL OUTPUT(lfile,'Computing scalar gradient...')
              CALL FI_GRADIENT(imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, dx,dy,dz, s(1,is), &
                   txc(1,1),txc(1,2),txc(1,3), wrk1d,wrk2d,wrk3d)
              
              plot_file = TRIM(ADJUSTL(str))//'Gradient'//time_str(1:MaskSize)

              IF ( ivar_vec(iv) .EQ. iscal_offset+2 ) THEN 
                 IF ( igate .GT. 0 ) THEN                              ! Conditioning
                    CALL MINMAX(imax,jmax,kmax, txc(1,1), umin,umax)
                    DO ij = 1,isize_field
                       IF ( MaskFile(ij) .EQ. 1 ) THEN; txc(ij,1)=log(txc(ij,1))
                       ELSE;                            txc(ij,1)=log(umin); ENDIF
                    ENDDO
                 ELSE                                                  ! Natural log
                    txc(1:isize_field,1) = log(txc(1:isize_field,1))
                    plot_file = 'Ln'//TRIM(ADJUSTL(plot_file))
                ENDIF
              ENDIF

              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)
           ENDIF

! Scalar gradient equation
           IF ( ivar_vec(iv) .EQ. iscal_offset+3 ) THEN
              CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i0, isize_wrk3d, txc(1,1), wrk3d)

              CALL OUTPUT(lfile,'Computing scalar gradient production...')
              CALL FI_GRADIENT_PRODUCTION(imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, dx,dy,dz, s(1,is), &
                   txc(1,1),txc(1,2),txc(1,3), txc(1,4),txc(1,5),txc(1,6),txc(1,7),txc(1,8),txc(1,9), wrk1d,wrk2d,wrk3d)
              plot_file = 'ScalarGradientProduction'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

              CALL OUTPUT(lfile,'Computing scalar gradient diffusion...')
              CALL FI_GRADIENT_DIFFUSION&
                   (iunifx,iunify,iunifz, imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, dx,dy,dz, s(1,is), &
                   txc(1,1),txc(1,2),txc(1,3),txc(1,4),txc(1,5),txc(1,6), wrk1d,wrk2d,wrk3d)
              txc(1:isize_field,1)=diff*txc(1:isize_field,1)
              plot_file = TRIM(ADJUSTL(str))//'GradientDiffusion'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

           ENDIF

           ENDDO

        ENDIF
        
! ###################################################################
! Velocity Derivatives
! ###################################################################
        IF ( ivar_vec(iv) .GE. iscal_offset+4 .AND. ivar_vec(iv) .LE. iscal_offset+10 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i0, isize_wrk3d, txc(1,1), wrk3d)

! -------------------------------------------------------------------
! Vorticity 
! -------------------------------------------------------------------
! VorticityVector
           IF ( ivar_vec(iv) .EQ. iscal_offset+4 ) THEN
              CALL OUTPUT(lfile,'Computing vorticity vector...')
              CALL FI_CURL(imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, dx,dy,dz, &
                   txc(1,1),txc(1,2),txc(1,3), txc(1,4),txc(1,5),txc(1,6),txc(1,7), wrk1d,wrk2d,wrk3d)
              
              plot_file = 'VorticityVector'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i1, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)
           ENDIF
           
! Enstrophy
           IF ( ivar_vec(iv) .EQ. iscal_offset+5 .OR. ivar_vec(iv) .EQ. iscal_offset+6 ) THEN
              CALL OUTPUT(lfile,'Computing enstrophy...')
              CALL FI_VORTICITY(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), txc(1,4), txc(1,5), txc(1,6), &
                   wrk1d, wrk2d, wrk3d)
              
              IF ( ivar_vec(iv) .EQ. iscal_offset+5 ) THEN ! Natural log
                 DO ij = 1,isize_field
                    txc(ij,4)=log(txc(ij,4))
                 ENDDO
                 plot_file = 'LnEnstrophy'//time_str(1:MaskSize)
              ELSE
                 plot_file = 'Enstrophy'//time_str(1:MaskSize)
              ENDIF

              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)
           ENDIF
           
! EnstrophyEquation
           IF ( ivar_vec(iv) .EQ. iscal_offset+6 ) THEN
              CALL OUTPUT(lfile,'Computing enstrophy production...')
              CALL FI_VORTICITY_PRODUCTION(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), &
                   txc(1,4), txc(1,5), txc(1,6), txc(1,7), txc(1,8), txc(1,9), wrk1d,wrk2d,wrk3d)
              plot_file = 'EnstrophyProduction'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

              CALL OUTPUT(lfile,'Computing enstrophy diffusion...')
              CALL FI_VORTICITY_DIFFUSION&
                   (iunifx,iunify,iunifz, imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), &
                   txc(1,4), txc(1,5), txc(1,6), txc(1,7), txc(1,8), txc(1,9), wrk1d,wrk2d,wrk3d)
              DO ij = 1,isize_field
                 txc(ij,4)=visc*txc(ij,4)
              ENDDO
              plot_file = 'EnstrophyDiffusion'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

           ENDIF
           
! -------------------------------------------------------------------
! Strain
! -------------------------------------------------------------------
! StrainEquation (I need the pressure)
           IF ( ivar_vec(iv) .EQ. iscal_offset+9 ) THEN
              IF ( imode_eqns .EQ. DNS_EQNS_INCOMPRESSIBLE .OR. &
                   imode_eqns .EQ. DNS_EQNS_ANELASTIC     )THEN
                 CALL OUTPUT(efile,'ENSIGHT. Strain eqn for incompressible undeveloped.')
                 CALL DNS_STOP(DNS_ERROR_UNDEVELOP)
              ELSE
                 CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, inb_flow,i0, isize_wrk3d, txc(1,1), wrk3d)
              ENDIF
              CALL OUTPUT(lfile,'Computing strain pressure...')
              CALL THERMO_CALORIC_TEMPERATURE&
                   (imax,jmax,kmax, s, txc(1,4), txc(1,5), txc(1,6), wrk3d)
              CALL THERMO_THERMAL_PRESSURE&
                   (imax,jmax,kmax, s, txc(1,5), txc(1,6), txc(1,4)) ! pressure in txc4
              CALL FI_STRAIN_PRESSURE&
                   (iunifx,iunify,iunifz, imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), txc(1,4), &
                   txc(1,5), txc(1,6), txc(1,7), txc(1,8), txc(1,9), wrk1d,wrk2d,wrk3d)
              DO ij = 1,isize_field
                 txc(ij,5)=C_2_R*txc(ij,5)
              ENDDO
              plot_file = 'StrainPressure'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,5), tmp_mpi)

              CALL OUTPUT(lfile,'Computing strain production...')
              CALL FI_STRAIN_PRODUCTION(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), &
                   txc(1,4), txc(1,5), txc(1,6), txc(1,7), txc(1,8), txc(1,9), wrk1d,wrk2d,wrk3d)
              DO ij = 1,isize_field
                 txc(ij,4)=C_2_R*txc(ij,4)
              ENDDO
              plot_file = 'StrainProduction'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

              CALL OUTPUT(lfile,'Computing strain diffusion...')
              CALL FI_STRAIN_DIFFUSION&
                   (iunifx,iunify,iunifz, imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), &
                   txc(1,4), txc(1,5), txc(1,6), txc(1,7), txc(1,8), txc(1,9), wrk1d,wrk2d,wrk3d)
              DO ij = 1,isize_field
                 txc(ij,4)=C_2_R*visc*txc(ij,4)
              ENDDO
              plot_file = 'StrainDiffusion'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)
           ENDIF

! Strain Tensor
           IF ( ivar_vec(iv) .EQ. iscal_offset+7 ) THEN
              CALL OUTPUT(efile,'ENSIGHT. Strain tensor undevelop.')
              CALL DNS_STOP(DNS_ERROR_UNDEVELOP)
           ENDIF

! Strain
           IF ( ivar_vec(iv) .EQ. iscal_offset+8 .OR. ivar_vec(iv) .EQ. iscal_offset+9 ) THEN
              CALL OUTPUT(lfile,'Computing strain...')
              CALL FI_STRAIN(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), txc(1,4), txc(1,5), txc(1,6), &
                   wrk1d, wrk2d, wrk3d)
              DO ij = 1,isize_field
                 txc(ij,4)=C_2_R*txc(ij,4)
              ENDDO
              
              IF ( ivar_vec(iv) .EQ. iscal_offset+8 ) THEN ! Natural log
                 DO ij = 1,isize_field
                    txc(ij,4)=log(txc(ij,4))
                 ENDDO
                 plot_file = 'LnStrain'//time_str(1:MaskSize)
              ELSE
                 plot_file = 'Strain'//time_str(1:MaskSize)
              ENDIF

              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)
           ENDIF
           
! -------------------------------------------------------------------
! Invariants
! -------------------------------------------------------------------
           IF ( ivar_vec(iv) .EQ. iscal_offset+10 ) THEN
              CALL OUTPUT(lfile,'Computing first invariant P...')
              CALL FI_INVARIANT_P(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), txc(1,4), &
                   txc(1,5), wrk1d, wrk2d, wrk3d)

              plot_file = 'InvariantP'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

              CALL OUTPUT(lfile,'Computing second invariant Q...')
              CALL FI_INVARIANT_Q(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), txc(1,4), &
                   txc(1,5), txc(1,6), txc(1,7), wrk1d, wrk2d, wrk3d)

              plot_file = 'InvariantQ'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

              CALL OUTPUT(lfile,'Computing third invariant R...')
              CALL FI_INVARIANT_R(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, txc(1,1), txc(1,2),txc(1,3), txc(1,4), &
                   txc(1,5), txc(1,6), txc(1,7), txc(1,8), txc(1,9), wrk1d, wrk2d, wrk3d)

              plot_file = 'InvariantR'//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)


           ENDIF
        ENDIF

! ###################################################################
! Other quantities
! ###################################################################
! -------------------------------------------------------------------
! Partition
! -------------------------------------------------------------------
        IF ( ivar_vec(iv) .EQ. iscal_offset+11 ) THEN
           
!           DO ij = 1,isize_field
!              wrk3d(ij)=real(gate(ij))
!           ENDDO
!
           plot_file = 'Partition'//time_str(1:MaskSize)
!           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, wrk3d, tmp_mpi)
#ifdef USE_ACCESS_STREAM
           OPEN(i55,file=plot_file,status='unknown',form='unformatted',access='stream')
#else
#ifdef USE_BUFLESS
           OPEN(i55,file=plot_file,form='unformatted',status='unknown',type='bufless')
#else
#ifdef USE_BINARY
           OPEN(i55,file=plot_file,form='binary',status='unknown')
#endif
#endif
#endif
           WRITE(i55) gate
           CLOSE(i55)
           
! -------------------------------------------------------------------
! buoyancy
! -------------------------------------------------------------------
        ELSE IF ( ivar_vec(iv) .EQ. iscal_offset+12 ) THEN
              CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, i1,i0, isize_wrk3d, txc(1,1), wrk3d)
                wrk1d(1:jmax) = C_0_R
               CALL FLOW_BUOYANCY(ibodyforce, imax,jmax,kmax, body_param, txc(1,1), txc(1,2), wrk1d)
                dummy =  C_1_R/froude
                txc(1:isize_field,2) = txc(1:isize_field,2)*dummy + body_param(5)
           
                plot_file = 'Buoyancy'//time_str(1:MaskSize)
                CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,2), tmp_mpi)               
 
! -------------------------------------------------------------------
! envelopes/superlayers
! -------------------------------------------------------------------
        ELSE IF ( ivar_vec(iv) .EQ. iscal_offset+13 ) THEN
! we use the # scalars as a surrogate for the # envelopes to have control through dns.ini
           WRITE(aux_file,*) itime; aux_file='lower.'//TRIM(ADJUSTL(aux_file))
           CALL DNS_READ_FIELDS(aux_file, i1, imax,i1,kmax, inb_scal,i0, isize_wrk2d, surface, wrk2d)

           DO is = 1,inb_scal 
              subdomain(3) = 1; subdomain(4) = 1
              WRITE(str,*) is; plot_file = 'LowerEnvelope'//TRIM(ADJUSTL(str))//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,i1,kmax, subdomain, surface(1,is), tmp_mpi)
           ENDDO

           WRITE(aux_file,*) itime; aux_file='upper.'//TRIM(ADJUSTL(aux_file))
           CALL DNS_READ_FIELDS(aux_file, i1, imax,i1,kmax, inb_scal,i0, isize_wrk2d, surface, wrk2d)

           DO is = 1,inb_scal 
              subdomain(3) = 1; subdomain(4) = 1
              WRITE(str,*) is; plot_file = 'UpperEnvelope'//TRIM(ADJUSTL(str))//time_str(1:MaskSize)
              CALL PLOT_WRITE(plot_file, i0, iheader, imax,i1,kmax, subdomain, surface(1,is), tmp_mpi)
           ENDDO

! -------------------------------------------------------------------
! Buoyancy Source
! -------------------------------------------------------------------
        ELSE IF ( ivar_vec(iv) .EQ. iscal_offset+14 .AND. ibodyforce .EQ. EQNS_BOD_REVERSAL ) THEN
           CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, i1,i0, isize_wrk3d, s, wrk3d)
           
           CALL OUTPUT(lfile,'Computing scalar gradient...')
           CALL FI_GRADIENT(imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
                dx,dy,dz, s, txc(1,1),txc(1,2),txc(1,3), wrk1d,wrk2d,wrk3d)
! prefactor
           dummy =-(body_param(2)-body_param(1))/(C_1_R-body_param(3)) - body_param(2)/body_param(3)
           dummy = dummy/(C_4_R*body_param(4)) * visc/schmidt(1) / froude
           DO ij = 1,isize_field
              txc(ij,1)=-txc(ij,1)*dummy / (cosh(C_05_R*(s(ij,1)-body_param(3))/body_param(4)))**2
              txc(ij,1)=log(abs(txc(ij,1)))
           ENDDO

           plot_file = 'LnBuoyancySource'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

! -------------------------------------------------------------------
! Horizontal divergence
! -------------------------------------------------------------------
        ELSE IF ( ivar_vec(iv) .EQ. iscal_offset+15 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, i3,i0, isize_wrk3d, txc(1,1), wrk3d)

           CALL OUTPUT(lfile,'Computing horizontal divergence...')
           CALL PARTIAL_X(imode_fdm, imax,jmax,kmax, i1bc,&
                dx, txc(1,1), txc(1,2), i0,i0, wrk1d,wrk2d,wrk3d)
           CALL PARTIAL_Z(imode_fdm, imax,jmax,kmax, k1bc,&
                dz, txc(1,3), txc(1,1), i0,i0, wrk1d,wrk2d,wrk3d)
           DO ij = 1,isize_field
              txc(ij,1) = txc(ij,1) + txc(ij,2)
           ENDDO

           plot_file = 'HorizontalDivergence'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)

! -------------------------------------------------------------------
! Turbulent quantities k and \epsilon
! -------------------------------------------------------------------
        ELSE IF ( ivar_vec(iv) .EQ. iscal_offset+16 ) THEN
           CALL DNS_READ_FIELDS(flow_file, i2, imax,jmax,kmax, inb_flow,i0, isize_wrk3d, txc(1,1), wrk3d)

! Initialize the density field in arrat tmp5 if needed
           IF      ( imode_eqns .EQ. DNS_EQNS_INCOMPRESSIBLE ) THEN
              txc(:,5) = C_1_R
           ELSE IF ( imode_eqns .EQ. DNS_EQNS_ANELASTIC      ) THEN ! not yet implemented
           ENDIF
                 
! turbulent dissipation rate into txc4 because I do not need the energy
           CALL OUTPUT(lfile,'Computing dissipation rate...')
           CALL FI_DISSIPATION(i1, imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
                area, visc, dx,dy,dz, txc(1,5),txc(1,1),txc(1,2),txc(1,3), txc(1,4), &
                txc(1,6),txc(1,7),txc(1,8), mean2d, wrk1d,wrk2d,wrk3d)
           DO ij = 1,isize_field
              txc(ij,4)=log(txc(ij,4))
           ENDDO

           plot_file = 'LnDissipation'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

! Buoyancy flux                                                                                     
           CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, i1,i0, isize_wrk3d, txc(1,4), wrk3d)

           CALL OUTPUT(lfile,'Computing buoyancy flux...')
           DO ij = 1,isize_field
              txc(ij,4) = txc(ij,4)*txc(ij,2)
           ENDDO

           plot_file = 'Cvb'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,4), tmp_mpi)

! turbulent kinetic energy
           CALL REYFLUCT2D(imax,jmax,kmax, dx,dz, area, txc(1,1))
           CALL REYFLUCT2D(imax,jmax,kmax, dx,dz, area, txc(1,2))
           CALL REYFLUCT2D(imax,jmax,kmax, dx,dz, area, txc(1,3))

           DO ij = 1,isize_field
              txc(ij,1) = txc(ij,1)*txc(ij,1) + txc(ij,2)*txc(ij,2) + txc(ij,3)*txc(ij,3)
           ENDDO

           plot_file = 'Tke'//time_str(1:MaskSize)
           CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, txc(1,1), tmp_mpi)


! Radiation
         ELSE IF (  ivar_vec(iv) .EQ. iscal_offset+17) THEN
               CALL DNS_READ_FIELDS(scal_file, i1, imax,jmax,kmax, inb_scal,i0, isize_wrk3d, s,wrk3d) !Read the scalars in s
               CALL FLOW_BULK1D_RADIATION(imax,jmax,kmax, dy, s(1,2),rad_param,wrk1d(1:jmax),wrk1d(jmax+1:2*jmax),wrk1d(2*jmax+1:3*jmax),wrk3d) !Calculate radiation function in wrk3d

!Write the file 

               plot_file = 'Radiation'//time_str(1:MaskSize)
               CALL PLOT_WRITE(plot_file, i0, iheader, imax,jmax,kmax, subdomain, wrk3d, tmp_mpi)
        


        ENDIF

     ENDDO

! -------------------------------------------------------------------
! print out the time sequence for animation
! -------------------------------------------------------------------
     WRITE(sRes,1000) rtime
     CALL OUTPUT('times.log','Time '//TRIM(ADJUSTL(sRes)))

  ENDDO
  
  CALL DNS_END(0)

  STOP

1000 FORMAT(G_FORMAT_R)

END PROGRAM PLOT_MAIN
