      SUBROUTINE DEC_FLUX_SG(n0, x, y, z, dx, dy, dz, txc1, txc2, txc3,&
           txc4, z1_a, z1_b, u_a, u_b, wrk1d, wrk2d, wrk3d, mean, &
           cf0x, cf0y, cf0z)  

      IMPLICIT NONE

#include "types.h"
#include "common.h"
#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

      TINTEGER n0

      TREAL x(imax)
      TREAL y(jmax)
      TREAL z(kmax_total)
      TREAL dx(imax)
      TREAL dy(jmax)
      TREAL dz(kmax_total)

      TREAL txc1(imax,jmax,kmax)
      TREAL txc2(imax,jmax,kmax)
      TREAL txc3(imax,jmax,kmax)
      TREAL txc4(imax,jmax,kmax)
      TREAL z1_a(imax,jmax,kmax)
      TREAL z1_b(imax,jmax,kmax)
      TREAL u_a(imax,jmax,kmax)
      TREAL u_b(imax,jmax,kmax)

      TREAL wrk1d(isize_wrk1d,*)
      TREAL wrk2d(isize_wrk2d,*)
      TREAL wrk3d(imax,jmax,kmax)
      TREAL cf0x(*), cf0y(*), cf0z(*)
      TREAL mean(jmax,*)

      TINTEGER ij, k, j, i, nmax, itxc, mmax
      TREAL AVG_IK
      CHARACTER*32 sname, pname, str

      itxc = imax*jmax*kmax

#ifdef PARALLEL
      IF ( ims_pro .EQ. 0 ) THEN
#endif
         WRITE(*,*) 'Number of terms in the deconvolution series ?'
         READ(*,*) nmax
         WRITE(*,*) 'Number of terms in the filter recurrence ?'
         READ(*,*) mmax
#ifdef PARALLEL
      ENDIF
      CALL MPI_BCAST(nmax, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, &
           ims_err)
      CALL MPI_BCAST(mmax, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, &
           ims_err)
#endif

#define ZC0(j)            mean(j,1)
#define Z_AVG(j)          mean(j,2)
#define Z_RMS(j)          mean(j,3)
#define Z_ARM_L1(j)       mean(j,4)
#define Z_ADM_L1(j)       mean(j,5)
#define Z_ARM_AVG(j)      mean(j,6)
#define Z_ADM_AVG(j)      mean(j,7)
#define Z_ARM_RMS(j)      mean(j,8)
#define Z_ADM_RMS(j)      mean(j,9)
#define Z2SG_AVG(j)       mean(j,10) 
#define Z2SG_RMS(j)       mean(j,11)
#define Z2SG_ARM_L1(j)    mean(j,12)
#define Z2SG_ADM_L1(j)    mean(j,13)
#define Z2SG_ARM_AVG(j)   mean(j,14)
#define Z2SG_ADM_AVG(j)   mean(j,15)
#define Z2SG_ARM_RMS(j)   mean(j,16)
#define Z2SG_ADM_RMS(j)   mean(j,17)

#define UC0(j)            mean(j,18)
#define U_AVG(j)          mean(j,19)
#define U_RMS(j)          mean(j,20)
#define U_ARM_L1(j)       mean(j,21)
#define U_ADM_L1(j)       mean(j,22)
#define U_ARM_AVG(j)      mean(j,23)
#define U_ADM_AVG(j)      mean(j,24)
#define U_ARM_RMS(j)      mean(j,25)
#define U_ADM_RMS(j)      mean(j,26)
#define U2SG_AVG(j)       mean(j,27)
#define U2SG_RMS(j)       mean(j,28)
#define U2SG_ARM_L1(j)    mean(j,29)
#define U2SG_ADM_L1(j)    mean(j,30)
#define U2SG_ARM_AVG(j)   mean(j,31)
#define U2SG_ADM_AVG(j)   mean(j,32)
#define U2SG_ARM_RMS(j)   mean(j,33)
#define U2SG_ADM_RMS(j)   mean(j,34)

#define UZSG_AVG(j)       mean(j,35)
#define UZSG_RMS(j)       mean(j,36)
#define UZSG_ARM_L1(j)    mean(j,37)
#define UZSG_ADM_L1(j)    mean(j,38)
#define UZSG_ARM_AVG(j)   mean(j,39)
#define UZSG_ADM_AVG(j)   mean(j,40)
#define UZSG_ARM_RMS(j)   mean(j,41)
#define UZSG_ADM_RMS(j)   mean(j,42)

#define L_AVGMAX 42

! ##################################################################
! # Scalar                                                         #
! ##################################################################
#ifdef PARALLEL
      IF ( ims_pro .EQ. 0 ) THEN
#endif
         WRITE(*,*) 'Computing scalar...'
#ifdef PARALLEL
      ENDIF
#endif
      WRITE(sname,*) itime; sname = 'sc'//TRIM(ADJUSTL(sname))
      CALL DNS_READ_FIELDS(sname, imax, jmax, kmax, kmax_total, &
           itxc, i1, i1, i1, z1_a, wrk3d)
! -------------------------------------------------------------------
!                           Compute profile of average (Z^2)_{sg}
! -------------------------------------------------------------------
! input ...: original field z1_a
! output ..: txc1, filtered field z1_a
!            txc2, variance field
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, z1_a, txc1, txc2, txc4, wrk3d)
      DO j = 1,jmax
         Z2SG_AVG(j) =&
              AVG_IK(imax, jmax, kmax, j, txc2, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc2(i,j,k) - Z2SG_AVG(j))**C_2_R
            ENDDO
         ENDDO
         Z2SG_RMS(j) = &
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
      ENDDO
! -------------------------------------------------------------------
!                                                Computing series
! -------------------------------------------------------------------
! input ...: bar(Z) in txc1
! output ..: Zm ARM in z1_a
!            Zm ADM in z1_b
      CALL DEC_SERIES(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0, nmax, &
           area, dx, dy, dz, cf0x, cf0y, cf0z, ZC0(1), Z2SG_AVG(1),&
           txc1, txc2, z1_a, z1_b, txc4, wrk1d, wrk3d)
! -------------------------------------------------------------------
!                                          Calculating error in Z
! -------------------------------------------------------------------
! original field
      CALL DNS_READ_FIELDS(sname, imax, jmax, kmax, kmax_total, &
           itxc, i1, i1, i1, txc3, wrk3d)
! statistics
      DO j = 1,jmax
         Z_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, txc3, dx, dz, area)
         Z_ARM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, z1_a, dx, dz, area)
         Z_ADM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, z1_b, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc3(i,j,k) - Z_AVG(j))**C_2_R
               wrk3d(i,2,k) = (z1_a(i,j,k) - Z_ARM_AVG(j))**C_2_R
               wrk3d(i,3,k) = (z1_b(i,j,k) - Z_ADM_AVG(j))**C_2_R
            ENDDO
         ENDDO
         Z_ARM_AVG(j) = Z_AVG(j) - Z_ARM_AVG(j)
         Z_ADM_AVG(j) = Z_AVG(j) - Z_ADM_AVG(j)
         Z_RMS(j) = &
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
         Z_ARM_RMS(j) = Z_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 2, wrk3d, dx, dz, area) )
         Z_ADM_RMS(j) = Z_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 3, wrk3d, dx, dz, area) )
      ENDDO
! L1 error
      DO k = 1,kmax
         DO ij = 1,imax*jmax
            txc1(ij,1,k) = ABS( txc3(ij,1,k) - z1_a(ij,1,k) )
            txc2(ij,1,k) = ABS( txc3(ij,1,k) - z1_b(ij,1,k) )
         ENDDO
      ENDDO
      DO j = 1,jmax
         Z_ARM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         Z_ADM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc2, dx, dz, area)
      ENDDO
! -------------------------------------------------------------------
!                                   Calculating error in (Z^2)_sg
! -------------------------------------------------------------------
! actual field
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, txc3, txc1, txc2, txc4, wrk3d)
! ARM 
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, z1_a, txc1, txc3, txc4, wrk3d)
      DO k = 1,kmax
         DO ij = 1,imax*jmax
            txc1(ij,1,k) = ABS( txc2(ij,1,k) - txc3(ij,1,k) )
         ENDDO
      ENDDO
      DO j = 1,jmax
         Z2SG_ARM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         Z2SG_ARM_AVG(j) =&
              AVG_IK(imax, jmax, kmax, j, txc3, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc3(i,j,k) - Z2SG_ARM_AVG(j))**C_2_R
            ENDDO
         ENDDO
         Z2SG_ARM_AVG(j) = Z2SG_AVG(j) - Z2SG_ARM_AVG(j)
         Z2SG_ARM_RMS(j) = Z2SG_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
      ENDDO
! ADM 
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, z1_b, txc1, txc3, txc4, wrk3d)
      DO k = 1,kmax
         DO ij = 1,imax*jmax
            txc1(ij,1,k) = ABS( txc2(ij,1,k) - txc3(ij,1,k) )
         ENDDO
      ENDDO
      DO j = 1,jmax
         Z2SG_ADM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         Z2SG_ADM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, txc3, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc3(i,j,k) - Z2SG_ADM_AVG(j))**C_2_R
            ENDDO
         ENDDO
         Z2SG_ADM_AVG(j) = Z2SG_AVG(j) - Z2SG_ADM_AVG(j)
         Z2SG_ADM_RMS(j) = Z2SG_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
      ENDDO

! ##################################################################
! # Velocity                                                       #
! ##################################################################
#ifdef PARALLEL
      IF ( ims_pro .EQ. 0 ) THEN
#endif
         WRITE(*,*) 'Computing velocity...'
#ifdef PARALLEL
      ENDIF
#endif
      WRITE(pname,*) itime; pname = 'pt'//TRIM(ADJUSTL(pname))
      CALL DNS_READ_FLOW(pname, imax, jmax, kmax, kmax_total, itxc, &
           u_a, txc1, txc1, txc1, txc1, wrk3d)
! -------------------------------------------------------------------
!                           Compute profile of average (U^2)_{sg}
! -------------------------------------------------------------------
! input ...: original field u_a
! output ..: txc1, filtered field u_a
!            txc2, variance field
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, u_a, txc1, txc2, txc4, wrk3d)
      DO j = 1,jmax
         U2SG_AVG(j) =&
              AVG_IK(imax, jmax, kmax, j, txc2, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc2(i,j,k) - U2SG_AVG(j))**C_2_R
            ENDDO
         ENDDO
         U2SG_RMS(j) = &
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
      ENDDO
! -------------------------------------------------------------------
!                                                Computing series
! -------------------------------------------------------------------
! input ...: bar(U) in txc1
! output ..: Um ARM in u_a
!            Um ADM in u_b
      CALL DEC_SERIES(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0, nmax, &
           area, dx, dy, dz, cf0x, cf0y, cf0z, UC0(1), U2SG_AVG(1),&
           txc1, txc2, u_a, u_b, txc4, wrk1d, wrk3d)
! -------------------------------------------------------------------
!                                          Calculating error in U
! -------------------------------------------------------------------
! original field
      CALL DNS_READ_FLOW(pname, imax, jmax, kmax, kmax_total, itxc, &
           txc3, txc2, txc2, txc2, txc2, wrk3d)
! statistics
      DO j = 1,jmax
         U_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, txc3, dx, dz, area)
         U_ARM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, u_a, dx, dz, area)
         U_ADM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, u_b, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc3(i,j,k) - U_AVG(j))**C_2_R
               wrk3d(i,2,k) = (z1_a(i,j,k) - U_ARM_AVG(j))**C_2_R
               wrk3d(i,3,k) = (z1_b(i,j,k) - U_ADM_AVG(j))**C_2_R
            ENDDO
         ENDDO
         U_ARM_AVG(j) = U_AVG(j) - U_ARM_AVG(j)
         U_ADM_AVG(j) = U_AVG(j) - U_ADM_AVG(j)
         U_RMS(j) = &
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
         U_ARM_RMS(j) = U_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 2, wrk3d, dx, dz, area) )
         U_ADM_RMS(j) = U_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 3, wrk3d, dx, dz, area) )
      ENDDO
! L1 error
      DO k = 1,kmax
         DO ij = 1,imax*jmax
            txc1(ij,1,k) = ABS( txc3(ij,1,k) - u_a(ij,1,k) )
            txc2(ij,1,k) = ABS( txc3(ij,1,k) - u_b(ij,1,k) )
         ENDDO
      ENDDO
      DO j = 1,jmax
         U_ARM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         U_ADM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc2, dx, dz, area)
      ENDDO
! -------------------------------------------------------------------
!                                   Calculating error in (U^2)_sg
! -------------------------------------------------------------------
! actual field
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, txc3, txc1, txc2, txc4, wrk3d)
! ARM 
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, u_a, txc1, txc3, txc4, wrk3d)
      DO k = 1,kmax
         DO ij = 1,imax*jmax
            txc1(ij,1,k) = ABS( txc2(ij,1,k) - txc3(ij,1,k) )
         ENDDO
      ENDDO
      DO j = 1,jmax
         U2SG_ARM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         U2SG_ARM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, txc3, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc3(i,j,k) - U2SG_ARM_AVG(j))**C_2_R
            ENDDO
         ENDDO
         U2SG_ARM_AVG(j) = U2SG_AVG(j) - U2SG_ARM_AVG(j)
         U2SG_ARM_RMS(j) = U2SG_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
      ENDDO
! ADM 
      CALL DEC_VARIANCE(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, u_b, txc1, txc3, txc4, wrk3d)
      DO k = 1,kmax
         DO ij = 1,imax*jmax
            txc1(ij,1,k) = ABS( txc2(ij,1,k) - txc3(ij,1,k) )
         ENDDO
      ENDDO
      DO j = 1,jmax
         U2SG_ADM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         U2SG_ADM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, txc3, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc3(i,j,k) - U2SG_ADM_AVG(j))**C_2_R
            ENDDO
         ENDDO
         U2SG_ADM_AVG(j) = U2SG_AVG(j) - U2SG_ADM_AVG(j)
         U2SG_ADM_RMS(j) = U2SG_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
      ENDDO

! ##################################################################
! # Cross-error                                                    #
! ##################################################################
#ifdef PARALLEL
      IF ( ims_pro .EQ. 0 ) THEN
#endif
         WRITE(*,*) 'Computing cross error...'
#ifdef PARALLEL
      ENDIF
#endif
! ARM --> txc1 (z1_a and u_a can be now eliminated)
      CALL DEC_CROSS(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, u_a, z1_a, txc2, txc3, txc1, txc4, wrk3d)
! ADM --> txc2
      CALL DEC_CROSS(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, u_b, z1_b, u_a, z1_a, txc2, txc4, wrk3d)
! original field --> txc3
      CALL DNS_READ_FIELDS(sname, imax, jmax, kmax, kmax_total, &
           itxc, i1, i1, i1, z1_a, wrk3d)
      CALL DNS_READ_FLOW(pname, imax, jmax, kmax, kmax_total, itxc, &
           u_a, txc3, txc3, txc3, txc3, wrk3d)
      CALL DEC_CROSS(mmax, imax, jmax, kmax, kmax_total, &
           iunifx, iunify, iunifz, i1bc, j1bc, k1bc, n0,&
           cf0x, cf0y, cf0z, u_a, z1_a, u_b, z1_b, txc3, txc4, wrk3d)

! statistics
      DO j = 1,jmax
         UZSG_AVG(j) =&
              AVG_IK(imax, jmax, kmax, j, txc3, dx, dz, area)
         UZSG_ARM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         UZSG_ADM_AVG(j) = &
              AVG_IK(imax, jmax, kmax, j, txc2, dx, dz, area)
         DO k = 1,kmax
            DO i = 1,imax
               wrk3d(i,1,k) = (txc3(i,j,k) - UZSG_AVG(j))**C_2_R
               wrk3d(i,2,k) = (txc1(i,j,k) - UZSG_ARM_AVG(j))**C_2_R
               wrk3d(i,3,k) = (txc2(i,j,k) - UZSG_ADM_AVG(j))**C_2_R
            ENDDO
         ENDDO
         UZSG_ARM_AVG(j) = UZSG_AVG(j) - UZSG_ARM_AVG(j)
         UZSG_ADM_AVG(j) = UZSG_AVG(j) - UZSG_ADM_AVG(j)
         UZSG_RMS(j) =&
              SQRT( AVG_IK(imax, jmax, kmax, 1, wrk3d, dx, dz, area) )
         UZSG_ARM_RMS(j) = UZSG_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 2, wrk3d, dx, dz, area) )
         UZSG_ADM_RMS(j) = UZSG_RMS(j) -&
              SQRT( AVG_IK(imax, jmax, kmax, 3, wrk3d, dx, dz, area) )
      ENDDO
! L1 error
      DO k = 1,kmax
         DO ij = 1,imax*jmax
            txc1(ij,1,k) = ABS( txc3(ij,1,k) - txc1(ij,1,k) )
            txc2(ij,1,k) = ABS( txc3(ij,1,k) - txc2(ij,1,k) )
         ENDDO
      ENDDO
      DO j = 1,jmax
         UZSG_ARM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc1, dx, dz, area)
         UZSG_ADM_L1(j) = &
              AVG_IK(imax, jmax, kmax, j, txc2, dx, dz, area)
      ENDDO

! ##################################################################
! ##################################################################

! Writting the data
#ifdef PARALLEL
      IF ( ims_pro .EQ. 0 ) THEN
#endif

      WRITE(str,*) nmax; str = 'prdec'//TRIM(ADJUSTL(str))
      WRITE(sname,*) itime; sname = TRIM(ADJUSTL(str))//TRIM(ADJUSTL(sname))

      OPEN(unit=i23,file=sname,status='unknown')
      WRITE(i23,'(A8,E14.7E3)') 'RTIME = ', rtime
      WRITE(i23,*) 'I J Y '&
           //'Zc0 '&
           //'Z_avg Z_rms Z_arm_L1 Z_adm_L1 '&
           //'Z_arm_avg Z_adm_avg Z_arm_rms Z_adm_rms '&
           //'Z2sg_avg Z2sg_rms Z2sg_arm_L1 Z2sg_adm_L1 '&
           //'Z2sg_arm_avg Z2sg_adm_avg Z2sg_arm_rms Z2sg_adm_rms '&
           //'Uc0 '&
           //'U_avg U_rms U_arm_L1 U_adm_L1 '&
           //'U_arm_avg U_adm_avg U_arm_rms U_adm_rms '&
           //'U2sg_avg U2sg_rms U2sg_arm_L1 U2sg_adm_L1 '&
           //'U2sg_arm_avg U2sg_adm_avg U2sg_arm_rms U2sg_adm_rms '&
           //'UZsg_avg UZsg_rms UZsg_arm_L1 UZsg_adm_L1 '&
           //'UZsg_arm_avg UZsg_adm_avg UZsg_arm_rms UZsg_adm_rms'
      DO j = 1,jmax
         WRITE(i23,1300) i1, j, y(j), &
              (mean(j,k),k=1,L_AVGMAX)
         
      ENDDO
      CLOSE(i23)

#ifdef PARALLEL
      ENDIF
#endif

 1300 FORMAT(I3,1X,I3,1(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))

      RETURN
      END

