#include "types.h"
#include "dns_const.h"
#include "avgij_map.h"

! ###########################
! # Spatially evolving case #
! ###########################

PROGRAM COHETT
  
  USE DNS_GLOBAL

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL wrk(:)
  ALLOCATABLE wrk
  TREAL t_dat(:)
  ALLOCATABLE t_dat
  TREAL l_dat(:)
  ALLOCATABLE l_dat
  TREAL wrk1d(:)
  ALLOCATABLE wrk1d
  TREAL txc1(:)
  ALLOCATABLE txc1
  TREAL txc2(:)
  ALLOCATABLE txc2
  TREAL spct(:)
  ALLOCATABLE spct
  TINTEGER connect(:)
  ALLOCATABLE connect
  TREAL mean(:)
  ALLOCATABLE mean

  TINTEGER MX_ITIME_SET
  PARAMETER (MX_ITIME_SET=1000)

  TINTEGER ntimeset, maxnrest      
  TINTEGER timeset(MX_ITIME_SET)

  TINTEGER i2
  TINTEGER i, n, l, nmax, nmax_org, flgstat
  TINTEGER samp_step
  CHARACTER*32 sfile
  CHARACTER*128 line
#ifdef PARALLEL
  INTEGER counts
#endif
  LOGICAL lfilexist

  i2 = 2

  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL('dns.ini')

#ifdef PARALLEL
  CALL DNS_MPI_INITIALIZE
#endif

! #########################################
! # Get information from stdin or dns.ini #
! #########################################

  ntimeset = MX_ITIME_SET
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) 'Series of integral Iteration ?'
     READ(*,'(A128)') line
     CALL LIST_INTEGER( line, ntimeset, timeset )
#ifdef PARALLEL
  ENDIF
  counts = ntimeset
  CALL MPI_BCAST( timeset, counts, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

  CALL SORT_INTEGER(ntimeset,timeset)

! maximum file size in case of variable nspa_rest allowed
  maxnrest = nspa_rest
! the first ln**** file has to be of nspa_rest
  nmax = nspa_rest
  DO n = 2,ntimeset
     IF ( timeset(n)-timeset(n-1) .NE. nspa_rest ) THEN
        CALL IO_WRITE_ASCII(wfile,'Warning: Possible nonsequential data')
     ENDIF
! this part to allow different sizes of ln**** was commmented and has to be rechecked.
!     maxnrest = MAX(maxnrest,timeset(n)-timeset(n-1))
!     nmax = nmax + timeset(n)-timeset(n-1)
     nmax = nmax + maxnrest
  ENDDO

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) 'Integral sampling rate:'
     READ(*,*) samp_step
#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST( samp_step, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, &
       ims_err)
#endif

  nmax_org = nmax
  nmax = nmax/samp_step

! Flag for use of st**** file

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) 'Use global statistics (0-no/1-yes)?'
     READ(*,*) flgstat
     IF ( flgstat .NE. 1 ) THEN
        flgstat = 0
     ENDIF

     WRITE(sfile,*) timeset(ntimeset); sfile='st'//TRIM(ADJUSTL(sfile))
     
     INQUIRE(file=sfile,EXIST=lfilexist)
     IF ( .NOT. lfilexist ) THEN
        line = TRIM(ADJUSTL(sfile))//' does not exit. Going to local.'
        CALL IO_WRITE_ASCII(wfile,line)
        flgstat = 0
     ENDIF

     DO n = 1, nstatlin
        i = statlin_i(n)
        DO l = 1,nstatavg
           IF ( statavg(l) .EQ. i ) EXIT
        ENDDO
        IF ( l .EQ. nstatavg+1 ) THEN
           WRITE(line,*) i; line='Avg-plane'//TRIM(ADJUSTL(line))//' not present. Going to local.'
           CALL IO_WRITE_ASCII(wfile,line)
           flgstat = 0
        ENDIF
     ENDDO

#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST( flgstat, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, &
       ims_err)
#endif

  ALLOCATE(wrk(2*nmax+15))
  ALLOCATE(t_dat(2*nmax_org))
  ALLOCATE(txc1(kmax*inb_vars*nstatlin*maxnrest))
  ALLOCATE(txc2(maxnrest*inb_vars*nstatlin*kmax))
  ALLOCATE(l_dat(nmax*inb_vars*nstatlin*kmax))
  ALLOCATE(spct(nmax*inb_vars*nstatlin))         
  ALLOCATE(wrk1d(nmax*inb_vars))
  ALLOCATE(connect(nstatlin*2))
  IF ( flgstat .EQ. 1 ) THEN
#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        ALLOCATE(mean(nstatavg*jmax*inb_mean_spatial))
#ifdef PARALLEL
     ENDIF
#endif
  ENDIF

! ###########################
  CALL DNS_READ_LINE_SERIES( samp_step, maxnrest, nmax, nmax_org, &
       t_dat, l_dat, ntimeset, timeset, txc1, txc2 ) 

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) '# Points        : ',nmax_org
     WRITE(*,*) '# Sampling      : ',samp_step
#ifdef PARALLEL
  ENDIF
#endif
  IF ( flgstat .EQ. 1 ) THEN
     sfile='st                             '
     CALL CONCATI(sfile, timeset(ntimeset))
#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        WRITE(*,*) '# Stat. file    : ', sfile
#ifdef PARALLEL
     ENDIF
#endif
     itime = timeset(ntimeset)
     CALL DNS_READ_AVGIJ ( sfile, mean )
  ENDIF
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) '# Points        : ',nmax
     WRITE(*,*) '# Sampling      : ',samp_step
     sfile='tch                             '
     CALL CONCATI(sfile, timeset(ntimeset))
     WRITE(*,*) '# T Coherency   : ', sfile
     WRITE(*,*) '#############################'
#ifdef PARALLEL
  ENDIF
#endif

  CALL FAVFLUCT_LINE( imax, jmax, kmax, kmax_total, flgstat, &
       nstatavg, nstatavg_points, nstatlin, nmax, inb_vars, &
       inb_scal,&
       statavg, statlin_i, statlin_j, mean, &
       mean((MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*(inb_scal-1))*&
       nstatavg*jmax),&
       l_dat, l_dat )

  CALL COHET_SPA( inb_vars, nmax, jmax, kmax, kmax_total, &
       nstatlin, statlin_i, statlin_j,&
       t_dat, l_dat, spct, wrk, wrk1d, connect )

  CALL DNS_END(0)

  STOP
END PROGRAM

