#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"

! ###########################
! # Spatially evolving case #
! ###########################
PROGRAM CORRTT

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
#ifdef LES
  USE LES_GLOBAL
#endif

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL t_dat(:)
  ALLOCATABLE t_dat
  TREAL l_dat(:)
  ALLOCATABLE l_dat
  TREAL wrk1d(:)
  ALLOCATABLE wrk1d
  TREAL txc1(:)
  ALLOCATABLE txc1
  TREAL txc2(:)
  ALLOCATABLE txc2
  TREAL autocrl(:)
  ALLOCATABLE autocrl
  TREAL crosscrl(:)
  ALLOCATABLE crosscrl
  TREAL mean(:)
  ALLOCATABLE mean

  TINTEGER MX_ITIME_SET
  PARAMETER (MX_ITIME_SET=256)
  TINTEGER CROSS
  PARAMETER (CROSS=3)

  TINTEGER ntimeset, maxnrest      
  TINTEGER timeset(MX_ITIME_SET)

  TINTEGER i, n, l, nmax, nmax_org, flgstat
  TINTEGER samp_step, l1, l2
  CHARACTER*32 sfile
  CHARACTER*128 line
  CHARACTER*512 sRes
#ifdef PARALLEL
  INTEGER icount
#endif
  LOGICAL lfilexist

  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL('dns.ini')

#ifdef PARALLEL
  CALL DNS_MPI_INITIALIZE
#endif

  inb_mean_spatial = MA_MOMENTUM_SIZE + MS_SCALAR_SIZE*inb_scal
#ifdef LES
  IF ( iles .EQ. 1 ) THEN
     inb_mean_spatial = inb_mean_spatial &
                      + LA_MOMENTUM_SIZE + (LS_SCALAR_SIZE+LC_CHEM_SIZE)*inb_scal
  ENDIF
#endif

! ##############################
! # Get information from stdin #
! ##############################
  CALL SCANINICHAR(lfile,'dns.ini','PostProcessing','Files','-1',sRes)
  CALL SCANINIINT(lfile,'dns.ini','PostProcessing','Sampling','1',samp_step)
  CALL SCANINIINT(lfile,'dns.ini','PostProcessing','UseStatFile','0',flgstat)

    IF ( sRes .EQ. '-1' ) THEN
#ifdef PARALLEL
     CALL IO_WRITE_ASCII(efile, 'CORRTT. No iteration numbers provided')
     CALL DNS_STOP(DNS_ERROR_OPTION)
#else
     WRITE(*,*) 'Integral Iterations ?'
     READ(*,'(A512)') sRes
     WRITE(*,*) 'Integral sampling rate:'
     READ(*,*) samp_step
     WRITE(*,*) 'Use global statistics (0-no/1-yes)?'
     READ(*,*) flgstat
#endif
  ENDIF
  ntimeset = MX_ITIME_SET
  CALL LIST_INTEGER(sRes, ntimeset, timeset )
  CALL SORT_INTEGER(ntimeset, timeset)

! maximum file size in case of variable nspa_rest allowed
  maxnrest = nspa_rest

! the first ln-file has to be of nspa_rest
  nmax = maxnrest
  DO n = 2,ntimeset
     IF ( (timeset(n)-timeset(n-1))/nspa_step .NE. nspa_rest ) THEN
        CALL IO_WRITE_ASCII(wfile,'Warning: Possible nonsequential data')
     ENDIF
! this part to allow different sizes of ln-files has to be rechecked.
!     maxnrest = MAX(maxnrest,timeset(n)-timeset(n-1))
!     nmax = nmax + timeset(n)-timeset(n-1)
     nmax = nmax + maxnrest
  ENDDO

! sampling
  nmax_org = nmax
  nmax = nmax/samp_step

! flag for use of st-file
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     IF ( flgstat .EQ. 1 ) THEN
        WRITE(sfile,*) timeset(ntimeset); sfile='st'//TRIM(ADJUSTL(sfile))

        INQUIRE(file=sfile,EXIST=lfilexist)
        IF ( .NOT. lfilexist ) THEN
           line = TRIM(ADJUSTL(sfile))//' does not exit. Going to local.'
           CALL IO_WRITE_ASCII(wfile,line)
           flgstat = 0
        ENDIF

        DO n = 1, nstatlin
           i = statlin_i(n)
           DO l = 1,nstatavg
              IF ( statavg(l) .EQ. i ) EXIT
           ENDDO
           IF ( l .EQ. nstatavg+1 ) THEN
              WRITE(line,*) i; line='Avg-plane'//TRIM(ADJUSTL(line))//' not present. Going to local.'
              CALL IO_WRITE_ASCII(wfile,line)
              flgstat = 0
           ENDIF
        ENDDO
     ENDIF
#ifdef PARALLEL
  ENDIF
#endif

! Allocating memory
  ALLOCATE(t_dat(2*nmax_org))
  ALLOCATE(txc1(kmax*inb_vars*nstatlin*maxnrest))
  ALLOCATE(txc2(maxnrest*inb_vars*nstatlin*kmax))
  ALLOCATE(l_dat(nmax*inb_vars*nstatlin*kmax))
  ALLOCATE(autocrl(nmax*inb_vars*nstatlin))         
  ALLOCATE(crosscrl(nmax*CROSS*nstatlin))         
  ALLOCATE(wrk1d(nmax*inb_wrk1d))
  IF ( flgstat .EQ. 1 ) THEN
#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        ALLOCATE(mean(nstatavg*jmax*inb_mean_spatial))
#ifdef PARALLEL
     ENDIF
#endif
  ENDIF

! ###########################
  CALL DNS_READ_LINE_SERIES( samp_step, maxnrest, nmax, nmax_org, &
       t_dat, l_dat, ntimeset, timeset, txc1, txc2 ) 

  IF ( flgstat .EQ. 1 ) THEN
     sfile='st                             '
     CALL CONCATI(sfile, timeset(ntimeset))
#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        WRITE(*,*) '# Stat. file    : ', sfile
#ifdef PARALLEL
     ENDIF
#endif
     itime = timeset(ntimeset)
     CALL DNS_READ_AVGIJ ( sfile, mean )
  ENDIF

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) '# Points        : ',nmax
     WRITE(*,*) '# Sampling      : ',samp_step
     sfile='tcr                             '
     CALL CONCATI(sfile, timeset(ntimeset))
     WRITE(*,*) '# T Correlation : ', sfile
     WRITE(*,*) '#############################'
#ifdef PARALLEL
  ENDIF
#endif

  CALL FAVFLUCT_LINE( imax, jmax, kmax, kmax_total, flgstat, &
       nstatavg, nstatavg_points, nstatlin, nmax, inb_vars, &
       inb_scal,&
       statavg, statlin_i, statlin_j, mean, &
       mean((MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*(inb_scal-1))*&
       nstatavg*jmax),&
       l_dat, l_dat )

  CALL CORRT_SPA(inb_vars, CROSS, nmax, kmax, kmax_total, &
       nstatlin, statlin_i, statlin_j, t_dat, l_dat, autocrl, crosscrl, wrk1d)

  CALL DNS_END(0)

  STOP
END PROGRAM CORRTT

