      SUBROUTINE FLAMELET_ASSEMBLE(ijac, iomega, itemp, NZ, &
           YBCOX, YBCFUEL, XPER, XTOL,&
           lewis, abc, z, rho, cp, xi, yomega, x, F, J, FD, tmp)

      IMPLICIT NONE

#include "types.h"
#include "common.h"

      TINTEGER ijac, iomega, itemp, NZ
      TREAL YBCOX(*), YBCFUEL(*), lewis(*)
      TREAL x(*)
      TREAL F(*)
      TREAL z(*)
      TREAL J(NZ*NSP,*)
      TREAL abc(*)
      TREAL xi(*), rho(*), cp(*), tmp(*)
      TREAL yomega(*), FD(*)
      
      TREAL xdelta, XPER, XTOL
      TINTEGER iv, ik, ii, ij
      TREAL sclv(MAX_NSP)
      TINTEGER i0, i1
#ifdef _DEBUG
      TINTEGER il, im
#endif

      i0 = 0
      i1 = 1

      CALL FLAMELET_F(iomega, i1, NZ, YBCOX, YBCFUEL, lewis, abc, &
           sclv, z, rho, cp, xi, yomega, x, F, tmp)

! Percentage change on variable
!  XPER = 1.0e-5
!  XTOL = 1.0e-12

      IF ( ijac .EQ. 1 ) THEN
         DO iv=1, NZ*NSP
!       Increment variable iv 
            xdelta = x(iv)*XPER
         
            IF ( ABS(xdelta) .LT. XTOL ) THEN
               xdelta = XTOL
            ENDIF

#ifdef _DEBUG
            PRINT *, iv, xdelta
#endif
            x(iv) = x(iv)+xdelta
         
!       Compute new F(x+delta)
            CALL FLAMELET_F(iomega, i0, NZ, YBCOX, YBCFUEL, lewis, abc,&
                 sclv, z, rho, cp, xi, yomega, x, FD, tmp)
         
            DO ik=1, NZ*NSP
               J(ik,iv) = (FD(ik)-F(ik))/xdelta
            ENDDO
            
            x(iv) = x(iv)-xdelta
         ENDDO
      ENDIF

      IF ( itemp .EQ. 0 ) THEN
         DO ii=(NSP-1)*NZ+1, NZ*NSP
            DO ij=1, NZ*NSP
               J(ii,ij) = C_0_R
               J(ij,ii) = C_0_R
            ENDDO
         ENDDO
         DO ii=(NSP-1)*NZ+1, NZ*NSP
            J(ii,ii) = C_1_R
            F(ii) = C_0_R
         ENDDO
      ENDIF

#ifdef _DEBUG
      DO iv=1, NSP
         DO ik=1, NZ
            WRITE(*,36) 'J(', iv, ',', ik, ')'
            DO il=1, NZ*NSP,5
               WRITE(*,35) (J(ik+(iv-1)*NZ,MIN(il+im,NZ*NSP)),im=0,4)
            ENDDO
         ENDDO
      ENDDO

      STOP

 34   FORMAT(2(I4,1X),2X,(E12.5E3,1X))
 35   FORMAT(5(E12.5E3,1X))
 36   FORMAT(A2,I3,A1,I3,A1)
#endif

! Change sign of F
      DO iv=1, NZ*NSP
         F(iv) = -F(iv)
      ENDDO

      RETURN
      END
