!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"
#include "dns_const.h"
#include "dns_error.h"

PROGRAM TRC

  USE DNS_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

! -----------------------------------------------------------------------
  TINTEGER kplanes
  ALLOCATABLE kplanes(:)
  TREAL t_dat(:)
  ALLOCATABLE t_dat
  TREAL l_dat(:)
  ALLOCATABLE l_dat
  TREAL wrk1d(:)
  ALLOCATABLE wrk1d
  TREAL txc1(:)
  ALLOCATABLE txc1
  TREAL txc2(:)
  ALLOCATABLE txc2

  TINTEGER nmax, nmax_org, maxnrest, nkplanes, samp_step
  CHARACTER*32 fname

  TINTEGER itime_size_max, itime_size, i
  PARAMETER(itime_size_max=128)
  TINTEGER itime_vec(itime_size_max)
  TINTEGER iopt_size_max, iopt_size
  PARAMETER(iopt_size_max=10)
  TINTEGER opt_vec(iopt_size_max)
  CHARACTER*512 sRes
#ifdef PARALLEL
  INTEGER icount
#endif

! #######################################################################
  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL('dns.ini')

#ifdef PARALLEL
  CALL DNS_MPI_INITIALIZE
#endif

! -------------------------------------------------------------------
! Allocating memory space
! -------------------------------------------------------------------
  ALLOCATE(kplanes(kmax_total))

! -------------------------------------------------------------------
! File names
! -------------------------------------------------------------------
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CALL SCANINICHAR&
          (lfile, 'dns.ini', 'PostProcessing', 'Files', '-1', sRes)
     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) 'Integral Iterations ?'
        READ(*,'(A512)') sRes
     ENDIF
     itime_size = itime_size_max
     CALL LIST_INTEGER(sRes, itime_size, itime_vec)
#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST(itime_size, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  icount = itime_size
  CALL MPI_BCAST(itime_vec, icount, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

! -------------------------------------------------------------------
! Additional options
! -------------------------------------------------------------------
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CALL SCANINICHAR&
          (lfile, 'dns.ini', 'PostProcessing', 'ParamCavg', '-1', sRes)
     iopt_size = iopt_size_max
     CALL LIST_INTEGER(sRes, iopt_size, opt_vec)

     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) 'Integral sampling rate:'
        READ(*,*) samp_step
        WRITE(*,*) 'K Planes to be dumped ?'
        READ(*,'(A512)') sRes
        nkplanes = kmax_total
        CALL LIST_INTEGER(sRes, nkplanes, kplanes)
     ENDIF
#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST(samp_step,    1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  CALL MPI_BCAST(nkplanes,     1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  icount = nkplanes
  CALL MPI_BCAST(kplanes, icount, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

! maximum file size in case of variable nspa_rest allowed
  maxnrest = nspa_rest

! the first ln-file has to be of nspa_rest
  nmax = maxnrest
  DO i = 2,itime_size
     IF ( (itime_vec(i)-itime_vec(i-1))/nspa_step .NE. nspa_rest ) THEN
        CALL IO_WRITE_ASCII(wfile,'Warning: Possible nonsequential data')
     ENDIF
! this part to allow different sizes of ln-files has to be rechecked.
!     maxnrest = MAX(maxnrest,itime_vec(i)-itime_vec(i-1))
!     nmax = nmax + itime_vec(i)-itime_vec(i-1)
     nmax = nmax + maxnrest
  ENDDO

! sampling
  nmax_org = nmax
  nmax = nmax/samp_step

! -------------------------------------------------------------------
! Further allocation of memory space
! -------------------------------------------------------------------
  ALLOCATE(t_dat(2*nmax_org))
  ALLOCATE(l_dat(nmax*inb_vars*nstatlin*kmax))
  ALLOCATE(txc1(kmax*inb_vars*nstatlin*maxnrest))
  ALLOCATE(txc2(maxnrest*inb_vars*nstatlin*kmax))
  ALLOCATE(wrk1d(kmax_total*inb_vars))

! #######################################################################
  IF (imode_sim .EQ. DNS_MODE_SPATIAL ) THEN

     CALL DNS_READ_LINE_SERIES(samp_step, maxnrest, nmax, nmax_org, &
          t_dat, l_dat, itime_size, itime_vec, txc1, txc2)

#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        WRITE(*,*) '# Points        : ',nmax
        WRITE(*,*) '# Sampling      : ',samp_step
        WRITE(fname,*) itime_vec(itime_size); fname = 'trc'//TRIM(ADJUSTL(fname))
        WRITE(*,*) '# T Record      : ', fname
        WRITE(*,*) '#############################'
#ifdef PARALLEL
     ENDIF
#endif

!     CALL FAVFLUCT_LINE( imax, jmax, kmax, kmax_total, flgstat, 
! $        nstatavg, nstatavg_points, nstatlin, nmax, inb_vars, 
! $        inb_scal,
! $        statavg, statlin_i, statlin_j, mean, 
! $        mean((MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*(inb_scal-1))*
! $        nstatavg*jmax),
! $        l_dat, l_dat )

     CALL WRT_TRC(nmax, t_dat, l_dat, itime_size, itime_vec, nkplanes, kplanes, wrk1d)

  ENDIF

  CALL DNS_END(0)

  STOP
END PROGRAM TRC

