!########################################################################
!# Tool/Library SUPERLAYER
!#
!########################################################################
!# HISTORY
!#
!# 2007/11/13 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Prepare inputs for Lipo's dissipation element analysis routines
!# Parallel MPI part is not developed
!# OpenMP part and F90 code taken from Jens Goebbert.
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"
#include "dns_const.h"
#include "dns_error.h"

PROGRAM DELEMENTS

  use mod_Dele
  use mod_Pdf
  use mod_Traj
  use mod_Data

  USE DNS_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

! -------------------------------------------------------------------
! Grid and associated arrays
  TREAL, DIMENSION(:),     ALLOCATABLE, SAVE         :: x, y, z, dx, dy, dz

! Flow variables
  TREAL, DIMENSION(:,:),   ALLOCATABLE, SAVE, TARGET :: q, txc

! envelopes
  TREAL, DIMENSION(:,:,:), ALLOCATABLE, SAVE         :: sl

! working arrays
  TREAL, DIMENSION(:),     ALLOCATABLE, SAVE         :: wrk1d, wrk2d, wrk3d

! Pointers to existing allocated space
  TREAL, DIMENSION(:),   POINTER :: u, v, w

  TINTEGER iopt, ifield, iperiodic, iwrk_size, itxc_size, i, j, k
  TINTEGER ith_max, nbins, nz, iread_flow
  TREAL dummy, threshold
  CHARACTER*32 fname

  TINTEGER itime_size_max, itime_size, it
  PARAMETER(itime_size_max=128)
  TINTEGER itime_vec(itime_size_max)

  TINTEGER iopt_size_max, iopt_size
  PARAMETER(iopt_size_max=10)
  TREAL opt_vec(iopt_size_max)

  TINTEGER ielem_size_max, ielem_size, ie
  PARAMETER(ielem_size_max=2048)
  TINTEGER ielem_vec(ielem_size_max)

  TINTEGER imappedfield_size_max, imappedfield_size, im
  PARAMETER(imappedfield_size_max=32)
  TINTEGER imappedfield_vec(imappedfield_size_max)

  CHARACTER*512 sRes
#ifdef PARALLEL
  INTEGER icount
#endif

! ###################################################################
  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL('dns.ini')
#ifdef PARALLEL
  CALL DNS_MPI_INITIALIZE
#endif

  iwrk_size  = imax*jmax*kmax
  iread_flow = 0

  ALLOCATE(x(imax_total))
  ALLOCATE(y(jmax_total))
  ALLOCATE(z(kmax_total))
  ALLOCATE(dx(imax_total*inb_grid))
  ALLOCATE(dy(jmax_total*inb_grid))
  ALLOCATE(dz(kmax_total*inb_grid))

  ALLOCATE(sl(imax,kmax,2))

  ALLOCATE(wrk1d(isize_wrk1d*5))
  ALLOCATE(wrk2d(isize_wrk2d*2))

! -------------------------------------------------------------------
! passing data to DE routines
! -------------------------------------------------------------------
  ndimx = imax
  ndimy = jmax
  ndimz = kmax
  allocate(psi(ndimx,ndimy,ndimz))
  allocate(jmax_loc(ndimx,ndimz),jmin_loc(ndimx,ndimz))

! -------------------------------------------------------------------
! File names
! -------------------------------------------------------------------
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CALL SCANINICHAR(lfile, 'dns.ini', 'PostProcessing', 'Files', '-1', sRes)
     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) 'Integral Iterations ?'
        READ(*,'(A512)') sRes
     ENDIF
     itime_size = itime_size_max
     CALL LIST_INTEGER(sRes, itime_size, itime_vec)
#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST(itime_size, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  icount = itime_size
  CALL MPI_BCAST(itime_vec, icount, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

! -------------------------------------------------------------------
! Read local options
! -------------------------------------------------------------------
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CALL SCANINICHAR(lfile, 'dns.ini', 'PostProcessing', 'DissElements', '-1', sRes)
     iopt_size = iopt_size_max
     CALL LIST_REAL(sRes, iopt_size, opt_vec)

     IF ( sRes .EQ. '-1' ) THEN
        WRITE(*,*) ' '
        WRITE(*,*) 'Conditioning Field ?'
        WRITE(*,*) '1. scalar.'
        WRITE(*,*) '2. scalar gradient.'
        WRITE(*,*) '3. vorticity magnitude.'
        READ(*,*) ifield
        WRITE(*,*) 'Use local threshold (1) or global max/min (2) ?'
        READ(*,*) ith_max
        
        WRITE(*,*) ' '
        WRITE(*,*) 'Option ?'
        WRITE(*,*) ' 0. Create connectivity information.'
        WRITE(*,*) ' 1. Element PDF of internal elements.'
        WRITE(*,*) ' 2. Element PDF of external elements.'
        WRITE(*,*) ' 3. Extract internal elements from connectivity.'
        WRITE(*,*) ' 4. Extract external elements from connectivity.'
        WRITE(*,*) ' 5. Extract trajectories from element data.'
        WRITE(*,*) ' 6. Extract points from element data.'
        WRITE(*,*) ' 7. Trajectory PDF from element data.'
        WRITE(*,*) ' 8. Field partition.'
        WRITE(*,*) ' 9. Extract gradient trajectory information.'
        READ(*,*) iopt

        IF ( iopt .EQ. 1 .OR. iopt .EQ. 2 .OR. iopt .EQ. 7 ) THEN
           WRITE(*,*) ' '
           WRITE(*,*) 'Number of PDF bins ?'
           READ(*,*) nbins
        ENDIF
        IF ( iopt .EQ. 7 ) THEN
           WRITE(*,*) 'Number of conditioning bins ?'
           READ(*,*) nz
        ENDIF

        IF ( iopt .EQ. 5 .OR. iopt .EQ. 6 .OR. iopt .EQ. 7 ) THEN
           WRITE(*,*) 'Element Ids ?'
           READ(*,'(A512)') sRes
           ielem_size = ielem_size_max
           CALL LIST_INTEGER(sRes, ielem_size, ielem_vec)
        ENDIF

        IF ( iopt .EQ. 5 .OR. iopt .EQ. 7 ) THEN
           WRITE(*,*) ' '
           WRITE(*,*) 'Number of trajectories per element (0-all) ?'
           READ(*,*) dEleSets_fixNoDETraj
           WRITE(*,*) 'Maximum number of points per trajectory ?'
           READ(*,*) dEleSets_noPtsPerTraj 
        ENDIF

        imappedfield_size = 0
        IF ( iopt .EQ. 5 .OR. iopt .EQ. 7 ) THEN
           WRITE(*,*) 'Additional mapped fields ?'
           WRITE(*,*) '0. None (only conditioning field)'
           WRITE(*,*) '1. Scalar gradient G_iG_i'
           WRITE(*,*) '2. Log of scalar gradient G_iG_i'
           WRITE(*,*) '3. Compressive strain rate a'
           WRITE(*,*) '4. Scaled compressive strain rate aG_iG_i'
           WRITE(*,*) '5. Curvature k'
           WRITE(*,*) '6. Scaled curvature kG'
           WRITE(*,*) '7. Scalar gradient G_iG_i squared'
           READ(*,'(A512)') sRes
           imappedfield_size = imappedfield_size_max
           CALL LIST_INTEGER(sRes, imappedfield_size, imappedfield_vec)
           IF ( imappedfield_vec(1) .EQ. 0 ) imappedfield_size=0

        ELSE IF ( iopt .EQ. 8 ) THEN
           WRITE(*,*) 'Threshold value ?'
           READ(*,*) threshold

        ELSE IF ( iopt .EQ. 9 ) THEN ! just space needed in point_pdf
!           imappedfield_size   = 2
           imappedfield_size   = 1
           imappedfield_vec(1) = 0
!           imappedfield_vec(2) = 0

        ENDIF

        ider_type = 0
        iperiodic = 1
        IF ( iopt .EQ. 0 ) THEN
           WRITE(*,*) ' '
!           WRITE(*,*) 'Second order (0) / compact (1) derivatives ?'
!           READ(*,*) ider_type    
           WRITE(*,*) 'Periodic algorithm (1-yes,0-no) ?'
           READ(*,*) iperiodic

        ENDIF

     ELSE
        ifield    = DINT(opt_vec(1))
        threshold = opt_vec(2)
        ith_max   = DINT(opt_vec(3))
        iopt      = DINT(opt_vec(4))
        nbins     = DINT(opt_vec(5))
! default conditions for queue submission
        ider_type = 0
        iperiodic = 1

     ENDIF

#ifdef PARALLEL
  ENDIF

  CALL MPI_BCAST(ifield,    1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  CALL MPI_BCAST(threshold, 1, MPI_REAL8,    0, MPI_COMM_WORLD, ims_err)
  CALL MPI_BCAST(ith_max,   1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
  CALL MPI_BCAST(iopt,      1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

! -------------------------------------------------------------------
! Further allocation of memory space
! inside loop, to free memory before going to DE routines
! here only parameters setup
! -------------------------------------------------------------------
  itxc_size = 0
  IF ( imappedfield_size .GT. 0 ) THEN
     DO im=1,imappedfield_size
! check if velocity field is used
        IF ( imappedfield_vec(im) .EQ. 3 .OR.&
             imappedfield_vec(im) .EQ. 4  ) iread_flow = 1
! aux data
        IF ( imappedfield_vec(im) .EQ. 1 .OR.&
             imappedfield_vec(im) .EQ. 2  ) itxc_size = MAX(itxc_size,2)
        IF ( imappedfield_vec(im) .EQ. 3 .OR.&
             imappedfield_vec(im) .EQ. 4  ) itxc_size = MAX(itxc_size,5)
        IF ( imappedfield_vec(im) .EQ. 5 .OR.&
             imappedfield_vec(im) .EQ. 6  ) itxc_size = MAX(itxc_size,4)
     ENDDO
     ALLOCATE(mappedfield(ndimx,ndimy,ndimz,imappedfield_size))
     ALLOCATE(mappedfield_name(imappedfield_size))
     ALLOCATE(mappedfield_symmetry(imappedfield_size))
  ENDIF

!  IF ( ider_type .EQ. 1 ) THEN
!     ALLOCATE(psi_x(ndimx,ndimy,ndimz),psi_y(ndimx,ndimy,ndimz),psi_z(ndimx,ndimy,ndimz))
!  ENDIF

!-------------------------------------------------------------------
! Read the grid 
! -------------------------------------------------------------------
#include "dns_read_grid.h"

! -------------------------------------------------------------------
! passing data to DE routines
! -------------------------------------------------------------------
  deltax = x(2)-x(1)
  deltay = y(2)-y(1)
  deltaz = z(2)-z(1)

! ###################################################################
! Define pointers
! ###################################################################
  u   => q(:,1)
  v   => q(:,2)
  w   => q(:,3)

! ###################################################################
! Postprocess given list of files
! ###################################################################
  DO it=1, itime_size

     itime = itime_vec(it)

! -------------------------------------------------------------------
! Allocate data used in preprocessing
! -------------------------------------------------------------------
     IF ( iread_flow .EQ. 1 ) THEN
        ALLOCATE(q(imax*jmax*kmax,3))
     ENDIF
     IF ( itxc_size .GT. 0 ) THEN
        ALLOCATE(txc(imax*jmax*kmax,itxc_size))
        ALLOCATE(wrk3d(iwrk_size))
     ENDIF

! -------------------------------------------------------------------
! Read data and define envelope surfaces
! txc1 is used to store the field used in threshold
! -------------------------------------------------------------------
     WRITE (*,*) 'Reading data and computing threshold...'
     
     IF ( ifield .EQ. 1 .AND. iopt .NE. 8 ) THEN
        WRITE(fname,*) itime; fname = TRIM(ADJUSTL(tag_scal))//TRIM(ADJUSTL(fname))
     ELSE IF ( ifield .EQ. 2 .AND. iopt .NE. 8 ) THEN
        WRITE(fname,*) itime; fname = 'chi'//TRIM(ADJUSTL(fname))
     ENDIF
     CALL DNS_READ_FIELDS(fname, i1, imax,jmax,kmax, inb_scal,inb_scal, i1, psi, dummy)

     WRITE(fname,*) itime; fname = 'sl'//TRIM(ADJUSTL(fname))
     CALL DNS_READ_FIELDS(fname, i0, imax,i1,kmax, i2,i0, i1, sl, dummy)
     
! get limiting nodes
     jmax_loc_max = 1
     jmin_loc_min = ndimy
     DO k = 1,kmax
        DO i = 1,imax
           DO j = 2,jmax-1
              IF ( (y(j-1)-sl(i,k,1))*(y(j)-sl(i,k,1)) .LE. C_0_R ) THEN
                 jmax_loc(i,k) = j-1
                 jmax_loc_max  = MAX(jmax_loc_max,jmax_loc(i,k))
              ENDIF
              IF ( (y(j-1)-sl(i,k,2))*(y(j)-sl(i,k,2)) .LE. C_0_R ) THEN
                 jmin_loc(i,k) = j
                 jmin_loc_min  = MIN(jmin_loc_min,jmin_loc(i,k))
              ENDIF
           ENDDO
           IF ( jmax_loc(i,k) .GT. jmax .OR. jmin_loc(i,k) .LT. 1 ) THEN
              WRITE(*,*) 'Threshold too small'
           ENDIF
         ENDDO
     ENDDO
     IF ( ith_max .EQ. 2 ) THEN
        DO k = 1,kmax
           DO i = 1,imax
              jmax_loc(i,k) = jmax_loc_max
              jmin_loc(i,k) = jmin_loc_min
           ENDDO
        ENDDO
     ENDIF
     
! -------------------------------------------------------------------
! Mapped fields
! -------------------------------------------------------------------
     IF ( imappedfield_size .GT. 0 ) THEN
        IF ( iread_flow .EQ. 1 ) THEN
           WRITE(fname,*) itime; fname = TRIM(ADJUSTL(tag_flow))//TRIM(ADJUSTL(fname))
           CALL DNS_READ_FIELDS(fname, i2, imax,jmax,kmax, i3,i0, iwrk_size, q, wrk3d)
        ENDIF

        DO im=1,imappedfield_size
! scalar gradient
           IF ( imappedfield_vec(im) .EQ. 1 ) THEN
              mappedfield_name(im)='Dissipation'
              mappedfield_symmetry(im)=.true.
              WRITE (*,*) 'Compute mapped field '//TRIM(ADJUSTL(mappedfield_name(im)))//' ...'
              CALL FI_GRADIENT(imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
                   dx,dy,dz, psi, mappedfield(1,1,1,im), txc(1,1),txc(1,2), wrk1d,wrk2d,wrk3d)

! log of scalar gradient
           ELSE IF ( imappedfield_vec(im) .EQ. 2 ) THEN
              mappedfield_name(im)='LogDissipation'
              mappedfield_symmetry(im)=.true.
              WRITE (*,*) 'Compute mapped field '//TRIM(ADJUSTL(mappedfield_name(im)))//' ...'
              CALL FI_GRADIENT(imode_fdm, imax,jmax,kmax, i1bc,j1bc,k1bc, &
                   dx,dy,dz, psi, mappedfield(1,1,1,im), txc(1,1),txc(1,2), wrk1d,wrk2d,wrk3d)
              DO i=1,ndimx*ndimy*ndimz
                 mappedfield(i,1,1,im) = log(mappedfield(i,1,1,im))
              ENDDO

! compressive strain rate a
           ELSE IF ( imappedfield_vec(im) .EQ. 3 ) THEN
              mappedfield_name(im)='StrainRateA'
              mappedfield_symmetry(im)=.true.
              WRITE (*,*) 'Compute mapped field '//TRIM(ADJUSTL(mappedfield_name(im)))//' ...'
              CALL FI_STRAIN_A(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, psi, u, v, w, txc(1,1), mappedfield(1,1,1,im),&
                   txc(1,2), txc(1,3), txc(1,4), txc(1,5), wrk1d, wrk2d, wrk3d)
              DO i=1,ndimx*ndimy*ndimz
                 mappedfield(i,1,1,im) =-mappedfield(i,1,1,im)
              ENDDO

! scaled compressive strain rate aG_iG_i
           ELSE IF ( imappedfield_vec(im) .EQ. 4 ) THEN
              mappedfield_name(im)='StrainRateAGiGi'
              mappedfield_symmetry(im)=.true.
              WRITE (*,*) 'Compute mapped field '//TRIM(ADJUSTL(mappedfield_name(im)))//' ...'
              CALL FI_STRAIN_A(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, psi, u, v, w, mappedfield(1,1,1,im), txc(1,1),&
                   txc(1,2), txc(1,3), txc(1,4), txc(1,5), wrk1d, wrk2d, wrk3d)
              DO i=1,ndimx*ndimy*ndimz
                 mappedfield(i,1,1,im) =-mappedfield(i,1,1,im)
              ENDDO

! curvature
           ELSE IF ( imappedfield_vec(im) .EQ. 5 ) THEN
              mappedfield_name(im)='CurvatureK'
              mappedfield_symmetry(im)=.false. ! antisymmetric
              WRITE (*,*) 'Compute mapped field '//TRIM(ADJUSTL(mappedfield_name(im)))//' ...'
              CALL FI_CURVATURE&
                   (iunifx,iunify,iunifz, imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc,&
                   dx, dy, dz, psi, mappedfield(1,1,1,im), txc(1,1), &
                   txc(1,2), txc(1,3), txc(1,4), wrk1d, wrk2d, wrk3d)

! scaled curvature
           ELSE IF ( imappedfield_vec(im) .EQ. 6 ) THEN
              mappedfield_name(im)='CurvatureKG'
              mappedfield_symmetry(im)=.false. ! antisymmetric
              WRITE (*,*) 'Compute mapped field '//TRIM(ADJUSTL(mappedfield_name(im)))//' ...'
              CALL FI_CURVATURE&
                   (iunifx,iunify,iunifz, imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc,&
                   dx, dy, dz, psi, txc(1,1), mappedfield(1,1,1,im),&
                   txc(1,2), txc(1,3), txc(1,4), wrk1d, wrk2d, wrk3d)

! scalar gradient squared
           ELSE IF ( imappedfield_vec(im) .EQ. 7 ) THEN
              mappedfield_name(im)='SquaredDissipation'
              mappedfield_symmetry(im)=.true.
              WRITE (*,*) 'Compute mapped field '//TRIM(ADJUSTL(mappedfield_name(im)))//' ...'
              CALL FI_GRADIENT(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
                   dx, dy, dz, psi, mappedfield(1,1,1,im), txc(1,1), txc(1,2), &
                   wrk1d, wrk2d, wrk3d)
              DO i=1,ndimx*ndimy*ndimz
                 mappedfield(i,1,1,im) = mappedfield(i,1,1,im)*mappedfield(i,1,1,im)
              ENDDO

           ENDIF

        ENDDO
     ENDIF

! -------------------------------------------------------------------
! passing data to DE routines
! -------------------------------------------------------------------
     itime_loc = itime
     rtime_loc = rtime
     WRITE(sRes,*) itime; filename = 'co'//TRIM(ADJUSTL(sRes))
     dEleSets_noDEles = ielem_size
     dEleSets_noMapField = imappedfield_size

!     IF ( ider_type .EQ. 1 ) THEN
!        WRITE (*,*) 'Computing derivatives...'
!        CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
!             dx, psi, psi_x, i0, i0, wrk1d, wrk2d, wrk3d)
!        CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
!             dy, psi, psi_y, i0, i0, wrk1d, wrk2d, wrk3d)
!        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
!             dz, psi, psi_z, i0, i0, wrk1d, wrk2d, wrk3d)
!     ENDIF

! -------------------------------------------------------------------
! Deallocate data used in preprocessing
! -------------------------------------------------------------------
     IF ( iread_flow .EQ. 1 ) THEN
        DEALLOCATE(u,v,w)
     ENDIF
     IF ( itxc_size .GT. 0 ) THEN
        DEALLOCATE(txc,wrk3d)
     ENDIF

! -------------------------------------------------------------------
! Call DE routines
! -------------------------------------------------------------------
     CALL initGlobalTraj()

     WRITE(*,*) 'Maximum number of elements set to ', huge(numbering(1,1,1,1))

! default
     IF ( iperiodic .EQ. 1 ) THEN
        subdomain(1,1) = 1
        subdomain(1,2) = imax
        subdomain(2,1) = 1
        subdomain(2,2) = jmax
        subdomain(3,1) = 1
        subdomain(3,2) = kmax
     ELSE
        subdomain(1,1) = 2
        subdomain(1,2) = imax-1
        subdomain(2,1) = 2
        subdomain(2,2) = jmax-1
        subdomain(3,1) = 2
        subdomain(3,2) = kmax-1
     ENDIF

! -------------------------------------------------------------------
! Connectivity information
! -------------------------------------------------------------------
     IF ( iopt .EQ. 0 ) THEN
        WRITE (*,*) 'Computing connectivity information...'
        IF ( iperiodic .EQ. 1 ) THEN
           CALL findTrajPeriodic()
        ELSE
           CALL findTraj()
        ENDIF
        CALL saveConnectivity()

! -------------------------------------------------------------------
! element pdfs of internal elements
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 1 ) THEN
        WRITE (*,*) 'Load connectivity...'
        CALL loadConnectivity()
        WRITE (*,*) 'Computing PDF internal elements...'
        CALL DE_Internal_Pdf( nbins, numbering , points )
        CALL extract_degenerate( numbering )

! -------------------------------------------------------------------
! element pdfs of external elements
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 2 ) THEN
        WRITE (*,*) 'Load connectivity...'
        CALL loadConnectivity()
        WRITE (*,*) 'Computing PDF external elements...'
        CALL DE_External_Pdf( nbins, numbering, points )
        CALL extract_degenerate( numbering )

! -------------------------------------------------------------------
! internal elements info
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 3 ) THEN
        WRITE (*,*) 'Load connectivity...'
        CALL loadConnectivity()
        CALL saveDEleInfo( numends, signs, points, numbering, 1 )

! -------------------------------------------------------------------
! external elements info
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 4 ) THEN
        WRITE (*,*) 'Load connectivity...'
        CALL loadConnectivity()
        CALL saveDEleInfo( numends, signs, points, numbering, -1 )

! -------------------------------------------------------------------
! trajectories visualization
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 5 ) THEN
        dEleSets_ipCondField = 4
        CALL loadDeleInfo(ielem_vec)
        CALL initDEle()
        itraj=.true.
        isavetraj=.true.
        do ie=1,dEleSets_noDEles
           write(*,*) 'Computing element ',ie
           itraj_stop=.false.
           subdomain(1,1) = subdomainX(ie,1)
           subdomain(1,2) = subdomainX(ie,2)
           subdomain(2,1) = subdomainY(ie,1)
           subdomain(2,2) = subdomainY(ie,2)
           subdomain(3,1) = subdomainZ(ie,1)
           subdomain(3,2) = subdomainZ(ie,2)
           CALL findTraj()
        ENDDO
        CALL saveDEleTraj(3)
        CALL finishDEle()

! -------------------------------------------------------------------
! extract critical points
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 6 ) THEN
        CALL loadDeleInfo(ielem_vec)
        CALL saveDElePts()

! -------------------------------------------------------------------
! trajectories PDFs
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 7 ) THEN
        dEleSets_ipCondField = 1 ! (x,y,z) trajectory geometry is not saved
        CALL loadDeleInfo(ielem_vec)
        CALL initDEle()
        itraj=.true.
        isavetraj=.true.
        do ie=1,dEleSets_noDEles
           write(*,*) 'Computing element ',ie
           itraj_stop=.false.
           subdomain(1,1) = subdomainX(ie,1)
           subdomain(1,2) = subdomainX(ie,2)
           subdomain(2,1) = subdomainY(ie,1)
           subdomain(2,2) = subdomainY(ie,2)
           subdomain(3,1) = subdomainZ(ie,1)
           subdomain(3,2) = subdomainZ(ie,2)
           CALL findTraj()
        ENDDO
        CALL pdfsDEleTraj(nz, nbins)
        CALL finishDEle()

! -------------------------------------------------------------------
! partition
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 8 ) THEN
! read the scalar dissipation rate into array psi
        WRITE(fname,*) itime; fname = 'chi'//TRIM(ADJUSTL(fname))
        CALL DNS_READ_FIELDS(fname, i1, imax,jmax,kmax, i1,i1, i1, psi, dummy)

        WRITE (*,*) 'Load connectivity...'
        CALL loadConnectivity()
        WRITE (*,*) 'Computing partition...'
        CALL makePartition( ifield, numbering, threshold, psi )

! -------------------------------------------------------------------
! trajectory information for point PDFs
! -------------------------------------------------------------------
     ELSE IF ( iopt .EQ. 9 ) THEN
        WRITE (*,*) 'Load connectivity...'
        CALL loadConnectivity()
        WRITE (*,*) 'Extracting information ...'
        CALL point_pdf( numbering, points )

     ENDIF

! deallocate
     CALL endGlobalTraj()
     IF ( imappedfield_size .GT. 0 ) THEN
        DEALLOCATE(mappedfield)
     ENDIF

  ENDDO

  CALL DNS_END(0)

  STOP
END PROGRAM DELEMENTS
