module mod_data
  
  implicit none
  save

  integer,          dimension(3,2)            :: subdomain
  integer                                     :: ndimx, ndimy, ndimz, numends, itime_loc
  integer                                     :: jmax_loc_max, jmin_loc_min
  integer,          dimension(:,:),   pointer :: jmax_loc, jmin_loc
  double precision                            :: deltax, deltay, deltaz, rtime_loc
  double precision, dimension(:,:,:), allocatable :: psi, psi_x, psi_y, psi_z
  logical                                     :: itraj=.false., itraj_stop=.false.
  integer                                     :: ider_type
  character*32                                :: filename

  integer, parameter :: type_numbering=4

  ! extraction of trajectories and sampling along them
  logical                                         :: isavetraj             = .false.
  integer                                         :: dEleSets_noDEles      = 0
  integer                                         :: dEleSets_maxNoDETraj  = 0
  integer                                         :: dEleSets_fixNoDETraj  = 0
  integer                                         :: dEleSets_noPtsPerTraj = 0
  integer                                         :: dEleSets_noMapField   = 0
  integer                                         :: dEleSets_ipCondField  = 4
  double precision, dimension(:,:,:), allocatable :: dEleMinMax
  integer,          dimension(:),     allocatable :: dEleNoTraj
  double precision, dimension(:,:,:,:), pointer   :: mappedfield
  logical,          dimension(:),       pointer   :: mappedfield_symmetry
  character*32    , dimension(:),       pointer   :: mappedfield_name
  
contains

  !########################################################################
  function linear_interpolate(dx,dy,dz, px,py,pz, psi)

    double precision, intent(in)  :: dx,dy,dz,psi(:,:,:)
    integer,          intent(in)  :: px,py,pz
    double precision              :: linear_interpolate
 
    integer :: px_p, py_p, pz_p

    px_p = px+1; if (px_p.gt.ndimx) px_p=1
    py_p = py+1
    pz_p = pz+1; if (pz_p.gt.ndimz) pz_p=1

    linear_interpolate = (1.d0-dx) *(1.d0-dy) *(1.d0-dz) *psi(px,  py,  pz)      &
                       + dx        *(1.d0-dy) *(1.d0-dz) *psi(px_p,py,  pz)      &
                       + (1.d0-dx) *dy        *(1.d0-dz) *psi(px,  py_p,pz)      &
                       + (1.d0-dx) *(1.d0-dy) *dz        *psi(px,  py,  pz_p)    &
                       + (1.d0-dx) *dy        *dz        *psi(px,  py_p,pz_p)    &
                       + dx        *(1.d0-dy) *dz        *psi(px_p,py,  pz_p)    &
                       + dx        *dy        *(1.d0-dz) *psi(px_p,py_p,pz)      &
                       + dx        *dy        *dz        *psi(px_p,py_p,pz_p)

  end function linear_interpolate
  
end module mod_data
