module mod_pdf

    implicit none
    save

contains

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/10 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Detect points that cross the shear layer w/o max/min
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine extract_degenerate( numbering )
    use mod_data

    integer(type_numbering), dimension(ndimx,ndimy,ndimz,2), intent(in) :: numbering

    integer :: i,j,k, count
    character(len=128):: str
    character(len=32) :: ofile='delements.log'

    count = 0
    do i=subdomain(1,1),subdomain(1,2)
       do k=subdomain(3,1),subdomain(3,2)
          do j=subdomain(2,1),subdomain(2,2)
             
             ! Shear layer cropping
             if( (j.lt.jmin_loc(i,k)).or.(j.gt.jmax_loc(i,k)) ) goto 35

             if( numbering(i,j,k,1).lt.0 .and. numbering(i,j,k,2).lt.0 ) then
                count = count + 1
             endif
             
35        enddo
       enddo
    enddo
    WRITE(str,*) count; str = 'degenarate points ......: '//TRIM(ADJUSTL(str)); CALL IO_WRITE_ASCII(ofile,str)

    return
  end subroutine extract_degenerate

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/10 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Construct elements from connectivity array numbering
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine extract_pairs_pdf( numbering, numpairs,numpairsmax,pairing, control_flag )
    use mod_data

    integer(type_numbering), dimension(ndimx,ndimy,ndimz,2), intent(in) :: numbering
    integer,                                 intent(inout) :: numpairs
    integer,                                 intent(in)    :: numpairsmax
    integer,                                 intent(in)    :: control_flag
    integer,                 dimension(3,*), intent(inout) :: pairing

    !------------------------------------------------------------------------
    integer :: localMaxId,localMinId
    integer :: i,j,k,k0

    character(len=128):: str
    character(len=32) :: ofile='delements.log'

    !########################################################################
    numpairs=0
    do i=subdomain(1,1),subdomain(1,2)
       WRITE(str,*) i; str='Iteration .....: '; CALL IO_WRITE_ASCII(ofile,str)

       do k=subdomain(3,1),subdomain(3,2)
          do j=subdomain(2,1),subdomain(2,2)
             
             ! Shear layer cropping
             if( (j.lt.jmin_loc(i,k)).or.(j.gt.jmax_loc(i,k)) ) goto 35

             ! control flag = 1 for internal elements
             ! control flag =-1 for external elements (superlayer)
             if(        ( numbering(i,j,k,1).gt.0 .or. numbering(i,j,k,2).gt.0 ) &
                  .and. ( numbering(i,j,k,1)*numbering(i,j,k,2)*control_flag .gt. 0 ) ) then

                localMaxId=numbering(i,j,k,1)
                localMinId=numbering(i,j,k,2)

                if(numpairs.eq.0) then
                   numpairs=1
                   pairing(1,1)=localMaxId
                   pairing(2,1)=localMinId
                   pairing(3,1)=1
                   goto 35
                else
                   do k0=1,numpairs
                      if(localMaxId.eq.pairing(1,k0).and.localMinId.eq.pairing(2,k0)) then
                         pairing(3,k0)=pairing(3,k0)+1
                         go to 35
                      endif
                   enddo
                endif

                numpairs=numpairs+1
                if (numpairs.GT.numpairsmax) THEN
                   WRITE(*,*) 'Increase numpairsmax',numpairsmax
                ENDIF
                pairing(1,numpairs)=localMaxId
                pairing(2,numpairs)=localMinId
                pairing(3,numpairs)=1

             endif

35        enddo
       enddo
    enddo

    return
  end subroutine extract_pairs_pdf

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/10 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Construct PDF based on internal elements
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
   subroutine de_internal_pdf( nbins, numbering , points )
    use mod_data

    integer,                                                 intent(in) :: nbins
    double precision,        dimension(3,numends),           intent(in) :: points
    integer(type_numbering), dimension(ndimx,ndimy,ndimz,2), intent(in) :: numbering

    !------------------------------------------------------------------------
    integer          :: localMaxId,localMinId
    integer          :: numpairsmax, numpairs
    integer          :: k0,px,py,pz,numpoints
    integer          :: alloc_err=0
    double precision :: zmax,zmin,dx,dy,dz
    double precision :: pdf(2*nbins),wrk1d(nbins,5) !,wrk2d(nbins*nbins,4)
!    parameter(numpairsmax=1000000)
    
    character(len=128):: str
    character(len=32) :: fname, varname(3), ofile='delements.log'

    integer,          dimension(:,:), pointer :: pairing
    double precision, dimension(:,:), pointer :: sample

    numpairsmax = numends ! number of elements less than number of extrema

    !########################################################################
    allocate(pairing(3,numpairsmax),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *pairing*. ERROR', alloc_err
    pairing(:,:)=0

    ! get elements. Control flag = 1 to extract internal elements
    call extract_pairs_pdf( numbering, numpairs,numpairsmax,pairing ,1 )
    WRITE(str,*) numpairs; str = 'internal numpairs ......: '//TRIM(ADJUSTL(str)); CALL IO_WRITE_ASCII(ofile,str)

    !########################################################################
    ! sample data
    !########################################################################
    allocate(sample(numpairs,3),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory. ERROR', alloc_err
    sample(:,:)=0.

    numpoints=0
    do k0=1,numpairs

       localMaxId=pairing(1,k0)
       localMinId=pairing(2,k0)

       px=points(1,localMaxId)
       py=points(2,localMaxId)
       pz=points(3,localMaxId)
       dx=points(1,localMaxId)-px
       dy=points(2,localMaxId)-py
       dz=points(3,localMaxId)-pz
       zmax = linear_interpolate(dx,dy,dz, px,py,pz, psi)

       px=points(1,localMinId)
       py=points(2,localMinId)
       pz=points(3,localMinId)
       dx=points(1,localMinId)-px
       dy=points(2,localMinId)-py
       dz=points(3,localMinId)-pz
       zmin = linear_interpolate(dx,dy,dz, px,py,pz, psi)

       sample(k0,2) = zmax-zmin

       dx=(points(1,localMaxId)-points(1,localMinId))*deltax
       dy=(points(2,localMaxId)-points(2,localMinId))*deltay
       dz=(points(3,localMaxId)-points(3,localMinId))*deltaz
       sample(k0,1) = dsqrt(dx**2+dy**2+dz**2)

       sample(k0,3) = log(sample(k0,1))

       numpoints = numpoints + pairing(3,k0)

    enddo

    WRITE(str,*) numpoints; str = 'internal numpoints .....: '//TRIM(ADJUSTL(str)); CALL IO_WRITE_ASCII(ofile,str)

    !########################################################################
    ! construct pdfs
    !########################################################################
    varname(1) = 'length'
    varname(2) = 'scalar'
    varname(3) = 'log(length)'

    fname='pdfDe '
    CALL PDF3D_N(fname, varname, 1, 0, itime_loc, rtime_loc, numpairs, &
         1, 1, 3, nbins, sample, pdf, wrk1d)

!    fname='jpdfDe '
!    CALL JPDF3D(fname, 0, 0, 0, itime_loc, numpairs, 1, 1, 0, 0, sample(1,2), &
!         sample(1,2), sample(1,1), nbins, nbins, wrk2d(1,1), wrk2d(1,2), wrk2d(1,3), wrk1d)

    deallocate(pairing)
    deallocate(sample)

    return
  end subroutine de_internal_pdf

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/10 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Construct PDF based on external elements
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine de_external_pdf( nbins, numbering, points )
    use mod_data

    integer,                                                 intent(in) :: nbins
    double precision,        dimension(3,numends),           intent(in) :: points
    integer(type_numbering), dimension(ndimx,ndimy,ndimz,2), intent(in) :: numbering

    !------------------------------------------------------------------------
    integer          :: k0,px,py,pz,numpoints
    integer          :: localMaxId,localMinId
    integer          :: numpairsmax, numpairs
    integer          :: alloc_err=0
    double precision :: zmax,zmin,dx,dy,dz
    double precision :: pdf(2*nbins),wrk1d(nbins,5)
!    parameter(numpairsmax=1000000)

    integer,          dimension(:,:), pointer :: pairing
    double precision, dimension(:),   pointer :: sample

    character(len=128):: str
    character(len=32) :: fname, varname(3), ofile='delements.log'

    numpairsmax = numends ! number of elements less than number of extrema

    !########################################################################
    allocate(pairing(3,numpairsmax),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory. ERROR', alloc_err
    pairing(:,:)=0

    !########################################################################
    ! get elements. Control flag =-1 to extract external elements
    ! look for elements with maximum inside turbulent zone, minimum outside
    ! look for elements with minimum inside turbulent zone, maximum outside
    call extract_pairs_pdf( numbering, numpairs,numpairsmax,pairing ,-1 )
    WRITE(str,*) numpairs; str = 'superlayer numpairs ....: '//TRIM(ADJUSTL(str)); CALL IO_WRITE_ASCII(ofile,str)

    !########################################################################
    ! sample data
    !########################################################################
    allocate(sample(numpairs),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory. ERROR', alloc_err
    sample(:)=0.

    numpoints=0
    do k0=1,numpairs

       localMaxId=pairing(1,k0)
       localMinId=pairing(2,k0)

       IF ( localMaxId .GT. 0 ) THEN
          px=points(1,localMaxId)
          py=points(2,localMaxId)
          pz=points(3,localMaxId)
          dx=points(1,localMaxId)-px
          dy=points(2,localMaxId)-py
          dz=points(3,localMaxId)-pz
          zmax = linear_interpolate(dx,dy,dz, px,py,pz, psi)

          sample(k0) = zmax
       ENDIF

       IF ( localMinId .GT. 0 ) THEN
          px=points(1,localMinId)
          py=points(2,localMinId)
          pz=points(3,localMinId)
          dx=points(1,localMinId)-px
          dy=points(2,localMinId)-py
          dz=points(3,localMinId)-pz
          zmin = linear_interpolate(dx,dy,dz, px,py,pz, psi)

          sample(k0) = 1.-zmin
       ENDIF

       numpoints=numpoints+pairing(3,k0)
    enddo

    WRITE(str,*) numpoints; str = 'superlayer numpoints .....: '//TRIM(ADJUSTL(str)); CALL IO_WRITE_ASCII(ofile,str)

    !########################################################################
    ! construct pdfs
    !########################################################################
    varname(1) = 'scalar'

    fname='pdfDe '
    CALL PDF3D_N(fname, varname, 1, 0, itime_loc, rtime_loc, numpairs, &
         1, 1, 1, nbins, sample, pdf, wrk1d)

    deallocate(pairing)
    deallocate(sample)

    return
  end subroutine de_external_pdf

  !########################################################################
  !# HISTORY
  !#
  !# 2007/12/10 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Construct statistical data based on points:
  !# 1. Extrema scalar difference
  !# 2. Trajectory scalar mean
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine point_pdf( numbering, points )
    use mod_data

    integer(type_numbering), dimension(ndimx,ndimy,ndimz,2), intent(in) :: numbering
    double precision,        dimension(3,numends),           intent(in) :: points

    !------------------------------------------------------------------------
    integer           :: extId, i, j, k, px, py, pz, minId, maxId
    double precision  :: dx, dy, dz, maxMinValue(2), dummy
    character(len=32) :: fname

    !########################################################################
    ! Sample
    ! mapped fields are completed with scalar difference and scalar mean, 
    ! and at the end is added the conditioning field 
    !########################################################################
    do k=subdomain(3,1),subdomain(3,2)
       do j=subdomain(2,1),subdomain(2,2)
          do i=subdomain(1,1),subdomain(1,2)
             
             maxId=numbering(i,j,k,1)
             minId=numbering(i,j,k,2)

             do extId = 1,2
                if ( numbering(i,j,k,extId) .gt. 0 ) then 
                   px=points(1,numbering(i,j,k,extId))
                   py=points(2,numbering(i,j,k,extId))
                   pz=points(3,numbering(i,j,k,extId))
                   dx=points(1,numbering(i,j,k,extId))-px
                   dy=points(2,numbering(i,j,k,extId))-py
                   dz=points(3,numbering(i,j,k,extId))-pz
                   maxMinValue(extId) = linear_interpolate(dx,dy,dz, px,py,pz, psi)
                endif
             enddo
             
             if ( maxId .gt. 0 .and. minId .gt. 0 ) then      ! internal elements
                mappedfield(i,j,k,1) = maxMinValue(1)-maxMinValue(2)
!                mappedfield(i,j,k,2) = 0.5*(maxMinValue(1)+maxMinValue(2))
             else if ( maxId .gt. 0 .and. minId .lt. 0 ) then ! external elements (upper)
                mappedfield(i,j,k,1) = maxMinValue(1)
!                mappedfield(i,j,k,2) = 0.5*maxMinValue(1)
             else if ( maxId .lt. 0 .and. minId .gt. 0 ) then ! external elements (lower)
                mappedfield(i,j,k,1) = 1.-maxMinValue(2)
!                mappedfield(i,j,k,2) = 0.5*(1.-maxMinValue(2))
             else if ( maxId .lt. 0 .and. minId .lt. 0 ) then ! intermediate boundary
                mappedfield(i,j,k,1) = 1.
!                mappedfield(i,j,k,2) = 0.5
             endif

          enddo
       enddo
    enddo

    !########################################################################
    ! save data
    !########################################################################
    WRITE(fname,*) itime_loc; fname = 'de'//TRIM(ADJUSTL(fname))
!    call dns_write_fields(fname, ndimx, ndimy, ndimz, ndimz, 1, 0, 2, mappedfield, dummy)
    call dns_write_fields(fname, 0, ndimx,ndimy,ndimz, 1, 1, mappedfield, dummy)

    return

  end subroutine point_pdf

  !########################################################################
  !# HISTORY
  !#
  !# 2008/01/18 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Make partition of the field:
  !#    Level 1 .....: Upper laminar zone
  !#    Level 2 .....: Upper turbulent interface
  !#    Level 3 .....: Turbulent zone
  !#    Level 4 .....: Lower turbulent interface
  !#    Level 5 .....: Lower laminar zone
  !#    Level 6 .....: Quasi-laminar diffusion layers
  !#    Level 7 .....: Otherwise
  !#
  !# It is assumed that mappedfield1 contains the scalar psi gradient
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !# type    In    Type of detection scheme:
  !#               1 Scalar, field varying between 0 at the top and 1 at the bottom
  !#               2 Dissipation, field 0 at both top and bottom
  !#
  !########################################################################
  subroutine makePartition( type, numbering, threshold, chi )
    use mod_data

    integer,                                                 intent(in) :: type
    integer(type_numbering), dimension(ndimx,ndimy,ndimz,2), intent(in) :: numbering
    double precision,                                        intent(in) :: threshold
    double precision,        dimension(ndimx,ndimy,ndimz),   intent(in) :: chi
    
    !------------------------------------------------------------------------
    integer(1),       dimension(:,:,:), allocatable, save :: gate
    character(len=32)                                     :: fname
    integer                                               :: iwrk_size, i,j,k, nlevels
    integer                                               :: alloc_err=0

    !########################################################################
    allocate(gate(ndimx,ndimy,ndimz),STAT=alloc_err)
    if( (alloc_err.NE.0) ) goto 500
    gate(:,:,:)=0

    !########################################################################
    !------------------------------------------------------------------------
    ! scalar type
    !------------------------------------------------------------------------
    if ( type .eq. 1 ) then
       do k = 1,ndimz
          do j = 1,ndimy
             do i = 1,ndimx
                if ( j .gt. jmax_loc(i,k) ) then
                   gate(i,j,k) = 1 ! upper laminar zone
                else if ( j .lt. jmin_loc(i,k) ) then
                   gate(i,j,k) = 5 ! lower laminar zone
                else
                   if ( numbering(i,j,k,1) .lt. 0 .and. numbering(i,j,k,2) .gt. 0 ) then
                      if ( chi(i,j,k) .gt. threshold ) then
                         gate(i,j,k) = 4 ! lower turbulent interface
                      else
                         gate(i,j,k) = 5 ! lower laminar zone
                      endif
                   else if ( numbering(i,j,k,1) .gt. 0 .and. numbering(i,j,k,2) .gt. 0 ) then
                      gate(i,j,k) = 3 ! turbulent zone
                   else if ( numbering(i,j,k,1) .gt. 0 .and. numbering(i,j,k,2) .lt. 0 ) then
                      if ( chi(i,j,k) .gt. threshold ) then
                         gate(i,j,k) = 2 ! upper turbulent interface
                      else
                         gate(i,j,k) = 1 ! upper laminar zone
                      endif
                   else if ( numbering(i,j,k,1) .lt. 0 .and. numbering(i,j,k,2) .lt. 0 ) then
                      if ( chi(i,j,k) .gt. threshold ) then
                         gate(i,j,k) = 6 ! quasi-laminar diffusion layers
                      else
                         if ( psi(i,j,k) .lt. 0.5 ) then
                            gate(i,j,k) = 1 ! upper laminar zone
                         else
                            gate(i,j,k) = 5 ! lower laminar zone
                         endif
                      endif
                   else ! this last one is due to the case of non-periodic boundary conditions
                      gate(i,j,k) = 7 
                   endif
                endif
             enddo
          enddo
       enddo

! using chi as aux array
       nlevels = 7
       iwrk_size = ndimx*ndimy*ndimz
       WRITE(fname,*) itime_loc; fname = 'par'//TRIM(ADJUSTL(fname))
       call DNS_WRITE_GATE(fname, ndimx,ndimy,ndimz,ndimz, nlevels, iwrk_size, gate, chi)
       
    !------------------------------------------------------------------------
    ! dissipation type
    !------------------------------------------------------------------------
    else
       do k = 1,ndimz
          do j = 1,ndimy
             do i = 1,ndimx
                if ( j .gt. jmax_loc(i,k) ) then
                   gate(i,j,k) = 1 ! upper laminar zone
                else if ( j .lt. jmin_loc(i,k) ) then
                   gate(i,j,k) = 5 ! lower laminar zone
                else
                   if ( numbering(i,j,k,2) .eq. -2 ) then
                      if ( mappedfield(i,j,k,1) .gt. threshold ) then
                         gate(i,j,k) = 4 ! lower turbulent interface
                      else
                         gate(i,j,k) = 5 ! lower laminar zone
                      endif
                   else if ( numbering(i,j,k,1) .gt. 0 .and. numbering(i,j,k,2) .gt. 0 ) then
                      gate(i,j,k) = 3 ! turbulent zone
                   else if ( numbering(i,j,k,2) .eq. -1 ) then
                      if ( mappedfield(i,j,k,1) .gt. threshold ) then
                         gate(i,j,k) = 2 ! upper turbulent interface
                      else
                         gate(i,j,k) = 1 ! upper laminar zone
                      endif
                   else ! this last one is due to the case of non-periodic boundary conditions
                      gate(i,j,k) = 6
                   endif
                endif
             enddo
          enddo
       enddo

       nlevels = 6
       iwrk_size = ndimx*ndimy*ndimz
       WRITE(fname,*) itime_loc; fname = 'parG2'//TRIM(ADJUSTL(fname))
       call DNS_WRITE_GATE(fname, ndimx,ndimy,ndimz,ndimz, nlevels, iwrk_size, gate, mappedfield(1,1,1,1))
       
    endif

    deallocate(gate)
    return

500 write(*,*) 'Not enough memory for *gate*', alloc_err
    return

  end subroutine makePartition

end module mod_pdf
